//////////////////////////////////////////////////////////////////////////
// BonSDK .NET binding interface library
// Copyright (c) http://2sen.dip.jp/dtv/
//
// This is part of BonSDK.
// BonSDK comes with ABSOLUTELY NO WARRANTY;
// for details see 'GNU GENERAL PUBLIC LICENSE 2.0'.

using System;
using System.Collections;
using System.Collections.Generic;
using System.IO;
using System.Runtime.InteropServices;
using System.Runtime.InteropServices.ComTypes;
using System.Text;

namespace Bon.IEPG
{
	/// <summary>
	/// iEPGf[^̃p[T[NXłB
	/// </summary>
	[Serializable]
	[ComVisible(true)]
	[Guid("CD930B38-C103-4e9f-AA9A-D0992DF6BB1F")]
	[ComDefaultInterface(typeof(IIEPGParser))]
	public sealed class IEPGParser : IIEPGParser
	{
		private Encoding encoding_ = Encoding.GetEncoding("shift_jis");
		private readonly Dictionary<string, string> stations_ = new Dictionary<string, string>();
		private IIEPGItem[] items_ = new IIEPGItem[0];

		/// <summary>
		/// RXgN^łB
		/// </summary>
		public IEPGParser()
		{
		}

		/// <summary>
		/// ftHg̃GR[fBO擾Eݒ肵܂B
		/// </summary>
		/// <remarks>ftHg̓VtgJISłB
		/// Xg[ɑ݂Content-Type𖳎Ă邽߁ÃvpeBŃGR[fBOw肷Kvꍇ܂B</remarks>
		public Encoding DefaultEncoding
		{
			get
			{
				return encoding_;
			}
			set
			{
				encoding_ = value;
			}
		}

		/// <summary>
		/// p[Xς̃ACe擾܂B
		/// </summary>
		public int Count
		{
			get
			{
				return items_.Length;
			}
		}

		/// <summary>
		/// p[Xς̃ACe擾܂B
		/// </summary>
		/// <param name="index">CfbNX</param>
		/// <returns>ACe</returns>
		public IIEPGItem this[int index]
		{
			get
			{
				return items_[index];
			}
		}

		/// <summary>
		/// Xe[Vϊ鎫ǉ܂B
		/// </summary>
		/// <param name="stationName">iEPG̃Xe[VɎw肳閼O</param>
		/// <param name="stationV2Name">iEPGo[W2̃Xe[VɎw肳u[hLXg+T[rXID</param>
		/// <remarks>iEPGo[W1̃Xe[Vɂ͌ȋK肪܂B
		/// ̂߁Â܂܂ŃT[rXIDoȂꍇ܂B
		/// ̂悤ȏꍇÃ\bhgpāAϊo^ĂKv܂B
		/// <para>Fparser.AddStation("mgjnCrW", "BSDT103");</para>
		/// ȂAo[W2̏ꍇ͏߂T[rXID肳Ă邽߁A肠܂B</remarks>
		public void AddStation(string stationName, string stationV2Name)
		{
			stations_.Add(stationName, stationV2Name);
		}

		/// <summary>
		/// COM IStreamC^[tFCXgpăp[X܂B
		/// </summary>
		/// <param name="stream">IStreamC^[tFCX̃CX^X</param>
		public void Parse(IStream stream)
		{
			Parse(new ComStreamWrapper(stream));
		}

		/// <summary>
		/// StreamNX̃CX^Xgpăp[X܂B
		/// </summary>
		/// <param name="stream">StreamNX̃CX^X</param>
		public void Parse(Stream stream)
		{
			Parse(new StreamReader(stream, encoding_));
		}

		/// <summary>
		/// TextReaderNX̃CX^Xgpăp[X܂B
		/// </summary>
		/// <param name="reader">TextReaderNX̃CX^X</param>
		/// <remarks>̃I[o[[hgpꍇADefaultEncodingvpeB͖܂B</remarks>
		public void Parse(TextReader reader)
		{
			bool tvpid = true;
			int version = 2;
			string boundary = string.Empty;
			bool header = false;
			List<IIEPGItem> items = new List<IIEPGItem>();
			Dictionary<string, string> values = new Dictionary<string, string>();
			while (true)
			{
				string line = reader.ReadLine();
				if (line == null)
				{
					throw new FormatException("Body not found.");
				}

				if (line.Length == 0)
				{
					StringBuilder body = new StringBuilder();
					bool finish = false;
					while (true)
					{
						line = reader.ReadLine();
						if (line == null)
						{
							finish = true;
							break;
						}
						if (tvpid == true)
						{
							if (line == (boundary + "--"))
							{
								finish = true;
								break;
							}
							if (line == boundary)
							{
								break;
							}
						}
						body.AppendFormat("{0}\r\n", line);
					}
					if (header == false)
					{
						items.Add(new IEPGItem(values, body.ToString(), stations_));
					}
					else
					{
						header = false;
					}
					if (finish == true)
					{
						break;
					}
					values = new Dictionary<string, string>();
					continue;
				}

				int position = line.IndexOf(':');
				if (position == -1)
				{
					throw new FormatException("Separator not found: " + line);
				}

				string name = line.Substring(0, position).Trim();
				string value = line.Substring(position + 1).Trim();
				if ((name.Length == 0) || (value.Length == 0))
				{
					throw new FormatException("Invalid format: " + line);
				}

				if (name == "version")
				{
					version = int.Parse(value);
					if ((version != 1) && (version != 2))
					{
						throw new FormatException("Not supported version: " + version);
					}
					if ((tvpid == false) && (version == 2))
					{
						throw new FormatException("Version mismatch: " + version);
					}
				}
				else if (name == "boundary")
				{
					if (tvpid == false)
					{
						throw new FormatException("Multipart descriptor not supported.");
					}
					boundary = value;
					header = true;
					continue;
				}
				else if (name == "Content-type")
				{
					if (value.IndexOf("application/x-tv-program-digital-info") == 0)
					{
						tvpid = true;
					}
					else if (value.IndexOf("application/x-tv-program-info") == 0)
					{
						tvpid = false;
					}
					else
					{
						throw new FormatException("Content type mismatch: " + value);
					}
				}

				values.Add(name, value);
			}

			items_ = items.ToArray();
		}

		/// <summary>
		/// p[Xς̃ACe̗񋓎q擾܂B
		/// </summary>
		/// <returns>񋓎q</returns>
		public IEnumerator GetEnumerator()
		{
			return items_.GetEnumerator();
		}

		private sealed class ComStreamWrapper : Stream
		{
			[Flags]
			private enum STATFLAG
			{
				DEFAULT = 0,
				NONAME	= 1,
				NOOPEN	= 2
			}

			private readonly IStream stream_;
			private long position_;

			public ComStreamWrapper(IStream stream)
			{
				stream_ = stream;
			}

			public override bool CanRead
			{
				get
				{
					return true;
				}
			}

			public override bool CanSeek
			{
				get
				{
					return true;
				}
			}

			public override bool CanWrite
			{
				get
				{
					return false;
				}
			}

			public override void Flush()
			{
			}

			public override long Length
			{
				get
				{
					System.Runtime.InteropServices.ComTypes.STATSTG stat;
					stream_.Stat(out stat, (int)(STATFLAG.NONAME | STATFLAG.NOOPEN));
					return stat.cbSize;
				}
			}

			public override long Position
			{
				get
				{
					return position_;
				}
				set
				{
					if (value > this.Length)
					{
						throw new ArgumentOutOfRangeException();
					}
					position_ = value;
				}
			}

			public override int Read(byte[] buffer, int offset, int count)
			{
				if ((offset + count) > buffer.Length)
				{
					throw new ArgumentOutOfRangeException();
				}

				IntPtr read = Marshal.AllocCoTaskMem(Marshal.SizeOf(typeof(int)));
				try
				{
					if (offset == 0)
					{
						stream_.Read(buffer, count, read);
					}
					else
					{
						byte[] temp = new byte[count];
						stream_.Read(temp, count, read);
						Array.Copy(temp, 0, buffer, offset, count);
					}
					return Marshal.ReadInt32(read);
				}
				finally
				{
					Marshal.FreeCoTaskMem(read);
				}
			}

			public override long Seek(long offset, SeekOrigin origin)
			{
				switch (origin)
				{
					case SeekOrigin.Begin:
						this.Position = offset;
						break;
					case SeekOrigin.Current:
						this.Position += offset;
						break;
					case SeekOrigin.End:
						this.Position = this.Length + offset;
						break;
				}
				return position_;
			}

			public override void SetLength(long value)
			{
				throw new InvalidOperationException("The method or operation is not implemented.");
			}

			public override void Write(byte[] buffer, int offset, int count)
			{
				throw new InvalidOperationException("The method or operation is not implemented.");
			}
		}
	}
}
