//////////////////////////////////////////////////////////////////////////
// BonSDK .NET binding interface library
// Copyright (c) http://2sen.dip.jp/dtv/
//
// This is part of BonSDK.
// BonSDK comes with ABSOLUTELY NO WARRANTY;
// for details see 'GNU GENERAL PUBLIC LICENSE 2.0'.

using System;
using System.ComponentModel;
using System.Collections.Generic;
using System.IO;
using System.Text;
using System.Runtime.InteropServices;

namespace Bon.Remoting
{
	/// <summary>
	/// vO̗\NXłB
	/// </summary>
	/// <remarks>̃NX̓VA\łB</remarks>
	[Serializable]
	[TypeConverter(typeof(ExpandableObjectConverter))]
	public sealed class RegisteredProgram : IRegisteredProgram
	{
		private static readonly char[] sanitizeChars_;

		private readonly Guid registeredID_ = Guid.NewGuid();
		private readonly DateTime registeredTime_ = DateTime.Now;
		private readonly IUrl tunerStreamUrl_;
		private readonly int tuningSpace_;
		private readonly int channel_;
		private readonly int serviceID_;
		private readonly int programID_;
		private DateTime startTime_;
		private DateTime endTime_;
		private readonly string storageID_;
		private readonly string fileName_;
		private Exception abortedException_;
		
		/// <summary>
		/// ^CvCjVCUłB
		/// </summary>
		static RegisteredProgram()
		{
			List<char> sanitizeChars = new List<char>();
			sanitizeChars.AddRange(Path.GetInvalidPathChars());
			sanitizeChars.AddRange(Path.GetInvalidFileNameChars());
			sanitizeChars.Add('{');
			sanitizeChars.Add('}');
			sanitizeChars_ = sanitizeChars.ToArray();
		}

		/// <summary>
		/// RXgN^łB
		/// </summary>
		/// <param name="tunerStreamUrl">`[i[̃Xg[URL</param>
		/// <param name="tuningSpace">`[jOXy[X</param>
		/// <param name="channel">`l</param>
		/// <param name="serviceID">T[rXID</param>
		/// <param name="programID">vOID</param>
		/// <param name="startTime">Xg[̋L^Jn</param>
		/// <param name="endTime">Xg[̋L^I</param>
		/// <param name="storageID">Xg[i[Xg[WID</param>
		/// <param name="fileName">Xg[i[t@C</param>
		/// <remarks>t@CɃpX؂蕶△܂܂Ăꍇ́AA_[XRAɒu܂B</remarks>
		public RegisteredProgram(IUrl tunerStreamUrl, int tuningSpace, int channel, int serviceID, int programID,
			DateTime startTime, DateTime endTime, string storageID, string fileName)
		{
			tunerStreamUrl_ = tunerStreamUrl;
			tuningSpace_ = tuningSpace;
			channel_ = channel;
			serviceID_ = serviceID;
			programID_ = programID;
			startTime_ = startTime;
			endTime_ = endTime;
			storageID_ = storageID;
			fileName_ = SanitizeFileName(fileName);
		}

		/// <summary>
		/// t@C댯ȕO܂B
		/// </summary>
		/// <param name="fileName">t@C</param>
		/// <returns>ύXt@C</returns>
		private static string SanitizeFileName(string fileName)
		{
			StringBuilder sb = new StringBuilder(fileName);
			int position = 0;
			while (position < sb.Length)
			{
				int index = fileName.IndexOfAny(sanitizeChars_, position);
				if (index == -1)
				{
					break;
				}
				sb[index] = '_';
				position = index + 1;
			}
			return sb.ToString();
		}

		/// <summary>
		/// \ӂɎʂIDłB
		/// </summary>
		[Category("General")]
		public Guid RegisteredID
		{
			get
			{
				return registeredID_;
			}
		}

		/// <summary>
		/// \񂳂ꂽ擾܂B
		/// </summary>
		[Category("General")]
		public DateTime RegisteredTime
		{
			get
			{
				return registeredTime_;
			}
		}

		/// <summary>
		/// Xg[i[Xg[WID擾܂B
		/// </summary>
		[Category("General")]
		public string StorageID
		{
			get
			{
				return storageID_;
			}
		}

		/// <summary>
		/// Xg[i[t@C擾܂B
		/// </summary>
		/// <remarks>̐ݒ́Aۂɕۑt@C̈ꕔƂĎg܂B</remarks>
		[Category("General")]
		public string FileName
		{
			get
			{
				return fileName_;
			}
		}

		/// <summary>
		/// A{[gԂ̗\̌擾Eݒ肵܂B
		/// </summary>
		[Category("General")]
		public Exception AbortedException
		{
			get
			{
				return abortedException_;
			}
			set
			{
				abortedException_ = value;
			}
		}

		/// <summary>
		/// `[i[̃Xg[URLłB
		/// </summary>
		[Category("Target Stream")]
		public IUrl TunerStreamUrl
		{
			get
			{
				return tunerStreamUrl_;
			}
		}

		/// <summary>
		/// `[jOXy[X擾܂B
		/// </summary>
		[Category("Target Stream")]
		public int TuningSpace
		{
			get
			{
				return tuningSpace_;
			}
		}

		/// <summary>
		/// `l擾܂B
		/// </summary>
		[Category("Target Stream")]
		public int Channel
		{
			get
			{
				return channel_;
			}
		}

		/// <summary>
		/// T[rXID擾܂B
		/// </summary>
		/// <remarks>T[rXIDƂ-1w肳ĂƁASẴXg[f[^L^܂B
		/// 0w肳ĂƁAKȎM\ȃXg[f[^IċL^܂B
		/// AAǂ̃Xg[f[^Iꂽ͕܂B</remarks>
		[Category("Target Stream")]
		public int ServiceID
		{
			get
			{
				return serviceID_;
			}
		}

		/// <summary>
		/// vOID擾܂B
		/// </summary>
		/// <remarks>vOIDƂ-1w肳ĂƁAvǑos܂B
		/// vOIDw肳Ăꍇ͈vvOoÃf[^L^܂B</remarks>
		[Category("Target Stream")]
		public int ProgramID
		{
			get
			{
				return programID_;
			}
		}

		/// <summary>
		/// Xg[̘^Jn擾Eݒ肵܂B
		/// </summary>
		[Category("Recording Time")]
		public DateTime StartTime
		{
			get
			{
				return startTime_;
			}
			set
			{
				startTime_ = value;
			}
		}

		/// <summary>
		/// Xg[̘^I擾Eݒ肵܂B
		/// </summary>
		[Category("Recording Time")]
		public DateTime EndTime
		{
			get
			{
				return endTime_;
			}
			set
			{
				endTime_ = value;
			}
		}
	}
}
