//////////////////////////////////////////////////////////////////////////
// BonSDK .NET binding interface library
// Copyright (c) http://2sen.dip.jp/dtv/
//
// This is part of BonSDK.
// BonSDK comes with ABSOLUTELY NO WARRANTY;
// for details see 'GNU GENERAL PUBLIC LICENSE 2.0'.

using System;
using System.ComponentModel;
using System.Runtime.InteropServices;

namespace Bon.Remoting
{
	/// <summary>
	/// [gQƂŃXg[̃vOEPGi[ReiNXłB
	/// </summary>
	/// <remarks>̃NX̓VA\œǂݎpłB</remarks>
	[Serializable]
	[TypeConverter(typeof(ExpandableObjectConverter))]
	public sealed class RemoteProgramInformation : IRemoteProgramInformation
	{
		private readonly int serviceID_;
		private readonly int programVideoID_;
		private readonly int programAudioID_;
		private readonly int programPcrID_;
		private readonly string serviceName_;
		private readonly EpgDataStates epgDataStatus_;
		private readonly int programID_ = -1;
		private readonly DateTime startTime_ = DateTime.MinValue;
		private readonly DateTime endTime_ = DateTime.MinValue;
		private readonly bool isScrambled_;
		private readonly string eventTitle_ = string.Empty;
		private readonly string eventInfo_ = string.Empty;
		private readonly string eventDetail_ = string.Empty;
		private readonly int videoResolution_ = -1;
		private readonly bool isVideoProgressive_;
		private readonly bool isVideoAspectWide_;
		private readonly bool isVideoPanAndScan_;
		private readonly AudioModes audioMode_;
		private readonly int genre_ = -1;

		/// <summary>
		/// RXgN^łB
		/// </summary>
		/// <param name="program">ƂȂvO</param>
		/// <remarks>Ŏw肳ꂽvORs[Ċi[܂B</remarks>
		public RemoteProgramInformation(IProgramInformation program)
		{
			serviceID_ = program.ServiceID;
			programVideoID_ = program.ProgramVideoID;
			programAudioID_ = program.ProgramAudioID;
			programPcrID_ = program.ProgramPcrID;
			serviceName_ = program.ServiceName;

			if (program.EpgData != null)
			{
				epgDataStatus_ = program.EpgData.EpgDataStatus;
				if ((epgDataStatus_ & EpgDataStates.EventIDStored) == EpgDataStates.EventIDStored)
				{
					programID_ = program.EpgData.ProgramID;
					startTime_ = program.EpgData.StartTime;
					endTime_ = program.EpgData.EndTime;
					isScrambled_ = program.EpgData.IsScrambled;
				}
				if ((epgDataStatus_ & EpgDataStates.ShortInfoStored) == EpgDataStates.ShortInfoStored)
				{
					eventTitle_ = program.EpgData.EventTitle;
					eventInfo_ = program.EpgData.EventInfo;
				}
				if ((epgDataStatus_ & EpgDataStates.ExtendInfoStored) == EpgDataStates.ExtendInfoStored)
				{
					eventDetail_ = program.EpgData.EventDetail;
				}
				if ((epgDataStatus_ & EpgDataStates.VideoInfoStored) == EpgDataStates.VideoInfoStored)
				{
					videoResolution_ = program.EpgData.VideoResolution;
					isVideoProgressive_ = program.EpgData.IsVideoProgressive;
					isVideoAspectWide_ = program.EpgData.IsVideoAspectWide;
					isVideoPanAndScan_ = program.EpgData.IsVideoPanAndScan;
				}
				if ((epgDataStatus_ & EpgDataStates.AudioInfoStored) == EpgDataStates.AudioInfoStored)
				{
					audioMode_ = program.EpgData.AudioMode;
				}
				if ((epgDataStatus_ & EpgDataStates.ContentInfoStored) == EpgDataStates.ContentInfoStored)
				{
					genre_ = program.EpgData.Genre;
				}
			}
		}

		/// <summary>
		/// T[rXID擾܂B
		/// </summary>
		[Category("Program Information")]
		[Description("Service ID.")]
		public int ServiceID
		{
			get
			{
				return serviceID_;
			}
		}

		/// <summary>
		/// f̃vOID擾܂B
		/// </summary>
		[Category("Program Information")]
		[Description("Program's video ID.")]
		public int ProgramVideoID
		{
			get
			{
				return programVideoID_;
			}
		}

		/// <summary>
		/// ̃vOID擾܂B
		/// </summary>
		[Category("Program Information")]
		[Description("Program's audio ID.")]
		public int ProgramAudioID
		{
			get
			{
				return programAudioID_;
			}
		}

		/// <summary>
		/// PCR̃vOID擾܂B
		/// </summary>
		[Category("Program Information")]
		[Description("Program's PCR ID.")]
		public int ProgramPcrID
		{
			get
			{
				return programPcrID_;
			}
		}
	
		/// <summary>
		/// T[rX擾܂B
		/// </summary>
		[Category("Program Information")]
		[Description("Program's service name.")]
		public string ServiceName
		{
			get
			{
				return serviceName_;
			}
		}

		///////////////////////////////

		/// <summary>
		/// Ɏ擾\ȃvpeBl擾܂B
		/// </summary>
		[Category("Program Information")]
		[Description("This information completion status.")]
		public EpgDataStates EpgDataStatus
		{
			get
			{
				return epgDataStatus_;
			}
		}

		///////////////////////////////
		// EventIDStored ŗL

		/// <summary>
		/// vOID擾܂B
		/// </summary>
		/// <remarks>l擾ɂ́AEpgDataStatusEventIDStoredĂKv܂B</remarks>
		[Category("Event ID")]
		[Description("Program's event ID.")]
		public int ProgramID
		{
			get
			{
				return programID_;
			}
		}

		/// <summary>
		/// Jn擾܂B
		/// </summary>
		/// <remarks>l擾ɂ́AEpgDataStatusEventIDStoredĂKv܂B</remarks>
		[Category("Event ID")]
		[Description("Program's start time.")]
		public DateTime StartTime
		{
			get
			{
				return startTime_;
			}
		}

		/// <summary>
		/// Xg[̎ԊԊu擾܂B
		/// </summary>
		/// <remarks>l擾ɂ́AEpgDataStatusEventIDStoredĂKv܂B</remarks>
		[Category("Event ID")]
		[Description("Program's end time.")]
		public DateTime EndTime
		{
			get
			{
				return endTime_;
			}
		}

		/// <summary>
		/// XNuĂ邩ǂ擾܂B
		/// </summary>
		/// <remarks>l擾ɂ́AEpgDataStatusEventIDStoredĂKv܂B</remarks>
		[Category("Event ID")]
		[Description("Is program scrambled?")]
		public bool IsScrambled
		{
			get
			{
				return isScrambled_;
			}
		}

		///////////////////////////////
		// ShortInfoStored ŗL

		/// <summary>
		/// Cxg^Cg擾܂B
		/// </summary>
		/// <remarks>l擾ɂ́AEpgDataStatusShortInfoStoredĂKv܂B</remarks>
		[Category("Short Information")]
		[Description("Program's title.")]
		public string EventTitle
		{
			get
			{
				return eventTitle_;
			}
		}

		/// <summary>
		/// Cxg擾܂B
		/// </summary>
		/// <remarks>l擾ɂ́AEpgDataStatusShortInfoStoredĂKv܂B</remarks>
		[Category("Short Information")]
		[Description("Program's information.")]
		public string EventInfo
		{
			get
			{
				return eventInfo_;
			}
		}

		///////////////////////////////
		// ExtendInfoStored ŗL

		/// <summary>
		/// Cxg̏ڍׂ擾܂B
		/// </summary>
		/// <remarks>l擾ɂ́AEpgDataStatusExtendInfoStoredĂKv܂B</remarks>
		[Category("Extend Information")]
		[Description("Program's information. (Detail)")]
		public string EventDetail
		{
			get
			{
				return eventDetail_;
			}
		}

		///////////////////////////////
		// VideoInfoStored ŗL

		/// <summary>
		/// f̑擾܂B
		/// </summary>
		/// <remarks>l擾ɂ́AEpgDataStatusVideoInfoStoredĂKv܂B</remarks>
		[Category("Video Information")]
		[Description("Program's video vertical resolution.")]
		public int VideoResolution
		{
			get
			{
				return videoResolution_;
			}
		}

		/// <summary>
		/// fvObVuǂ擾܂B
		/// </summary>
		/// <remarks>l擾ɂ́AEpgDataStatusVideoInfoStoredĂKv܂B</remarks>
		[Category("Video Information")]
		[Description("Is program video with progressive scan?")]
		public bool IsVideoProgressive
		{
			get
			{
				return isVideoProgressive_;
			}
		}

		/// <summary>
		/// fCh䗦ǂ擾܂B
		/// </summary>
		/// <remarks>l擾ɂ́AEpgDataStatusVideoInfoStoredĂKv܂B</remarks>
		[Category("Video Information")]
		[Description("Is program video with wide aspect ratio?")]
		public bool IsVideoAspectWide
		{
			get
			{
				return isVideoAspectWide_;
			}
		}

		/// <summary>
		/// fpXLĂ邩ǂ擾܂B
		/// </summary>
		/// <remarks>l擾ɂ́AEpgDataStatusVideoInfoStoredĂKv܂B</remarks>
		[Category("Video Information")]
		[Description("Is program video with pan and scan?")]
		public bool IsVideoPanAndScan
		{
			get
			{
				return isVideoPanAndScan_;
			}
		}

		///////////////////////////////
		// AudioInfoStored ŗL

		/// <summary>
		/// ̏擾܂B
		/// </summary>
		/// <remarks>l擾ɂ́AEpgDataStatusAudioInfoStoredĂKv܂B</remarks>
		[Category("Audio Information")]
		[Description("Program's audio mode.")]
		public AudioModes AudioMode
		{
			get
			{
				return audioMode_;
			}
		}

		///////////////////////////////
		// ContentInfoStored ŗL

		/// <summary>
		/// W擾܂B
		/// </summary>
		/// <remarks>l擾ɂ́AEpgDataStatusContentInfoStoredĂKv܂B</remarks>
		[Category("Content Information")]
		[Description("Program's genre code.")]
		public int Genre
		{
			get
			{
				return genre_;
			}
		}
	}
}
