//////////////////////////////////////////////////////////////////////////
// BonSDK .NET binding interop library
// Copyright (c) http://2sen.dip.jp/dtv/
//
// This is part of BonSDK.
// BonSDK comes with ABSOLUTELY NO WARRANTY;
// for details see 'GNU GENERAL PUBLIC LICENSE 2.0'.


#include "stdafx.h"

#include "ManagedLock.h"
#include "DataQueue.h"
#include "RawTsData.h"
#include "Tuner.h"

namespace Bon
{
	DataQueue::DataQueue(Tuner^ parent, int serviceID, int programID, __int64 maxSize, bool locked)
		: parent_(parent), maxSize_(maxSize), locked_(locked), totalSize_(0), serviceID_(serviceID), programID_(programID),
		queue_(gcnew System::Collections::Generic::Queue<ITsData^>()),
		queued_(gcnew System::Threading::ManualResetEvent(false))
	{
		if (serviceID_ < -1)
		{
			throw gcnew BonException("Invalid program ID: ID={0}", serviceID_);
		}
	}

	DataQueue::~DataQueue()
	{
		Internal::ManagedLock lock(this);

		if (parent_ != nullptr)
		{
			parent_->RemoveQueue(this);
			parent_ = nullptr;
		}

		Close();
	}

	void DataQueue::Close()
	{
		Internal::ManagedLock lock(queue_);

		while (queue_->Count > 0)
		{
			ITsData^ data = queue_->Dequeue();
			data->~ITsData();
		}
		queued_->Set();
		totalSize_ = 0;
	}

	void DataQueue::Clear()
	{
		Internal::ManagedLock lock(queue_);

		while (queue_->Count > 0)
		{
			ITsData^ data = queue_->Dequeue();
			data->~ITsData();
		}
		if (parent_ != nullptr)
		{
			queued_->Reset();
		}
		totalSize_ = 0;
	}

	void DataQueue::Purge()
	{
		if (locked_ == true)
		{
			throw gcnew System::InvalidOperationException("DataQueue locked.");
		}
		Clear();
	}

	System::Threading::WaitHandle^ DataQueue::CanDequeue::get()
	{
		return queued_;
	}

	System::IntPtr DataQueue::CanDequeueHandle::get()
	{
		return queued_->Handle;
	}

	int DataQueue::Count::get()
	{
		Internal::ManagedLock lock(queue_);

		return queue_->Count;
	}

	__int64 DataQueue::MaxSize::get()
	{
		return maxSize_;
	}

	__int64 DataQueue::TotalSize::get()
	{
		Internal::ManagedLock lock(queue_);

		return totalSize_;
	}

	bool DataQueue::IsLocked::get()
	{
		return locked_;
	}

	int DataQueue::ServiceID::get()
	{
		Internal::ManagedLock lock(queue_);

		return serviceID_;
	}

	void DataQueue::ServiceID::set(int serviceID)
	{
		// ÕT[rXID̕ύX͂̃vpeBōsÃt@T[ĥ悤Ȃ̂
		// ۂɂStreamSink::ChangeServiceIDŏB
		// Iɂ͂₱AO猩DataQueueɑ΂ăT[rXIDݒ肵AΉ
		// Xg[oƂт삪B
		// DataQueue::ServiceID::set --> StreamSink::ChangeServiceID --> DataQueue::SetServiceID
		if (serviceID < -1)
		{
			throw gcnew BonException("Invalid service ID: ID={0}", serviceID);
		}

		Internal::ManagedLock lock(queue_);

		if (locked_ == true)
		{
			throw gcnew System::InvalidOperationException("DataQueue locked.");
		}

		parent_->ChangeServiceID(this, serviceID);
	}

	void DataQueue::SetServiceID(int serviceID)
	{
		// ServiceID::setStreamSink::ChangeServiceIDoRĊԐړIɌĂяoB
		Internal::ManagedLock lock(queue_);

		serviceID_ = serviceID;
		while (queue_->Count > 0)
		{
			ITsData^ data = queue_->Dequeue();
			data->~ITsData();
		}
		queued_->Reset();
		totalSize_ = 0;
	}

	int DataQueue::ProgramID::get()
	{
		return programID_;
	}

	void DataQueue::ProgramID::set(int programID)
	{
		Internal::ManagedLock lock(queue_);

		if (locked_ == true)
		{
			throw gcnew System::InvalidOperationException("DataQueue locked.");
		}

		programID_ = programID;
		while (queue_->Count > 0)
		{
			ITsData^ data = queue_->Dequeue();
			data->~ITsData();
		}
		queued_->Reset();
		totalSize_ = 0;
	}

	void DataQueue::Enqueue(ITsData^ data)
	{
		Internal::ManagedLock lock(queue_);

		int size = data->Size;
		if ((totalSize_ + size) > maxSize_)
		{
			BON_TRACE(TraceType_Warning, TEXT("Overflow DataQueue's queue: size=%1!d!, maxSize=%2!d!"), totalSize_ + size, maxSize_);

#ifndef NO_CLEAR_OVERFLOW
			Clear();
#else
			while (queue_->Count > 0)
			{
				totalSize_ -= queue_->Dequeue()->Size;
				if ((totalSize_ + size) <= maxSize_)
				{
					break;
				}
			}
#endif
		}

		queue_->Enqueue(data);
		if (queue_->Count == 1)
		{
			queued_->Set();
		}
		totalSize_ += size;
	}

	ITsData^ DataQueue::Dequeue()
	{
		queued_->WaitOne();

		Internal::ManagedLock lock(queue_);

		if (queue_->Count == 0)
		{
			throw gcnew System::ObjectDisposedException("DataQueue disposed");
		}

		ITsData^ data = queue_->Dequeue();
		if (queue_->Count == 0)
		{
			queued_->Reset();
		}
		totalSize_ -= data->Size;

		return data;
	}

	IRawTsData^ DataQueue::RawDequeue()
	{
		ITsData^ data = Dequeue();
		return gcnew RawTsData(data);
	}
}
