//////////////////////////////////////////////////////////////////////////
// BonSDK .NET binding interop library
// Copyright (c) http://2sen.dip.jp/dtv/
//
// This is part of BonSDK.
// BonSDK comes with ABSOLUTELY NO WARRANTY;
// for details see 'GNU GENERAL PUBLIC LICENSE 2.0'.


#include "stdafx.h"

#include "ManagedLock.h"
#include "MediaDecoderHandlerTrampoline.h"

namespace Bon
{
	namespace Internal
	{
		MediaDecoderEventArgs::MediaDecoderEventArgs(IMediaDecoder *pDecoder, int eventID, ULONG_PTR param)
			: pDecoder_(pDecoder), eventID_(eventID), param_(param)
		{
		}

		IMediaDecoder *MediaDecoderEventArgs::MediaDecoder::get()
		{
			return pDecoder_;
		}

		int MediaDecoderEventArgs::EventID::get()
		{
			return eventID_;
		}

		ULONG_PTR MediaDecoderEventArgs::Parameter::get()
		{
			return param_;
		}

		MediaDecoderHandlerTrampoline::MediaDecoderHandlerTrampoline(IBonObject *pOwner)
			: CBonObject(pOwner), tickCount_(::GetTickCount()),
			callBack_(gcnew System::Threading::WaitCallback(WorkItemEntry)),
			queue_(gcnew System::Collections::Generic::Queue<MediaDecoderEventArgs^>())
		{
		}

		void MediaDecoderHandlerTrampoline::RegisterMediaDecoderEvent(MediaDecoderEventHandler^ handler)
		{
			ManagedLock lock(queue_);

			handler_ = static_cast<MediaDecoderEventHandler^>(System::Delegate::Combine(handler_, handler));
		}

		void MediaDecoderHandlerTrampoline::UnregisterMediaDecoderEvent(MediaDecoderEventHandler^ handler)
		{
			ManagedLock lock(queue_);

			handler_ = static_cast<MediaDecoderEventHandler^>(System::Delegate::Remove(handler_, handler));
		}

		const DWORD MediaDecoderHandlerTrampoline::OnDecoderEvent(IMediaDecoder *pDecoder, const DWORD dwEventID, const ULONG_PTR param)
		{
			// TSŃG[oĂƁAʂ̃CxgptH[}XቺB
			// ̂߁AACxgL[ɂA܂Ƃ߂ďB

			ManagedLock lock(queue_);

			MediaDecoderEventHandler^ handler = handler_;
			if (handler != nullptr)
			{
				MediaDecoderEventArgs^ e = gcnew MediaDecoderEventArgs(pDecoder, static_cast<int>(dwEventID), param);
				queue_->Enqueue(e);

				DWORD now = ::GetTickCount();
				DWORD differ = 0;
				if (now >= tickCount_)
				{
					differ = now - tickCount_;
				}
				else
				{
					differ = (0xffffffffUL - tickCount_) + now + 1;
				}

				// 0.5b
				if (differ >= 500)
				{
					System::Threading::ThreadPool::QueueUserWorkItem(callBack_,
						System::Collections::Generic::KeyValuePair<
						MediaDecoderEventHandler^, System::Collections::Generic::Queue<MediaDecoderEventArgs^>^>
						(handler, queue_));
					queue_ = gcnew System::Collections::Generic::Queue<MediaDecoderEventArgs^>();

					tickCount_ = ::GetTickCount();
				}
			}
			return 0;
		}

		void MediaDecoderHandlerTrampoline::WorkItemEntry(System::Object^ argument)
		{
			System::Threading::Thread::CurrentThread->Name = "MediaDecoderHandlerTrampoline invoker";

			System::Collections::Generic::KeyValuePair<
				MediaDecoderEventHandler^, System::Collections::Generic::Queue<MediaDecoderEventArgs^>^> pair =
				static_cast<System::Collections::Generic::KeyValuePair<
				MediaDecoderEventHandler^, System::Collections::Generic::Queue<MediaDecoderEventArgs^>^> >(argument);
			while (pair.Value->Count > 0)
			{
				MediaDecoderEventArgs^ e = pair.Value->Dequeue();
				try
				{
					pair.Key->Invoke(e);
				}
				catch (...)
				{
				}
			}
		}
	}
}
