//////////////////////////////////////////////////////////////////////////
// BonSDK .NET binding interop library
// Copyright (c) http://2sen.dip.jp/dtv/
//
// This is part of BonSDK.
// BonSDK comes with ABSOLUTELY NO WARRANTY;
// for details see 'GNU GENERAL PUBLIC LICENSE 2.0'.


#include "stdafx.h"

#include "ManagedLock.h"
#include "CreateBonInstance.h"
#include "DataQueue.h"
#include "StreamSink.h"
#include "MediaDecoderHandlerTrampoline.h"
#include "TuningSpaceInformation.h"
#include "ChannelInformation.h"
#include "ServiceInformation.h"
#include "EpgData.h"
#include "ProgramInformation.h"
#include "SelectedChannel.h"
#include "Tuner.h"

namespace Bon
{
	Tuner::Tuner(ITunerDefinition^ tunerDefinition, System::String^ casCardReader)
		: tunerDefinition_(tunerDefinition), lockCookie_(System::Guid::Empty), running_(false),
		enableEmmProcess_(false), enableLnbPower_(true),
		pTsSourceTuner_(0), pTsPacketSync_(0), pProgAnalyzer_(0), pTsDescrambler_(0), pStreamSink_(0),
		pTsPacketSyncTrampoline_(0), pTsDescramblerTrampoline_(0),
		packetSyncEvents_(gcnew System::Collections::Generic::List<IPacketSyncEvent^>()),
		descrambleEvents_(gcnew System::Collections::Generic::List<IDescrambleEvent^>())
	{
		try
		{
			pTsPacketSyncTrampoline_ = new Internal::MediaDecoderHandlerTrampoline();
			pTsDescramblerTrampoline_ = new Internal::MediaDecoderHandlerTrampoline();

			pTsSourceTuner_ = Internal::CreateBonInstance<ITsSourceTuner>(TEXT("CTsSourceTuner"));
			pTsPacketSync_ = Internal::CreateBonInstance<ITsPacketSync>(TEXT("CTsPacketSync"),
				static_cast<CBonObject *>(pTsPacketSyncTrampoline_));
			pProgAnalyzer_ = Internal::CreateBonInstance<IProgAnalyzer>(TEXT("CProgAnalyzer"));
			pTsDescrambler_ = Internal::CreateBonInstance<ITsDescrambler>(TEXT("CTsDescrambler"),
				static_cast<CBonObject *>(pTsDescramblerTrampoline_));
			pStreamSink_ = new Internal::StreamSink(pProgAnalyzer_, 512 * 1024);

			dynamic_cast<IMediaDecoder *>(pTsSourceTuner_)->SetOutputDecoder(dynamic_cast<IMediaDecoder *>(pTsPacketSync_));
			dynamic_cast<IMediaDecoder *>(pTsPacketSync_)->SetOutputDecoder(dynamic_cast<IMediaDecoder *>(pTsDescrambler_));
			dynamic_cast<IMediaDecoder *>(pTsDescrambler_)->SetOutputDecoder(dynamic_cast<IMediaDecoder *>(pProgAnalyzer_));
			dynamic_cast<IMediaDecoder *>(pProgAnalyzer_)->SetOutputDecoder(pStreamSink_);

			pin_ptr<const wchar_t> pTunerName = PtrToStringChars(tunerDefinition_->TunerName);

			int tunerResult = pTsSourceTuner_->OpenTuner(pTunerName);
			if (tunerResult > 0)
			{
				throw gcnew BonException("Cannot open tuner: Name={0}, Result={1}",
					tunerDefinition_->TunerName, tunerResult);
			}

			pin_ptr<const wchar_t> pCasCardReader = PtrToStringChars(casCardReader);

			if (pTsDescrambler_->OpenDescrambler(pCasCardReader) == false)
			{
				pTsSourceTuner_->CloseTuner();
				throw gcnew BonException("Cannot open card reader");
			}

			pTsPacketSync_->DiscardNullPacket(true);
			pTsDescrambler_->DiscardScramblePacket(false);
			pTsDescrambler_->EnableBuffering(true);

			lockCookie_ = System::Guid::NewGuid();
			System::Threading::ThreadPool::QueueUserWorkItem(
				gcnew System::Threading::WaitCallback(this, &Tuner::ScanChannelEntry));
		}
		catch (...)
		{
			this->~Tuner();
			throw;
		}
	}

	Tuner::!Tuner()
	{
		BON_SAFE_RELEASE(pTsSourceTuner_);
		BON_SAFE_RELEASE(pTsPacketSync_);
		BON_SAFE_RELEASE(pTsDescrambler_);
		BON_SAFE_RELEASE(pProgAnalyzer_);
		BON_SAFE_DELETE(pStreamSink_);
		BON_SAFE_DELETE(pTsDescramblerTrampoline_);
		BON_SAFE_DELETE(pTsPacketSyncTrampoline_);

		packetSyncEvents_->Clear();
		descrambleEvents_->Clear();
	}

	Tuner::~Tuner()
	{
		Internal::ManagedLock lock(this);

		BON_SAFE_RELEASE(pTsSourceTuner_);
		BON_SAFE_RELEASE(pTsPacketSync_);
		BON_SAFE_RELEASE(pTsDescrambler_);
		BON_SAFE_RELEASE(pProgAnalyzer_);
		BON_SAFE_DELETE(pStreamSink_);
		BON_SAFE_DELETE(pTsDescramblerTrampoline_);
		BON_SAFE_DELETE(pTsPacketSyncTrampoline_);

		packetSyncEvents_->Clear();
		descrambleEvents_->Clear();
	}

	void Tuner::OnTsPacketSyncEvent(Internal::MediaDecoderEventArgs^ e)
	{
		Internal::ManagedLock lock(packetSyncEvents_);

		switch (e->EventID)
		{
			case ITsPacketSync::EID_SYNC_ERR :
				for (int i = 0; i < packetSyncEvents_->Count; i++)
				{
					try
					{
						packetSyncEvents_[i]->OnSynchronizedError(static_cast<int>(e->Parameter));
					}
					catch (...)
					{
					}
				}
				break;
			
			case ITsPacketSync::EID_FORMAT_ERR :
				for (int i = 0; i < packetSyncEvents_->Count; i++)
				{
					try
					{
						packetSyncEvents_[i]->OnFormatError(static_cast<int>(e->Parameter));
					}
					catch (...)
					{
					}
				}
				break;

			case ITsPacketSync::EID_TRANSPORT_ERR :
				for (int i = 0; i < packetSyncEvents_->Count; i++)
				{
					try
					{
						packetSyncEvents_[i]->OnTransportError(static_cast<int>(e->Parameter));
					}
					catch (...)
					{
					}
				}
				break;

			case ITsPacketSync::EID_CONTINUITY_ERR :
				for (int i = 0; i < packetSyncEvents_->Count; i++)
				{
					try
					{
						packetSyncEvents_[i]->OnContinuityError(static_cast<int>(e->Parameter));
					}
					catch (...)
					{
					}
				}
				break;

			default:
				BON_TRACE(TraceType_Debug, TEXT("Unknown OnTsPacketSyncEvent(%1!p!, %2!d!, %3!p!)"),
					e->MediaDecoder, e->EventID, e->Parameter);
				break;
		}
	}

	void Tuner::OnTsDescramblerEvent(Internal::MediaDecoderEventArgs^ e)
	{
		Internal::ManagedLock lock(descrambleEvents_);

		switch (e->EventID)
		{
			case ITsDescrambler::EID_CANT_DESCRAMBLE :
				for (int i = 0; i < descrambleEvents_->Count; i++)
				{
					try
					{
						descrambleEvents_[i]->OnCannotDescramble(static_cast<int>(e->Parameter));
					}
					catch (...)
					{
					}
				}
				break;
			
			case ITsDescrambler::EID_ECM_PROCESSED :
				for (int i = 0; i < descrambleEvents_->Count; i++)
				{
					try
					{
						descrambleEvents_[i]->OnEcmProcessed(static_cast<DescramblingStates>(e->Parameter));
					}
					catch (...)
					{
					}
				}
				break;

			case ITsDescrambler::EID_EMM_PROCESSED :
				for (int i = 0; i < descrambleEvents_->Count; i++)
				{
					try
					{
						descrambleEvents_[i]->OnEmmProcessed(e->Parameter != 0);
					}
					catch (...)
					{
					}
				}
				break;

			default:
				BON_TRACE(TraceType_Debug, TEXT("Unknown OnTsDescramblerEvent(%1!p!, %2!d!, %3!p!)"),
					e->MediaDecoder, e->EventID, e->Parameter);
				break;
		}
	}

	void Tuner::RegisterEvent(IPacketSyncEvent^ packetSync)
	{
		Internal::ManagedLock lock(packetSyncEvents_);

		packetSyncEvents_->Add(packetSync);
		if (packetSyncEvents_->Count == 1)
		{
			pTsPacketSyncTrampoline_->RegisterMediaDecoderEvent(
				gcnew Internal::MediaDecoderEventHandler(this, &Tuner::OnTsPacketSyncEvent));
		}
	}

	void Tuner::RegisterEvent(IDescrambleEvent^ descramble)
	{
		Internal::ManagedLock lock(descrambleEvents_);

		descrambleEvents_->Add(descramble);
		if (descrambleEvents_->Count == 1)
		{
			pTsDescramblerTrampoline_->RegisterMediaDecoderEvent(
				gcnew Internal::MediaDecoderEventHandler(this, &Tuner::OnTsDescramblerEvent));
		}
	}

	void Tuner::UnregisterEvent(IPacketSyncEvent^ packetSync)
	{
		Internal::ManagedLock lock(packetSyncEvents_);

		packetSyncEvents_->Remove(packetSync);
		if (packetSyncEvents_->Count == 0)
		{
			pTsPacketSyncTrampoline_->UnregisterMediaDecoderEvent(
				gcnew Internal::MediaDecoderEventHandler(this, &Tuner::OnTsPacketSyncEvent));
		}
	}

	void Tuner::UnregisterEvent(IDescrambleEvent^ descramble)
	{
		Internal::ManagedLock lock(descrambleEvents_);

		descrambleEvents_->Remove(descramble);
		if (descrambleEvents_->Count == 0)
		{
			pTsDescramblerTrampoline_->UnregisterMediaDecoderEvent(
				gcnew Internal::MediaDecoderEventHandler(this, &Tuner::OnTsDescramblerEvent));
		}
	}

	System::String^ Tuner::TunerName::get()
	{
		return tunerDefinition_->TunerName;
	}

	System::String^ Tuner::DeviceName::get()
	{
		return tunerDefinition_->DeviceName;
	}

	array<ITuningSpaceInformation^>^ Tuner::TuningSpaces::get()
	{
		Internal::ManagedLock lock(this);

		array<ITuningSpaceInformation^>^ tuningSpaces = gcnew array<ITuningSpaceInformation^>(tunerDefinition_->TuningSpaces->Length);
		for (int i = 0; i < tuningSpaces->Length; i++)
		{
			tuningSpaces[i] = tunerDefinition_->TuningSpaces[i]->Clone();
		}
		return tuningSpaces;
	}

	int Tuner::DeviceID::get()
	{
		Internal::ManagedLock lock(this);

		if (pTsSourceTuner_ == nullptr)
		{
			throw gcnew System::ObjectDisposedException("Tuner");
		}

		return pTsSourceTuner_->GetDeviceID();
	}

	bool Tuner::IsLocked::get()
	{
		return lockCookie_ != System::Guid::Empty;
	}

	void Tuner::Unlock(System::Guid cookie)
	{
		Internal::ManagedLock lock(this);

		if (cookie != lockCookie_)
		{
			throw gcnew System::InvalidOperationException("Invalid lock cookie.");
		}

		lockCookie_ = System::Guid::Empty;
	}

	bool Tuner::IsRunning::get()
	{
		return running_;
	}

	array<System::Byte>^ Tuner::CardID::get()
	{
		Internal::ManagedLock lock(this);

		if (pTsDescrambler_ == nullptr)
		{
			throw gcnew System::ObjectDisposedException("Tuner");
		}

		CMediaData mediaData(0);
		pTsDescrambler_->GetCardID(&mediaData);

		array<System::Byte>^ buffer = gcnew array<System::Byte>(mediaData.GetSize());
		pin_ptr<BYTE> pBuffer = &buffer[0];
		memcpy(pBuffer, mediaData.GetData(), mediaData.GetSize());

		return buffer;
	}

	bool Tuner::EnableEmmProcess::get()
	{
		return enableEmmProcess_;
	}

	void Tuner::EnableEmmProcess::set(bool value)
	{
		Internal::ManagedLock lock(this);

		if (pTsDescrambler_ == nullptr)
		{
			throw gcnew System::ObjectDisposedException("Tuner");
		}

		if (lockCookie_ != System::Guid::Empty)
		{
			throw gcnew System::InvalidOperationException("Tuner locked.");
		}

		if (enableEmmProcess_ != value)
		{
			enableEmmProcess_ = value;
			if (running_ == true)
			{
				pTsDescrambler_->EnableEmmProcess(enableEmmProcess_);
			}
		}
	}

	bool Tuner::EnableLnbPower::get()
	{
		return enableLnbPower_;
	}

	void Tuner::EnableLnbPower::set(bool value)
	{
		Internal::ManagedLock lock(this);

		if (pTsSourceTuner_ == nullptr)
		{
			throw gcnew System::ObjectDisposedException("Tuner");
		}

		if (lockCookie_ != System::Guid::Empty)
		{
			throw gcnew System::InvalidOperationException("Tuner locked.");
		}

		if (enableLnbPower_ != value)
		{
			enableLnbPower_ = value;
			if (running_ == true)
			{
				pTsSourceTuner_->SetLnbPower(enableLnbPower_);
			}
		}
	}

	float Tuner::SignalLevel::get()
	{
		Internal::ManagedLock lock(this);

		if (pTsSourceTuner_ == nullptr)
		{
			throw gcnew System::ObjectDisposedException("Tuner");
		}

		return pTsSourceTuner_->GetSignalLevel();
	}

	int Tuner::TuningSpace::get()
	{
		Internal::ManagedLock lock(this);

		if (pTsSourceTuner_ == nullptr)
		{
			throw gcnew System::ObjectDisposedException("Tuner");
		}

		return pTsSourceTuner_->GetCurSpace();
	}

	int Tuner::Channel::get()
	{
		Internal::ManagedLock lock(this);

		if (pTsSourceTuner_ == nullptr)
		{
			throw gcnew System::ObjectDisposedException("Tuner");
		}

		return pTsSourceTuner_->GetCurChannel();
	}

	ISelectedChannel^ Tuner::GetChannel()
	{
		Internal::ManagedLock lock(this);

		if (pTsSourceTuner_ == nullptr)
		{
			throw gcnew System::ObjectDisposedException("Tuner");
		}

		return gcnew SelectedChannel(pTsSourceTuner_->GetCurSpace(), pTsSourceTuner_->GetCurChannel());
	}

	void Tuner::SetChannel(int tuningSpace, int channel, int serviceID)
	{
		Internal::ManagedLock lock(this);

		if (pTsSourceTuner_ == nullptr)
		{
			throw gcnew System::ObjectDisposedException("Tuner");
		}

		if (lockCookie_ != System::Guid::Empty)
		{
			throw gcnew System::InvalidOperationException("Tuner locked.");
		}

		// ύXȂ̏ꍇ͖
		if ((pTsSourceTuner_->GetCurSpace() == tuningSpace) && (pTsSourceTuner_->GetCurChannel() == channel))
		{
			return;
		}

		pTsSourceTuner_->SetChannel(tuningSpace, channel);
		pStreamSink_->ChangeServiceIDForAllQueue(serviceID);
	}

	System::Guid Tuner::SetChannelWithLock(int tuningSpace, int channel, int serviceID)
	{
		Internal::ManagedLock lock(this);

		if (pTsSourceTuner_ == nullptr)
		{
			throw gcnew System::ObjectDisposedException("Tuner");
		}

		if (lockCookie_ != System::Guid::Empty)
		{
			throw gcnew System::InvalidOperationException("Tuner locked.");
		}

		// ύXꍇ
		if ((pTsSourceTuner_->GetCurSpace() != tuningSpace) || (pTsSourceTuner_->GetCurChannel() != channel))
		{
			pTsSourceTuner_->SetChannel(tuningSpace, channel);
			pStreamSink_->ChangeServiceIDForAllQueue(serviceID);
		}

		// bNNbL[蓖Ă
		lockCookie_ = System::Guid::NewGuid();
		return lockCookie_;
	}

	int Tuner::NetworkID::get()
	{
		Internal::ManagedLock lock(this);

		if (pProgAnalyzer_ == nullptr)
		{
			throw gcnew System::ObjectDisposedException("Tuner");
		}

		return pProgAnalyzer_->GetNetworkID();
	}

	System::String^ Tuner::NetworkName::get()
	{
		Internal::ManagedLock lock(this);

		if (pProgAnalyzer_ == nullptr)
		{
			throw gcnew System::ObjectDisposedException("Tuner");
		}

		const CBonString networkName = pProgAnalyzer_->GetNetworkName();
		return gcnew System::String(networkName);
	}

	int Tuner::TransportStreamID::get()
	{
		Internal::ManagedLock lock(this);

		if (pProgAnalyzer_ == nullptr)
		{
			throw gcnew System::ObjectDisposedException("Tuner");
		}

		return static_cast<int>(pProgAnalyzer_->GetTsID());
	}

	System::DateTime Tuner::StreamTime::get()
	{
		Internal::ManagedLock lock(this);

		if (pProgAnalyzer_ == nullptr)
		{
			throw gcnew System::ObjectDisposedException("Tuner");
		}

		SYSTEMTIME streamTime;
		memset(&streamTime, 0, sizeof streamTime);
		bool result = pProgAnalyzer_->GetStreamTime(&streamTime);
		if (result == false)
		{
			return System::DateTime::MinValue;
		}

		// GetStreamTime() is local time
		return System::DateTime(streamTime.wYear, streamTime.wMonth, streamTime.wDay,
			streamTime.wHour, streamTime.wMinute, streamTime.wSecond, streamTime.wMilliseconds,
			System::DateTimeKind::Local);
	}

	array<IProgramInformation^>^ Tuner::Programs::get()
	{
		System::Collections::Generic::List<IProgramInformation^>^ programs =
			gcnew System::Collections::Generic::List<IProgramInformation^>();

		Internal::ManagedLock lock(this);

		if (pProgAnalyzer_ == nullptr)
		{
			throw gcnew System::ObjectDisposedException("Tuner");
		}

		for (WORD i = 0; i < pProgAnalyzer_->GetServiceNum(); i++)
		{
			WORD serviceID = pProgAnalyzer_->GetServiceID(i);

			WORD programVideoID = pProgAnalyzer_->GetProgramVideoPID(serviceID);
			WORD programAudioID = pProgAnalyzer_->GetProgramAudioPID(serviceID);
			WORD programPcrID = pProgAnalyzer_->GetProgramPcrPID(serviceID);

			const CBonString serviceName = pProgAnalyzer_->GetServiceName(serviceID);

			const IEpgItem *pEpgItem = pProgAnalyzer_->GetCurrentEpgInfo(serviceID);

			programs->Add(gcnew ProgramInformation(
				serviceID, programVideoID, programAudioID, programPcrID,
				gcnew System::String(serviceName), gcnew EpgData(pEpgItem)));
		}

		return programs->ToArray();
	}

	DescramblingStates Tuner::DescramblingStatus::get()
	{
		Internal::ManagedLock lock(this);

		if (pTsDescrambler_ == nullptr)
		{
			throw gcnew System::ObjectDisposedException("Tuner");
		}

		return static_cast<DescramblingStates>(
			pTsDescrambler_->GetDescramblingState(pProgAnalyzer_->GetCurrentServiceID()));
	}

	IDataQueue^ Tuner::CreateQueue(int serviceID, int programID, __int64 maxSize, bool locked)
	{
		Internal::ManagedLock lock(this);

		if (pStreamSink_ == nullptr)
		{
			throw gcnew System::ObjectDisposedException("Tuner");
		}

		System::Collections::Generic::KeyValuePair<IDataQueue^, int> queue =
			pStreamSink_->CreateQueue(this, serviceID, programID, maxSize, locked);
		if (queue.Value == 1)
		{
			pTsDescrambler_->EnableEmmProcess(enableEmmProcess_);
			pTsSourceTuner_->SetLnbPower(enableLnbPower_);
			dynamic_cast<IMediaDecoder *>(pTsSourceTuner_)->PlayDecoder();

			running_ = true;
		}
		return queue.Key;
	}

	void Tuner::ChangeServiceID(DataQueue^ queue, int newServiceID)
	{
		Internal::ManagedLock lock(this);

		if (pStreamSink_ == nullptr)
		{
			throw gcnew System::ObjectDisposedException("Tuner");
		}

		pStreamSink_->ChangeServiceID(queue, newServiceID);
	}

	void Tuner::RemoveQueue(DataQueue^ queue)
	{
		Internal::ManagedLock lock(this);

		if (pStreamSink_ == nullptr)
		{
			return;
		}

		int count = pStreamSink_->RemoveQueue(queue);
		if (count == 0)
		{
			dynamic_cast<IMediaDecoder *>(pTsSourceTuner_)->StopDecoder();
			pTsDescrambler_->EnableEmmProcess(false);
			pTsSourceTuner_->SetLnbPower(false);

			running_ = false;
		}
	}

	void Tuner::ScanService(int count)
	{
		int i = 0;
		while (true)
		{
			{
				Internal::ManagedLock lock(this);

				if (pStreamSink_ == 0)
				{
					return;
				}

				const DWORD serviceCount = pProgAnalyzer_->GetServiceNum();
				if (serviceCount > 0)
				{
					System::Collections::Generic::List<ServiceInformation^>^ services =
						gcnew System::Collections::Generic::List<ServiceInformation^>();
					for (DWORD j = 0; j < serviceCount; j++)
					{
						WORD serviceID = pProgAnalyzer_->GetServiceID(j);
						const CBonString serviceName = pProgAnalyzer_->GetServiceName(serviceID);
						if (serviceName.IsEmpty() == false)
						{
							services->Add(gcnew ServiceInformation(gcnew System::String(serviceName), static_cast<int>(serviceID)));
						}
						else
						{
							break;
						}
					}
					if (services->Count == serviceCount)
					{
						DWORD tuningSpace = pTsSourceTuner_->GetCurSpace();
						DWORD channel = pTsSourceTuner_->GetCurChannel();
						safe_cast<TuningSpaceInformation^>(tunerDefinition_->TuningSpaces[tuningSpace])->SetServices(channel, services->ToArray());
						break;
					}
				}

				i++;
				if (i >= count)
				{
					break;
				}
			}

			System::Threading::Thread::Sleep(100);
		}
	}

	void Tuner::ScanChannelEntry(System::Object^ dummy)
	{
		try
		{
			{
				Internal::ManagedLock lock(this);

				if (pStreamSink_ == 0)
				{
					return;
				}

				pTsDescrambler_->EnableEmmProcess(enableEmmProcess_);
				pTsSourceTuner_->SetLnbPower(enableLnbPower_);
				dynamic_cast<IMediaDecoder *>(pTsSourceTuner_)->PlayDecoder();
			}

			try
			{
				for (int i = 0; i < tunerDefinition_->TuningSpaces->Length; i++)
				{
					ITuningSpaceInformation^ tuningSpace = tunerDefinition_->TuningSpaces[i];
					int j = 0;
					for each (IChannelInformation^ channel in tuningSpace)
					{
						{
							Internal::ManagedLock lock(this);
							if (pStreamSink_ == 0)
							{
								return;
							}
							pTsSourceTuner_->SetChannel(i, j);
						}
						ScanService(50);
						j++;
					}
				}
			}
			finally
			{
				Internal::ManagedLock lock(this);

				if (pStreamSink_ != 0)
				{
					if (pStreamSink_->GetCount() == 0)
					{
						dynamic_cast<IMediaDecoder *>(pTsSourceTuner_)->StopDecoder();
						pTsDescrambler_->EnableEmmProcess(false);
						pTsSourceTuner_->SetLnbPower(false);
					}

					pTsSourceTuner_->SetChannel(0, 0);
				}
			}
		}
		finally
		{
			lockCookie_ = System::Guid::Empty;
		}
	}
}
