//////////////////////////////////////////////////////////////////////////
// BonSDK .NET binding interop library
// Copyright (c) http://2sen.dip.jp/dtv/
//
// This is part of BonSDK.
// BonSDK comes with ABSOLUTELY NO WARRANTY;
// for details see 'GNU GENERAL PUBLIC LICENSE 2.0'.


#include "stdafx.h"

#include "ManagedLock.h"
#include "CreateBonInstance.h"
#include "Tuner.h"
#include "TunerFactory.h"
#include "TuningSpaceInformation.h"
#include "ChannelInformation.h"
#include "TunerDefinition.h"

namespace Bon
{
	static void BonTraceCallback(BonTraceType type, LPCTSTR pMessage)
	{
		if (TunerFactory::traceCallback_ != nullptr)
		{
			TraceType traceType = static_cast<TraceType>(type);
			TunerFactory::traceCallback_(traceType, gcnew System::String(pMessage));
		}
	}

	static TunerFactory::TunerFactory()
	{
		BON_TRACE_REGISTER_CALLBACK(BonTraceCallback);
	}

	void TunerFactory::SetTraceCallback(TraceCallbackDelegate^ traceCallback)
	{
		traceCallback_ = traceCallback;
	}

	bool TunerFactory::IsTraceTarget(TraceType type)
	{
		return IS_BON_TRACE_TARGET(static_cast<BonTraceType>(type));
	}

	void TunerFactory::UpdateTunerDefinitions()
	{
		IBonClassEnumerator* pBonClassEnumerator =
			Internal::CreateBonInstance<IBonClassEnumerator>(L"CBonClassEnumerator");

		try
		{
			System::Collections::Generic::List<ITunerDefinition^>^ tunerDefinitions =
				gcnew System::Collections::Generic::List<ITunerDefinition^>();

			if (pBonClassEnumerator->EnumBonClass(L"IHalTsTuner"))
			{
				DWORD index = 0;
				while (true)
				{
					const CBonString className = pBonClassEnumerator->GetBonClassName(index++);
					if (className.GetLength() == 0)
					{
						break;
					}

					IHalDevice *pHalDevice = BON_SAFE_CREATE<IHalDevice *>(className);
					if (pHalDevice != 0)
					{
						try
						{
							const CBonString deviceName = pHalDevice->GetDeviceName();

							if (pHalDevice->GetDeviceType() == BON_NAME_TO_GUID(TEXT("IHalTsTuner")))
							{
								IHalTsTuner *pHalTsTuner = dynamic_cast<IHalTsTuner *>(pHalDevice);

								System::Collections::Generic::List<ITuningSpaceInformation^>^ tuningSpaces =
									gcnew System::Collections::Generic::List<ITuningSpaceInformation^>();
								
								DWORD tuningSpaceIndex = 0;
								for (;;)
								{
									const CBonString tuningSpaceName = pHalTsTuner->EnumTuningSpace(tuningSpaceIndex);
									if (tuningSpaceName.IsEmpty())
									{
										break;
									}

									System::Collections::Generic::List<ChannelInformation^>^ channels =
										gcnew System::Collections::Generic::List<ChannelInformation^>();

									DWORD channelIndex = 0;
									for (;;)
									{
										const CBonString channelName = pHalTsTuner->EnumChannelName(tuningSpaceIndex, channelIndex);
										if (channelName.IsEmpty())
										{
											break;
										}
										channels->Add(gcnew ChannelInformation(gcnew System::String(channelName)));
										channelIndex++;
									}

									tuningSpaces->Add(gcnew TuningSpaceInformation(
										gcnew System::String(tuningSpaceName), channels->ToArray()));
									tuningSpaceIndex++;
								}

								tunerDefinitions->Add(gcnew TunerDefinition(
									gcnew System::String(className), gcnew System::String(deviceName), tuningSpaces->ToArray()));
							}
						}
						finally
						{
							BON_SAFE_RELEASE(pHalDevice);
						}
					}
				}
			}

			tunerDefinitions_ = tunerDefinitions->ToArray();
		}
		finally
		{
			BON_SAFE_RELEASE(pBonClassEnumerator);
		}
	}

	TunerFactory::TunerFactory()
		: tunerDefinitions_(nullptr)
	{
	}

	int TunerFactory::Count::get()
	{
		Internal::ManagedLock lock(this);

		if (tunerDefinitions_ == nullptr)
		{
			UpdateTunerDefinitions();
		}

		return tunerDefinitions_->Length;
	}

	ITunerDefinition^ TunerFactory::default::get(int index)
	{
		Internal::ManagedLock lock(this);

		if (tunerDefinitions_ == nullptr)
		{
			UpdateTunerDefinitions();
		}

		return tunerDefinitions_[index];
	}

	void TunerFactory::Update()
	{
		Internal::ManagedLock lock(this);

		UpdateTunerDefinitions();
	}

	System::Collections::IEnumerator^ TunerFactory::GetEnumerator()
	{
		Internal::ManagedLock lock(this);

		if (tunerDefinitions_ == nullptr)
		{
			UpdateTunerDefinitions();
		}

		return tunerDefinitions_->GetEnumerator();
	}
}
