//////////////////////////////////////////////////////////////////////////
// BonSDK Stream service remote controller
// Copyright (c) http://2sen.dip.jp/dtv/
//
// This is part of BonSDK.
// BonSDK comes with ABSOLUTELY NO WARRANTY;
// for details see 'GNU GENERAL PUBLIC LICENSE 2.0'.

using System;
using System.Collections;
using System.Collections.Specialized;
using System.Collections.Generic;
using System.IO;
using System.Windows.Forms;
using Bon.Remoting;

namespace Bon
{
	internal sealed class IEPGProgramRegister : IDisposable
	{
		private readonly IRemoteStreamService remoting_ = new RemoteStreamService();
		private readonly List<StreamServiceTag> tags_ = new List<StreamServiceTag>();
		private readonly string tvpidPath_;

		public IEPGProgramRegister(string tvpidPath)
		{
			tvpidPath_ = tvpidPath;

			// L^ς݂̃Xg[T[rXURLQ擾
			StringCollection streamServiceBaseUrls = Properties.Settings.Default.StreamServiceBaseUrls;
			if (streamServiceBaseUrls == null)
			{
				streamServiceBaseUrls = new StringCollection();
				Properties.Settings.Default.StreamServiceBaseUrls = streamServiceBaseUrls;
				Properties.Settings.Default.Save();
			}

			// Xg[T[rXURLQAÑXg[T[rXڑ񋓂
			foreach (string streamServiceBaseUrl in streamServiceBaseUrls)
			{
				try
				{
					StreamServiceTag tag = new StreamServiceTag(new Url(streamServiceBaseUrl), remoting_);
					tags_.Add(tag);
				}
				catch
				{
				}
			}
		}

		public void Dispose()
		{
			remoting_.Dispose();
		}

		public void ShowDialog(IWin32Window parent)
		{
			using (FileStream tvpidStream = new FileStream(tvpidPath_, FileMode.Open, FileAccess.Read, FileShare.Read))
			{
				using (RegisterIEPGForm form = new RegisterIEPGForm(tags_, tvpidStream))
				{
					if (form.ShowDialog(parent) == DialogResult.OK)
					{
						foreach (IInputProgramInformation programInformation in form)
						{
							programInformation.SelectedRecorder.Recorder.RegisterProgram(
								programInformation.StreamUrl, programInformation.TuningSpace, programInformation.Channel,
								programInformation.ServiceID, programInformation.ProgramID,
								programInformation.StartTime, programInformation.EndTime, programInformation.StorageID, programInformation.FileName);
						}
					}
				}
			}
		}
	}
}
