﻿//////////////////////////////////////////////////////////////////////////
// BonSDK Stream service remote controller
// Copyright (c) http://2sen.dip.jp/dtv/
//
// This is part of BonSDK.
// BonSDK comes with ABSOLUTELY NO WARRANTY;
// for details see 'GNU GENERAL PUBLIC LICENSE 2.0'.

using System;
using System.IO;
using System.Threading;
using System.Windows.Forms;

namespace Bon
{
	public static class Program
	{
		/// <summary>
		/// アプリケーションのメイン エントリ ポイントです。
		/// </summary>
		/// <param name="args">コマンドライン引数群</param>
		[STAThread]
		public static void Main(string[] args)
		{
			try
			{
				Application.EnableVisualStyles();
				Application.SetCompatibleTextRenderingDefault(false);
				Application.ThreadException += new System.Threading.ThreadExceptionEventHandler(Application_ThreadException);

				bool minimized = false;
				if (args.Length > 0)
				{
					if (string.Compare(args[0], "/minimized", true) == 0)
					{
						minimized = true;
					}
					else if (File.Exists(args[0]) == true)
					{
						using (IEPGProgramRegister register = new IEPGProgramRegister(args[0]))
						{
							register.ShowDialog(null);
						}
						return;
					}
				}

				Application.Run(new MainForm(minimized));
			}
			catch (Exception ex)
			{
				using (ExceptionForm form = new ExceptionForm(ex))
				{
					form.ShowDialog();
				}
			}
		}

		private static void Application_ThreadException(object sender, ThreadExceptionEventArgs e)
		{
			using (ExceptionForm form = new ExceptionForm(e.Exception))
			{
				Form rootForm = null;
				if (Application.OpenForms.Count > 0)
				{
					rootForm = Application.OpenForms[0];
					while (true)
					{
						Form parent = rootForm.Parent as Form;
						if (parent == null)
						{
							break;
						}
						rootForm = parent;
					}
					if (rootForm.IsDisposed == true)
					{
						rootForm = null;
					}
				}
				form.ShowDialog(rootForm);
			}
		}
	}
}
