//////////////////////////////////////////////////////////////////////////
// BonSDK Stream service remote controller
// Copyright (c) http://2sen.dip.jp/dtv/
//
// This is part of BonSDK.
// BonSDK comes with ABSOLUTELY NO WARRANTY;
// for details see 'GNU GENERAL PUBLIC LICENSE 2.0'.

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Bon.Remoting;

namespace Bon
{
	/// <summary>
	/// R[_[ɓo^ĂvO\𑀍삷tH[łB
	/// </summary>
	internal partial class RecorderForm : Form
	{
		private static readonly DateTimeReverseComparer comparer_ = new DateTimeReverseComparer();

		private readonly StreamServiceTag selectedTag_;
		private readonly IEnumerable<StreamServiceTag> tags_;

		/// <summary>
		/// RXgN^łB
		/// </summary>
		/// <param name="selectedTag">\Ώۂ̃R[_^O</param>
		/// <param name="tags">ǗĂSẴ^OQi`[i[̓Ɏgpj</param>
		public RecorderForm(StreamServiceTag selectedTag, IEnumerable<StreamServiceTag> tags)
		{
			InitializeComponent();

			selectedTag_ = selectedTag;
			tags_ = tags;
			listView.Enabled = selectedTag_.Recorder.Enabled;

			this.Text = string.Format("Recorder {0}- {1}",
				listView.Enabled ? string.Empty : "(Disabled) ", selectedTag_.StreamServiceBaseUrl);

			UpdateList();
		}

		/// <summary>
		/// vO\ꗗ̃Xgr[XV܂B
		/// </summary>
		private void UpdateList()
		{
			SortedList<DateTime, ListViewItem> items = new SortedList<DateTime, ListViewItem>(comparer_);
			foreach (IRegisteredProgram program in selectedTag_.Recorder)
			{
				try
				{
					RecordingStates status = selectedTag_.Recorder.GetRecordingStatus(program.RegisteredID);
					ListViewItem item = new ListViewItem();
					switch (status)
					{
						case RecordingStates.Waiting:
							item.Text = "Waiting";
							item.ImageIndex = 0;
							break;
						case RecordingStates.RecordingForManualStart:
							item.Text = "ManualStart";
							item.ImageIndex = 1;
							break;
						case RecordingStates.RecordingForRegisterdProgram:
							item.Text = "Start";
							item.ImageIndex = 1;
							break;
						case RecordingStates.RecordingStoppedForManual:
							item.Text = "Stopped";
							item.ImageIndex = 2;
							break;
						case RecordingStates.RecordingFnished:
							item.Text = "Finished";
							item.ImageIndex = 3;
							break;
						case RecordingStates.RecordingAborted:
							item.Text = "Aborted";
							item.ImageIndex = 4;
							break;
					}
					item.SubItems.Add(program.StartTime.ToString("yyyy/MM/dd HH:mm:ss"));
					item.SubItems.Add(program.EndTime.ToString("yyyy/MM/dd HH:mm:ss"));
					item.SubItems.Add(string.Format("{0}:{1}:{2}", program.TuningSpace, program.Channel, program.ServiceID));
					item.SubItems.Add(program.ProgramID.ToString());
					item.SubItems.Add(program.FileName);
					item.SubItems.Add(program.StorageID);
					item.SubItems.Add(program.TunerStreamUrl.ToString());
					item.SubItems.Add(program.RegisteredID.ToString());
					item.SubItems.Add(program.RegisteredTime.ToString("yyyy/MM/dd HH:mm:ss"));
					item.Tag = program;
					items.Add(program.StartTime, item);
				}
				catch
				{
				}
			}

			listView.BeginUpdate();
			try
			{
				listView.Items.Clear();
				foreach (ListViewItem item in items.Values)
				{
					listView.Items.Add(item);
				}
			}
			finally
			{
				listView.EndUpdate();
			}
		}

		/// <summary>
		/// vO\ǉ{^ꂽB
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void addProgramToolStripButton_Click(object sender, EventArgs e)
		{
			// vO\o^tH[\
			using (RegisterProgramForm form = new RegisterProgramForm(selectedTag_, tags_))
			{
				if (form.ShowDialog(this) == DialogResult.OK)
				{
					// \񂷂
					selectedTag_.Recorder.RegisterProgram(
						form.StreamUrl, form.TuningSpace, form.Channel, form.ServiceID, form.ProgramID,
						form.StartTime, form.EndTime, form.StorageID, form.FileName);

					UpdateList();
				}
			}
		}

		/// <summary>
		/// tbV{^ꂽB
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void refreshToolStripButton_Click(object sender, EventArgs e)
		{
			UpdateList();
		}

		/// <summary>
		/// Xgr[Ń}EXꂽB
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void listView_MouseUp(object sender, MouseEventArgs e)
		{
			if (e.Button != MouseButtons.Right)
			{
				return;
			}

			ListViewItem item = listView.GetItemAt(e.Location.X, e.Location.Y);
			if (item == null)
			{
				return;
			}

			item.Selected = true;

			IRegisteredProgram program = (IRegisteredProgram)item.Tag;
			try
			{
				switch (selectedTag_.Recorder.GetRecordingStatus(program.RegisteredID))
				{
					case RecordingStates.Waiting:
						manualStartToolStripMenuItem.Enabled = true;
						manualStopToolStripMenuItem.Enabled = false;
						break;
					case RecordingStates.RecordingForRegisterdProgram:
						manualStartToolStripMenuItem.Enabled = false;
						manualStopToolStripMenuItem.Enabled = true;
						break;
					case RecordingStates.RecordingForManualStart:
						manualStartToolStripMenuItem.Enabled = false;
						manualStopToolStripMenuItem.Enabled = true;
						break;
					case RecordingStates.RecordingStoppedForManual:
						manualStartToolStripMenuItem.Enabled = true;
						manualStopToolStripMenuItem.Enabled = false;
						break;
					case RecordingStates.RecordingFnished:
						manualStartToolStripMenuItem.Enabled = true;
						manualStopToolStripMenuItem.Enabled = false;
						break;
					case RecordingStates.RecordingAborted:
						manualStartToolStripMenuItem.Enabled = true;
						manualStopToolStripMenuItem.Enabled = false;
						break;
				}
			}
			catch
			{
				manualStartToolStripMenuItem.Enabled = false;
				manualStopToolStripMenuItem.Enabled = false;
			}

			// ReLXgj[\
			contextMenuStrip.Tag = program;
			contextMenuStrip.Show(listView, e.Location);
		}

		/// <summary>
		/// Xgr[_uNbNꂽB
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void listView_DoubleClick(object sender, EventArgs e)
		{
			if (listView.SelectedItems.Count == 0)
			{
				return;
			}

			ListViewItem item = listView.SelectedItems[0];
			IRegisteredProgram program = (IRegisteredProgram)item.Tag;
			if (program.AbortedException == null)
			{
				return;
			}

			using (ExceptionForm form = new ExceptionForm(program.AbortedException))
			{
				form.ShowDialog(this);
			}
		}

		/// <summary>
		/// 蓮Jnj[NbNꂽB
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void manualStartToolStripMenuItem_Click(object sender, EventArgs e)
		{
			IRegisteredProgram program = (IRegisteredProgram)contextMenuStrip.Tag;
			selectedTag_.Recorder.StartRecord(program.RegisteredID);
			UpdateList();
		}

		/// <summary>
		/// 蓮~j[NbNꂽB
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void manualStopToolStripMenuItem_Click(object sender, EventArgs e)
		{
			IRegisteredProgram program = (IRegisteredProgram)contextMenuStrip.Tag;
			selectedTag_.Recorder.StopRecord(program.RegisteredID);
			UpdateList();
		}

		/// <summary>
		/// 폜j[NbNꂽB
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void removeProgramToolStripMenuItem_Click(object sender, EventArgs e)
		{
			IRegisteredProgram program = (IRegisteredProgram)contextMenuStrip.Tag;
			if (MessageBox.Show(this, "Remove this program, OK?", "Remove program from recorder...",
				MessageBoxButtons.OKCancel, MessageBoxIcon.Warning, MessageBoxDefaultButton.Button2) == DialogResult.OK)
			{
				selectedTag_.Recorder.Remove(program.RegisteredID);
				UpdateList();
			}
		}

		private sealed class DateTimeReverseComparer : IComparer<DateTime>
		{
			public int Compare(DateTime x, DateTime y)
			{
				return y.CompareTo(x);
			}
		}
	}
}