//////////////////////////////////////////////////////////////////////////
// BonSDK Stream service remote controller
// Copyright (c) http://2sen.dip.jp/dtv/
//
// This is part of BonSDK.
// BonSDK comes with ABSOLUTELY NO WARRANTY;
// for details see 'GNU GENERAL PUBLIC LICENSE 2.0'.

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Text;
using System.Threading;
using System.Windows.Forms;
using Bon.Remoting;
using Bon.IEPG;

namespace Bon
{
	/// <summary>
	/// vO\Rg[łB
	/// </summary>
	internal partial class RegisterIEPGControl : UserControl, IInputProgramInformation
	{
		private delegate void AddStreamServiceEntryDelegate(TreeNode streamServiceNode);
		private delegate void AddTunerEntryDelegate(TreeNode streamServiceNode, TreeNode tunerNode, TreeNode selectNode);

		private readonly RegisterIEPGForm parent_;
		private readonly IIEPGItem item_;

		/// <summary>
		/// RXgN^łB
		/// </summary>
		/// <param name="parent">ẽtH[</param>
		/// <param name="tags">ǗĂSẴ^OQ</param>
		/// <param name="tvpidStream">iEPGXg[</param>
		public RegisterIEPGControl(RegisterIEPGForm parent, IEnumerable<StreamServiceTag> tags, IIEPGItem item)
		{
			InitializeComponent();

			parent_ = parent;
			item_ = item;

			// R[_[񋓂ăXg[Wc[r[𐶐
			TreeNode firstStorageIDNode = null;
			Dictionary<string, TreeNode> nodeDictionary = new Dictionary<string, TreeNode>();
			foreach (StreamServiceTag tag in tags)
			{
				try
				{
					string[] storageIDs = tag.Recorder.StorageIDs;
					TreeNode streamServiceNode = new TreeNode(
						string.Format("{0}:{1}{2}", tag.StreamServiceBaseUrl.Host, tag.StreamServiceBaseUrl.Port,
						tag.Recorder.Enabled ? string.Empty : " (Disabled)"));
					streamServiceNode.ImageIndex = 0;
					streamServiceNode.SelectedImageIndex = 0;
					foreach (string storageID in storageIDs)
					{
						long currentSpace = tag.Recorder.GetCurrentSpaces(storageID);
						TreeNode storageIDNode = new TreeNode(
							string.Format("{0} / {1} MB", storageID, currentSpace / 1024 / 1024));
						storageIDNode.ImageIndex = 1;
						storageIDNode.SelectedImageIndex = 1;
						storageIDNode.Tag = new KeyValuePair<StreamServiceTag, string>(tag, storageID);
						streamServiceNode.Nodes.Add(storageIDNode);
						nodeDictionary.Add(tag.StreamServiceBaseUrl.ToString() + storageID, storageIDNode);
						if (firstStorageIDNode == null)
						{
							firstStorageIDNode = storageIDNode;
						}
					}
					storageTreeView.Nodes.Add(streamServiceNode);
				}
				catch
				{
				}
			}

			storageTreeView.ExpandAll();

			if (firstStorageIDNode != null)
			{
				storageTreeView.SelectedNode = firstStorageIDNode;
			}

			startTimeDateTimePicker.Value = item.StartTime;
			endTimeDateTimePicker.Value = item.EndTime;

			fileNameTextBox.Text = item.Title;

			splitContainer.Panel2MinSize = 332;

			foreach (StreamServiceTag tag in tags)
			{
				AddRemoteTunerCollection(tag);
			}
		}

		/// <summary>
		/// `[i[RNVǉ܂B
		/// </summary>
		/// <param name="tag">Xg[T[rX^O</param>
		/// <remarks>񓯊ŃXg[T[rXm[hǉ܂B</remarks>
		private void AddRemoteTunerCollection(StreamServiceTag tag)
		{
			TreeNode streamServiceNode = null;

			tunerTreeView.BeginUpdate();
			try
			{
				// c[m[h𐶐
				streamServiceNode = new TreeNode(tag.StreamServiceBaseUrl.ToString());
				streamServiceNode.ImageIndex = 0;
				streamServiceNode.SelectedImageIndex = 0;
				streamServiceNode.Tag = tag;

				tunerTreeView.Nodes.Add(streamServiceNode);
				tunerTreeView.ExpandAll();
			}
			finally
			{
				tunerTreeView.EndUpdate();
			}

			// ڑɎԂ邩Ȃ̂ŁAXbhv[ŏ
			ThreadPool.QueueUserWorkItem(this.GetRemoteTunerCollectionEntry, streamServiceNode);
		}

		/// <summary>
		/// IRemoteTunerCollectionC^[tFCXɐڑXbhv[Gg|CgłB
		/// </summary>
		/// <param name="state">ڑTreeNodeNX̃CX^X</param>
		private void GetRemoteTunerCollectionEntry(object state)
		{
			TreeNode node = (TreeNode)state;
			StreamServiceTag tag = (StreamServiceTag)node.Tag;

			try
			{
				TreeNode firstNode = null;
				bool sent = false;

				// RNV񋓂A`[i[Q
				foreach (IRemoteTuner tuner in tag.TunerCollection)
				{
					// `[i[݂̌̏Ԃ𓾂
					IRemoteTunerStatus status = tuner.GetTunerStatus();
					IRemoteTuningSpaceInformation[] tuningSpaces = tuner.GetTuningSpaces();

					// `[i[ɑΉc[m[h𐶐
					TreeNode tunerNode = new TreeNode(string.Format("[{0}] {1}", status.DeviceID, status.DeviceName));
					tunerNode.ImageIndex = 2;
					tunerNode.SelectedImageIndex = 2;

					try
					{
						// T[rXIDړĨ`[jOXy[Xƃ`l𓾂
						for (int i = 0; i < tuningSpaces.Length; i++)
						{
							IRemoteChannelInformation[] channels = tuningSpaces[i].Channels;
							for (int j = 0; j < channels.Length; j++)
							{
								IRemoteServiceInformation[] services = channels[j].Services;
								for (int k = 0; k < services.Length; k++)
								{
									// T[rXID
									IRemoteServiceInformation service = services[k];
									if (service.ServiceID == item_.ServiceID)
									{
										// T[rXc[m[h𐶐
										TreeNode serviceNode = new TreeNode(string.Format("[{0}] {1}", service.ServiceID, service.Name));
										serviceNode.ImageIndex = 7;
										serviceNode.SelectedImageIndex = 7;
										serviceNode.Tag = new StreamInformationTag(status.StreamUrls, i, j, service.ServiceID);

										// `lc[m[h𐶐
										TreeNode channelNode = new TreeNode(string.Format("{0} / {1}", tuningSpaces[i].Name, channels[j].Name));
										channelNode.ImageIndex = 6;
										channelNode.SelectedImageIndex = 6;
										channelNode.Tag = new StreamInformationTag(status.StreamUrls, i, j, -1);

										channelNode.Nodes.Add(serviceNode);
										tunerNode.Nodes.Add(channelNode);

										if (firstNode == null)
										{
											firstNode = serviceNode;
										}
									}
								}
							}
						}
					}
					catch
					{
					}

					// Kp\ȃT[rX`[i[ɂȂ
					if (tunerNode.Nodes.Count == 0)
					{
						TreeNode noAvailableNode = new TreeNode("(Service not available)");
						noAvailableNode.ImageIndex = 15;
						noAvailableNode.SelectedImageIndex = 15;
						tunerNode.Nodes.Add(noAvailableNode);
					}

					if (sent == false)
					{
						this.Invoke(new AddTunerEntryDelegate(this.AddTunerEntry), node, tunerNode, firstNode);
						sent = true;
					}
					else
					{
						this.Invoke(new AddTunerEntryDelegate(this.AddTunerEntry), node, tunerNode, null);
					}
				}
			}
			catch (Exception ex)
			{
				tag.Exception = ex;
				this.Invoke(new AddStreamServiceEntryDelegate(this.SetStreamServiceInvalidEntry), node);
			}
		}

		/// <summary>
		/// c[r[Ƀ`[i[ǉ}[VOGg|CgłB
		/// </summary>
		/// <param name="streamServiceNode">Xg[T[rXc[m[h</param>
		/// <param name="tunerNode">ǉc[m[h</param>
		/// <param name="selectNode">Iׂm[h</param>
		private void AddTunerEntry(TreeNode streamServiceNode, TreeNode tunerNode, TreeNode selectNode)
		{
			tunerTreeView.BeginUpdate();
			try
			{
				streamServiceNode.Nodes.Add(tunerNode);
				streamServiceNode.ExpandAll();
				streamServiceNode.EnsureVisible();
				if (selectNode != null)
				{
					tunerTreeView.SelectedNode = selectNode;
				}
			}
			catch
			{
				streamServiceNode.ExpandAll();
			}
			finally
			{
				tunerTreeView.EndUpdate();
			}
		}

		/// <summary>
		/// Xg[T[rXɖACRݒ肷}[VOGg|CgłB
		/// </summary>
		/// <param name="streamServiceNode">ݒ肷c[m[h</param>
		private void SetStreamServiceInvalidEntry(TreeNode streamServiceNode)
		{
			try
			{
				streamServiceNode.ImageIndex = 1;
				streamServiceNode.SelectedImageIndex = 1;
				streamServiceNode.EnsureVisible();
			}
			catch
			{
			}
		}

		/// <summary>
		/// L`FbN{bNXύXꂽB
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void enableCheckBox_CheckedChanged(object sender, EventArgs e)
		{
			panel1.Enabled = requireRegistCheckBox.Checked;
			splitContainer.Panel2.Enabled = requireRegistCheckBox.Checked;

			parent_.UpdateButton();
		}

		/// <summary>
		/// JnύXꂽB
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void startTimeDateTimePicker_ValueChanged(object sender, EventArgs e)
		{
			parent_.UpdateButton();
		}

		/// <summary>
		/// IύXꂽB
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void endTimeDateTimePicker_ValueChanged(object sender, EventArgs e)
		{
			parent_.UpdateButton();
		}

		/// <summary>
		/// `[i[c[r[̃m[hIꂽB
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void tunerTreeView_AfterSelect(object sender, TreeViewEventArgs e)
		{
			// Xg[URLQ\
			TreeNode selectedNode = tunerTreeView.SelectedNode;
			if (selectedNode != null)
			{
				StreamInformationTag tag = selectedNode.Tag as StreamInformationTag;
				if (tag != null)
				{
					isRawStreamCheckBox.Checked = tag.ServiceID == -1;

					// Ώۂ̃`[i[ɕωꍇ
					StreamInformationTag lastTag = (StreamInformationTag)streamListView.Tag;
					if ((lastTag == null) || (object.ReferenceEquals(lastTag.StreamUrls, tag.StreamUrls) == false))
					{
						streamListView.Tag = tag;
						streamListView.BeginUpdate();
						try
						{
							streamListView.Items.Clear();
							ListViewItem firstStreamUrlItem = null;
							foreach (IUrl streamUrl in tag.StreamUrls)
							{
								ListViewItem item = new ListViewItem(
									string.Format("{0}:{1}", streamUrl.Host, streamUrl.Port));
								item.ImageIndex = 2;
								item.Tag = streamUrl;
								streamListView.Items.Add(item);
								if (firstStreamUrlItem == null)
								{
									firstStreamUrlItem = item;
								}
							}
							if (firstStreamUrlItem != null)
							{
								firstStreamUrlItem.Selected = true;
							}
						}
						finally
						{
							streamListView.EndUpdate();
						}
					}
				}
			}

			parent_.UpdateButton();
		}

		/// <summary>
		/// Xg[Xgr[̃m[hIꂽB
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void streamListView_SelectedIndexChanged(object sender, EventArgs e)
		{
			parent_.UpdateButton();
		}

		/// <summary>
		/// Xg[WIDc[r[̃m[hIꂽB
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void storageTreeView_AfterSelect(object sender, TreeViewEventArgs e)
		{
			parent_.UpdateButton();
		}

		/// <summary>
		/// o^Kvǂ擾܂B
		/// </summary>
		public bool RequireRegist
		{
			get
			{
				return requireRegistCheckBox.Checked;
			}
		}

		/// <summary>
		/// ͏Ԃɖ肪Ȃǂ擾܂B
		/// </summary>
		public bool IsValid
		{
			get
			{
				TreeNode selectedNode = storageTreeView.SelectedNode;
				if (selectedNode == null)
				{
					return false;
				}

				if ((selectedNode.Tag is KeyValuePair<StreamServiceTag, string>) == false)
				{
					return false;
				}

				if (streamListView.SelectedItems.Count == 0)
				{
					return false;
				}

				if (fileNameTextBox.Text.Length == 0)
				{
					return false;
				}

				TimeSpan duration = endTimeDateTimePicker.Value - startTimeDateTimePicker.Value;
				if (duration <= TimeSpan.Zero)
				{
					return false;
				}

				selectedNode = tunerTreeView.SelectedNode;
				if (selectedNode == null)
				{
					return false;
				}

				if (selectedNode.Tag is StreamInformationTag)
				{
					return true;
				}
				else
				{
					return false;
				}
			}
		}

		/// <summary>
		/// Iꂽ`[jOXy[X擾܂B
		/// </summary>
		public int TuningSpace
		{
			get
			{
				TreeNode tunerNode = tunerTreeView.SelectedNode;
				StreamInformationTag tag = (StreamInformationTag)tunerNode.Tag;
				return tag.TuningSpace;
			}
		}

		/// <summary>
		/// Iꂽ`l擾܂B
		/// </summary>
		public int Channel
		{
			get
			{
				TreeNode tunerNode = tunerTreeView.SelectedNode;
				StreamInformationTag tag = (StreamInformationTag)tunerNode.Tag;
				return tag.Channel;
			}
		}

		/// <summary>
		/// ͂ꂽT[rXID擾܂B
		/// </summary>
		public int ServiceID
		{
			get
			{
				if (isRawStreamCheckBox.Checked == false)
				{
					TreeNode tunerNode = tunerTreeView.SelectedNode;
					StreamInformationTag tag = (StreamInformationTag)tunerNode.Tag;
					return tag.ServiceID;
				}
				else
				{
					return -1;
				}
			}
		}

		/// <summary>
		/// ͂ꂽvOID擾܂B
		/// </summary>
		public int ProgramID
		{
			get
			{
				if (isRawStreamCheckBox.Checked == false)
				{
					return item_.ProgramID;
				}
				else
				{
					return -1;
				}
			}
		}

		/// <summary>
		/// IꂽR[_[^O擾܂B
		/// </summary>
		public StreamServiceTag SelectedRecorder
		{
			get
			{
				return ((KeyValuePair<StreamServiceTag, string>)storageTreeView.SelectedNode.Tag).Key;
			}
		}

		/// <summary>
		/// IꂽXg[WID擾܂B
		/// </summary>
		public string StorageID
		{
			get
			{
				return ((KeyValuePair<StreamServiceTag, string>)storageTreeView.SelectedNode.Tag).Value;
			}
		}

		/// <summary>
		/// IꂽXg[URL擾܂B
		/// </summary>
		public IUrl StreamUrl
		{
			get
			{
				return (IUrl)streamListView.SelectedItems[0].Tag;
			}
		}

		/// <summary>
		/// Jn擾܂B
		/// </summary>
		public DateTime StartTime
		{
			get
			{
				return startTimeDateTimePicker.Value;
			}
		}

		/// <summary>
		/// I擾܂B
		/// </summary>
		public DateTime EndTime
		{
			get
			{
				return endTimeDateTimePicker.Value;
			}
		}

		/// <summary>
		/// ͂ꂽt@C擾܂B
		/// </summary>
		public string FileName
		{
			get
			{
				return fileNameTextBox.Text;
			}
		}

		/// <summary>
		/// c[r[őIꂽXg[̏i[NXłB
		/// </summary>
		private sealed class StreamInformationTag
		{
			private readonly IUrl[] streamUrls_;
			private readonly int tuningSpace_;
			private readonly int channel_;
			private readonly int serviceID_;

			/// <summary>
			/// RXgN^łB
			/// </summary>
			/// <param name="streamUrls">Xg[URLQ</param>
			/// <param name="tuningSpace">`[jOXy[Xԍ</param>
			/// <param name="channel">`lԍ</param>
			/// <param name="serviceID">T[rXID</param>
			public StreamInformationTag(IUrl[] streamUrls, int tuningSpace, int channel, int serviceID)
			{
				streamUrls_ = streamUrls;
				tuningSpace_ = tuningSpace;
				channel_ = channel;
				serviceID_ = serviceID;
			}

			/// <summary>
			/// Xg[URLQ擾܂B
			/// </summary>
			public IUrl[] StreamUrls
			{
				get
				{
					return streamUrls_;
				}
			}

			/// <summary>
			/// `[jOXy[Xԍ擾܂B
			/// </summary>
			public int TuningSpace
			{
				get
				{
					return tuningSpace_;
				}
			}

			/// <summary>
			/// `lԍ擾܂B
			/// </summary>
			public int Channel
			{
				get
				{
					return channel_;
				}
			}

			/// <summary>
			/// T[rXID擾܂B
			/// </summary>
			public int ServiceID
			{
				get
				{
					return serviceID_;
				}
			}
		}
	}
}