//////////////////////////////////////////////////////////////////////////
// BonSDK Stream service remote controller
// Copyright (c) http://2sen.dip.jp/dtv/
//
// This is part of BonSDK.
// BonSDK comes with ABSOLUTELY NO WARRANTY;
// for details see 'GNU GENERAL PUBLIC LICENSE 2.0'.

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Threading;
using System.Windows.Forms;
using Bon.Remoting;

namespace Bon
{
	/// <summary>
	/// vO\tH[łB
	/// </summary>
	internal partial class RegisterProgramForm : Form, IInputProgramInformation
	{
		private delegate void AddStreamServiceEntryDelegate(TreeNode streamServiceNode);
		private delegate void AddTunerEntryDelegate(TreeNode streamServiceNode, TreeNode tunerNode);

		private readonly StreamServiceTag recorder_;

		/// <summary>
		/// RXgN^łB
		/// </summary>
		/// <param name="selectedTag">\Ώۂ̃R[_^O</param>
		/// <param name="tags">ǗĂSẴ^OQi`[i[̓Ɏgpj</param>
		public RegisterProgramForm(StreamServiceTag selectedTag, IEnumerable<StreamServiceTag> tags)
		{
			InitializeComponent();

			recorder_ = selectedTag;

			// Xg[WID񋓂
			try
			{
				foreach (string storageID in recorder_.Recorder.StorageIDs)
				{
					long currentSpace = recorder_.Recorder.GetCurrentSpaces(storageID);
					ListViewItem item = new ListViewItem(storageID);
					item.SubItems.Add(string.Format("{0} MB", currentSpace / 1024 / 1024));
					item.ImageIndex = 1;
					item.Tag = storageID;
					storageListView.Items.Add(item);
				}
			}
			catch
			{
			}

			// ߂̎̎̌JグftHgƂ
			DateTime now = DateTime.Now.AddHours(1);
			startTimeDateTimePicker.Value = new DateTime(now.Year, now.Month, now.Day, now.Hour, 0, 0);

			// ꎞԂftHgƂ
			endTimeDateTimePicker.Value = startTimeDateTimePicker.Value.AddHours(1);

			foreach (StreamServiceTag tag in tags)
			{
				AddRemoteTunerCollection(tag);
			}
		}

		/// <summary>
		/// `[i[RNVǉ܂B
		/// </summary>
		/// <param name="tag">Xg[T[rX^O</param>
		/// <remarks>񓯊ŃXg[T[rXm[hǉ܂B</remarks>
		private void AddRemoteTunerCollection(StreamServiceTag tag)
		{
			TreeNode streamServiceNode = null;

			tunerTreeView.BeginUpdate();
			try
			{
				// c[m[h𐶐
				streamServiceNode = new TreeNode(tag.StreamServiceBaseUrl.ToString());
				streamServiceNode.ImageIndex = 0;
				streamServiceNode.SelectedImageIndex = 0;
				streamServiceNode.Tag = tag;

				tunerTreeView.Nodes.Add(streamServiceNode);
				tunerTreeView.ExpandAll();
			}
			finally
			{
				tunerTreeView.EndUpdate();
			}

			// ڑɎԂ邩Ȃ̂ŁAXbhv[ŏ
			ThreadPool.QueueUserWorkItem(this.GetRemoteTunerCollectionEntry, streamServiceNode);
		}

		/// <summary>
		/// IRemoteTunerCollectionC^[tFCXɐڑXbhv[Gg|CgłB
		/// </summary>
		/// <param name="state">ڑTreeNodeNX̃CX^X</param>
		private void GetRemoteTunerCollectionEntry(object state)
		{
			TreeNode node = (TreeNode)state;
			StreamServiceTag tag = (StreamServiceTag)node.Tag;

			try
			{
				// RNV񋓂A`[i[Q
				foreach (IRemoteTuner tuner in tag.TunerCollection)
				{
					// `[i[݂̌̏Ԃ𓾂
					IRemoteTunerStatus status = tuner.GetTunerStatus();
					IRemoteTuningSpaceInformation[] tuningSpaces = tuner.GetTuningSpaces();

					// `[i[ɑΉc[m[h𐶐
					TreeNode tunerNode = new TreeNode(string.Format("[{0}] {1}", status.DeviceID, status.DeviceName));
					tunerNode.ImageIndex = 2;
					tunerNode.SelectedImageIndex = 2;

					for (int i = 0; i < tuningSpaces.Length; i++)
					{
						// `[jOXy[XɃc[m[h𐶐
						TreeNode tuningSpaceNode = new TreeNode(tuningSpaces[i].Name);
						tuningSpaceNode.ImageIndex = 5;
						tuningSpaceNode.SelectedImageIndex = 5;

						// `l񋓂
						IRemoteChannelInformation[] channels = tuningSpaces[i].Channels;
						for (int j = 0; j < channels.Length; j++)
						{
							// `l̃c[m[h𐶐
							TreeNode channelNode = new TreeNode(channels[j].Name);
							channelNode.ImageIndex = 6;
							channelNode.SelectedImageIndex = 6;
							channelNode.Tag = new StreamInformationTag(status.StreamUrls, i, j, -1);

							IRemoteServiceInformation[] services = channels[j].Services;
							for (int k = 0; k < services.Length; k++)
							{
								// T[rX̃c[m[h𐶐
								TreeNode serviceNode = new TreeNode(string.Format("[{0}] {1}", services[k].ServiceID, services[k].Name));
								serviceNode.ImageIndex = 7;
								serviceNode.SelectedImageIndex = 7;
								serviceNode.Tag = new StreamInformationTag(status.StreamUrls, i, j, services[k].ServiceID);

								channelNode.Nodes.Add(serviceNode);
							}

							tuningSpaceNode.Nodes.Add(channelNode);
						}

						tunerNode.Nodes.Add(tuningSpaceNode);
					}

					this.Invoke(new AddTunerEntryDelegate(this.AddTunerEntry), node, tunerNode);
				}
			}
			catch (Exception ex)
			{
				tag.Exception = ex;
				this.Invoke(new AddStreamServiceEntryDelegate(this.SetStreamServiceInvalidEntry), node);
			}
		}

		/// <summary>
		/// c[r[Ƀ`[i[ǉ}[VOGg|CgłB
		/// </summary>
		/// <param name="streamServiceNode">Xg[T[rXc[m[h</param>
		/// <param name="tunerNode">ǉc[m[h</param>
		private void AddTunerEntry(TreeNode streamServiceNode, TreeNode tunerNode)
		{
			tunerTreeView.BeginUpdate();
			try
			{
				streamServiceNode.Nodes.Add(tunerNode);
				streamServiceNode.ExpandAll();
				streamServiceNode.EnsureVisible();
			}
			catch
			{
				streamServiceNode.ExpandAll();
			}
			finally
			{
				tunerTreeView.EndUpdate();
			}
		}

		/// <summary>
		/// Xg[T[rXɖACRݒ肷}[VOGg|CgłB
		/// </summary>
		/// <param name="streamServiceNode">ݒ肷c[m[h</param>
		private void SetStreamServiceInvalidEntry(TreeNode streamServiceNode)
		{
			try
			{
				streamServiceNode.ImageIndex = 1;
				streamServiceNode.SelectedImageIndex = 1;
				streamServiceNode.EnsureVisible();
			}
			catch
			{
			}
		}

		/// <summary>
		/// {^̏ԂύX܂B
		/// </summary>
		/// <remarks>e͂ꍇOK{^悤ɂ܂B</remarks>
		private void UpdateButton()
		{
			okButton.Enabled = IsValid;
		}

		/// <summary>
		/// JnύXꂽB
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void startTimeDateTimePicker_ValueChanged(object sender, EventArgs e)
		{
			UpdateButton();
		}

		/// <summary>
		/// IύXꂽB
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void endTimeDateTimePicker_ValueChanged(object sender, EventArgs e)
		{
			UpdateButton();
		}

		/// <summary>
		/// `[i[c[r[̃m[hIꂽB
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void tunerTreeView_AfterSelect(object sender, TreeViewEventArgs e)
		{
			// Xg[URLQ\
			TreeNode selectedNode = tunerTreeView.SelectedNode;
			if (selectedNode != null)
			{
				StreamInformationTag tag = selectedNode.Tag as StreamInformationTag;
				if (tag != null)
				{
					serviceIDTextBox.Text = tag.ServiceID.ToString();

					// Ώۂ̃`[i[ɕωꍇ
					StreamInformationTag lastTag = (StreamInformationTag)streamListView.Tag;
					if ((lastTag == null) || (object.ReferenceEquals(lastTag.StreamUrls, tag.StreamUrls) == false))
					{
						streamListView.Tag = tag;
						streamListView.BeginUpdate();
						try
						{
							streamListView.Items.Clear();
							ListViewItem firstStreamUrlItem = null;
							foreach (IUrl streamUrl in tag.StreamUrls)
							{
								ListViewItem item = new ListViewItem(
									string.Format("{0}:{1}", streamUrl.Host, streamUrl.Port));
								item.ImageIndex = 2;
								item.Tag = streamUrl;
								streamListView.Items.Add(item);
								if (firstStreamUrlItem == null)
								{
									firstStreamUrlItem = item;
								}
							}
							if (firstStreamUrlItem != null)
							{
								firstStreamUrlItem.Selected = true;
							}
						}
						finally
						{
							streamListView.EndUpdate();
						}
					}
				}
			}

			UpdateButton();
		}

		/// <summary>
		/// Xg[Xgr[̃m[hIꂽB
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void streamListView_SelectedIndexChanged(object sender, EventArgs e)
		{
			UpdateButton();
		}

		/// <summary>
		/// Xg[WIDXgr[̃m[hIꂽB
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void storageListView_SelectedIndexChanged(object sender, EventArgs e)
		{
			UpdateButton();
		}

		/// <summary>
		/// ͏Ԃɖ肪Ȃǂ擾܂B
		/// </summary>
		public bool IsValid
		{
			get
			{
				if (storageListView.SelectedItems.Count == 0)
				{
					return false;
				}

				if (streamListView.SelectedItems.Count == 0)
				{
					return false;
				}

				int serviceID = 0;
				if (int.TryParse(serviceIDTextBox.Text, out serviceID) == false)
				{
					return false;
				}
				if (serviceID < -1)
				{
					// i-1Raw streamw肵ƂɂȂ̂ŋj
					return false;
				}

				if (fileNameTextBox.Text.Length == 0)
				{
					return false;
				}

				TimeSpan duration = endTimeDateTimePicker.Value - startTimeDateTimePicker.Value;
				if (duration <= TimeSpan.Zero)
				{
					return false;
				}

				TreeNode selectedNode = tunerTreeView.SelectedNode;
				if (selectedNode == null)
				{
					return false;
				}

				if (selectedNode.Tag is StreamInformationTag)
				{
					return true;
				}
				else
				{
					return false;
				}
			}
		}

		/// <summary>
		/// Iꂽ`[jOXy[X擾܂B
		/// </summary>
		public int TuningSpace
		{
			get
			{
				TreeNode tunerNode = tunerTreeView.SelectedNode;
				StreamInformationTag tag = (StreamInformationTag)tunerNode.Tag;
				return tag.TuningSpace;
			}
		}

		/// <summary>
		/// Iꂽ`l擾܂B
		/// </summary>
		public int Channel
		{
			get
			{
				TreeNode tunerNode = tunerTreeView.SelectedNode;
				StreamInformationTag tag = (StreamInformationTag)tunerNode.Tag;
				return tag.Channel;
			}
		}

		/// <summary>
		/// ͂ꂽT[rXID擾܂B
		/// </summary>
		public int ServiceID
		{
			get
			{
				return int.Parse(serviceIDTextBox.Text);
			}
		}

		/// <summary>
		/// ͂ꂽvOID擾܂B
		/// </summary>
		public int ProgramID
		{
			get
			{
				return int.Parse(programIDTextBox.Text);
			}
		}

		/// <summary>
		/// IꂽXg[WID擾܂B
		/// </summary>
		public string StorageID
		{
			get
			{
				return (string)storageListView.SelectedItems[0].Tag;
			}
		}

		/// <summary>
		/// IꂽR[_[^O擾܂B
		/// </summary>
		public StreamServiceTag SelectedRecorder
		{
			get
			{
				return recorder_;
			}
		}

		/// <summary>
		/// IꂽXg[URL擾܂B
		/// </summary>
		public IUrl StreamUrl
		{
			get
			{
				return (IUrl)streamListView.SelectedItems[0].Tag;
			}
		}

		/// <summary>
		/// Jn擾܂B
		/// </summary>
		public DateTime StartTime
		{
			get
			{
				return startTimeDateTimePicker.Value;
			}
		}

		/// <summary>
		/// I擾܂B
		/// </summary>
		public DateTime EndTime
		{
			get
			{
				return endTimeDateTimePicker.Value;
			}
		}

		/// <summary>
		/// ͂ꂽt@C擾܂B
		/// </summary>
		public string FileName
		{
			get
			{
				return fileNameTextBox.Text;
			}
		}

		/// <summary>
		/// c[r[őIꂽXg[̏i[NXłB
		/// </summary>
		private sealed class StreamInformationTag
		{
			private readonly IUrl[] streamUrls_;
			private readonly int tuningSpace_;
			private readonly int channel_;
			private readonly int serviceID_;

			/// <summary>
			/// RXgN^łB
			/// </summary>
			/// <param name="streamUrls">Xg[URLQ</param>
			/// <param name="tuningSpace">`[jOXy[Xԍ</param>
			/// <param name="channel">`lԍ</param>
			/// <param name="serviceID">T[rXID</param>
			public StreamInformationTag(IUrl[] streamUrls, int tuningSpace, int channel, int serviceID)
			{
				streamUrls_ = streamUrls;
				tuningSpace_ = tuningSpace;
				channel_ = channel;
				serviceID_ = serviceID;
			}

			/// <summary>
			/// Xg[URLQ擾܂B
			/// </summary>
			public IUrl[] StreamUrls
			{
				get
				{
					return streamUrls_;
				}
			}

			/// <summary>
			/// `[jOXy[Xԍ擾܂B
			/// </summary>
			public int TuningSpace
			{
				get
				{
					return tuningSpace_;
				}
			}

			/// <summary>
			/// `lԍ擾܂B
			/// </summary>
			public int Channel
			{
				get
				{
					return channel_;
				}
			}

			/// <summary>
			/// T[rXID擾܂B
			/// </summary>
			public int ServiceID
			{
				get
				{
					return serviceID_;
				}
			}
		}
	}
}