//////////////////////////////////////////////////////////////////////////
// BonSDK Stream service
// Copyright (c) http://2sen.dip.jp/dtv/
//
// This is part of BonSDK.
// BonSDK comes with ABSOLUTELY NO WARRANTY;
// for details see 'GNU GENERAL PUBLIC LICENSE 2.0'.

using System;
using System.Collections;
using System.IO;
using System.Net;
using System.Text;
using System.Xml;
using System.Xml.Schema;
using System.Xml.Xsl;
using System.Collections.Generic;

namespace Bon
{
	/// <summary>
	/// XMLX^CV[ggpāAꂽXg[o͂NXłB
	/// </summary>
	internal sealed class DynamicPresentationRenderer
	{
		private static readonly XmlReaderSettings settings_ = new XmlReaderSettings();

		private readonly string nodeName_;
		private readonly XmlDocument document_ = new XmlDocument();
		private readonly XmlElement rootElement_;

		/// <summary>
		/// ^CvCjVCUłB
		/// </summary>
		static DynamicPresentationRenderer()
		{
			settings_.CheckCharacters = true;
			settings_.CloseInput = false;
			settings_.ConformanceLevel = ConformanceLevel.Document;
			settings_.IgnoreComments = false;
			settings_.IgnoreProcessingInstructions = false;
			settings_.IgnoreWhitespace = false;
			settings_.ProhibitDtd = false;
			settings_.ValidationFlags =
				XmlSchemaValidationFlags.AllowXmlAttributes |
				XmlSchemaValidationFlags.ProcessIdentityConstraints |
				XmlSchemaValidationFlags.ProcessInlineSchema |
				XmlSchemaValidationFlags.ProcessSchemaLocation;
		}

		/// <summary>
		/// RXgN^łB
		/// </summary>
		/// <param name="rootName">[gm[h̖O</param>
		/// <param name="nodeName">em[h̖O</param>
		public DynamicPresentationRenderer(string rootName, string nodeName)
		{
			nodeName_ = nodeName;

			XmlProcessingInstruction pi = document_.CreateProcessingInstruction("xml", "version=\"1.0\"");
			document_.AppendChild(pi);
			rootElement_ = document_.CreateElement(rootName);
			document_.AppendChild(rootElement_);
		}

		private void Add(XmlElement parent, EnvironmentValue value)
		{
			XmlElement element = document_.CreateElement(nodeName_);
			if (string.IsNullOrEmpty(value.Value) == false)
			{
				XmlText text = document_.CreateTextNode(value.Value);
				element.AppendChild(text);
			}
			element.SetAttribute("name", value.Name);
			foreach (KeyValuePair<string, string> attribute in value.GetAttributes())
			{
				element.SetAttribute(attribute.Key, attribute.Value);
			}
			foreach (EnvironmentValue childValue in value)
			{
				Add(element, childValue);
			}
			parent.AppendChild(element);
		}

		/// <summary>
		/// ϐǉ܂B
		/// </summary>
		/// <param name="value">l</param>
		public void Add(EnvironmentValue value)
		{
			Add(rootElement_, value);
		}

		/// <summary>
		/// ϐǉ܂B
		/// </summary>
		/// <param name="values">lQ</param>
		public void Add(IEnumerable<EnvironmentValue> values)
		{
			foreach (EnvironmentValue value in values)
			{
				Add(value);
			}
		}

		/// <summary>
		/// w肳ꂽX^CV[gŐ`sAʂXg[ɏo͂܂B
		/// </summary>
		/// <param name="styleSheetUrl">X^CV[gURL</param>
		/// <param name="outputStream">o͐̃Xg[</param>
		/// <remarks>̃I[o[[h̓NCAgŃX^CV[g܂B</remarks>
		public void Render(string styleSheetUrl, Stream outputStream)
		{
			XmlProcessingInstruction pi = document_.CreateProcessingInstruction(
				"xml-stylesheet", string.Format("type=\"text/xsl\" href=\"{0}\"", styleSheetUrl));
			document_.InsertBefore(pi, rootElement_);
			document_.Save(outputStream);
			document_.RemoveChild(pi);
		}

		/// <summary>
		/// w肳ꂽX^CV[gŐ`sAʂXg[ɏo͂܂B
		/// </summary>
		/// <param name="styleSheetStream">X^CV[gXg[</param>
		/// <param name="outputStream">o͐̃Xg[</param>
		/// <remarks>̃I[o[[h̓T[o[ŃX^CV[g܂B</remarks>
		public void Render(Stream styleSheetStream, Stream outputStream)
		{
			XslCompiledTransform transform = new XslCompiledTransform();
			transform.Load(XmlReader.Create(styleSheetStream, settings_));
			transform.Transform(document_, null, outputStream);
		}
	}

	/// <summary>
	/// DynamicPresentationRendererNXŎgpAϐ`NXłB
	/// </summary>
	internal sealed class EnvironmentValue : IEnumerable<EnvironmentValue>
	{
		private readonly string name_;
		private readonly string value_;
		private readonly List<KeyValuePair<string, string>> attributes_ = new List<KeyValuePair<string, string>>();
		private readonly List<EnvironmentValue> children_ = new List<EnvironmentValue>();

		/// <summary>
		/// RXgN^łB
		/// </summary>
		/// <param name="name">ϐ</param>
		public EnvironmentValue(string name)
		{
			name_ = name;
		}

		/// <summary>
		/// RXgN^łB
		/// </summary>
		/// <param name="name">ϐ</param>
		/// <param name="value">l</param>
		public EnvironmentValue(string name, string value)
		{
			name_ = name;
			value_ = value;
		}

		/// <summary>
		/// RXgN^łB
		/// </summary>
		/// <param name="name">ϐ</param>
		/// <param name="format">tH[}bg</param>
		/// <param name="args">ǉQ</param>
		public EnvironmentValue(string name, string format, params object[] args)
		{
			name_ = name;
			value_ = string.Format(format, args);
		}

		/// <summary>
		/// ϐ擾܂B
		/// </summary>
		public string Name
		{
			get
			{
				return name_;
			}
		}

		/// <summary>
		/// l擾܂B
		/// </summary>
		public string Value
		{
			get
			{
				return value_;
			}
		}

		/// <summary>
		/// ̊ϐ̎qǉ܂B
		/// </summary>
		/// <param name="value">l</param>
		public void Add(EnvironmentValue value)
		{
			children_.Add(value);
		}

		/// <summary>
		/// ǉ̑lݒ肵܂B
		/// </summary>
		/// <param name="name"></param>
		/// <param name="value">l</param>
		public void SetAttribute(string name, string value)
		{
			attributes_.Add(new KeyValuePair<string, string>(name, value));
		}

		/// <summary>
		/// ǉ̑lݒ肵܂B
		/// </summary>
		/// <param name="name"></param>
		/// <param name="format">tH[}bg</param>
		/// <param name="args">ǉQ</param>
		public void SetAttribute(string name, string format, params object[] args)
		{
			SetAttribute(name, string.Format(format, args));
		}

		/// <summary>
		/// Q񋓂RNV擾܂B
		/// </summary>
		/// <returns>RNV</returns>
		public IEnumerable<KeyValuePair<string, string>> GetAttributes()
		{
			return attributes_;
		}

		/// <summary>
		/// q̊ϐQ񋓂܂B
		/// </summary>
		/// <returns>񋓎q</returns>
		public IEnumerator<EnvironmentValue> GetEnumerator()
		{
			return children_.GetEnumerator();
		}

		/// <summary>
		/// q̊ϐQ񋓂܂B
		/// </summary>
		/// <returns>񋓎q</returns>
		IEnumerator IEnumerable.GetEnumerator()
		{
			return children_.GetEnumerator();
		}
	}
}
