//////////////////////////////////////////////////////////////////////////
// BonSDK Stream service
// Copyright (c) http://2sen.dip.jp/dtv/
//
// This is part of BonSDK.
// BonSDK comes with ABSOLUTELY NO WARRANTY;
// for details see 'GNU GENERAL PUBLIC LICENSE 2.0'.

using System;
using System.Diagnostics;
using System.Runtime.InteropServices;
using System.Text;
using System.Threading;

namespace Bon
{
	internal static class Logger
	{
		[DllImport("kernel32.dll")]
		private static extern int GetCurrentThreadId();

		public static void Write(string message)
		{
			InternalWrite(TraceType.Information, message);
		}

		public static void Write(string format, params object[] args)
		{
			InternalWrite(TraceType.Information, string.Format(format, args));
		}

		public static void Write(TraceType type, string message)
		{
			InternalWrite(type, message);
		}

		public static void Write(TraceType type, string format, params object[] args)
		{
			InternalWrite(type, string.Format(format, args));
		}

		public static void Write(Exception e)
		{
			Write(TraceType.Error, e);
		}

		public static void Write(TraceType type, Exception e)
		{
			StringBuilder sb = new StringBuilder();
			sb.AppendFormat("{0}\r\n\r\nException:{1}\r\nProcess:{2}\r\nThread:{3} [Win32={4}:Managed={5}]\r\nHRESULT:{6:X8}\r\nStackTrace:\r\n{7}",
				e.Message, e.GetType().FullName, Process.GetCurrentProcess().Id,
				Thread.CurrentThread.Name, GetCurrentThreadId(), Thread.CurrentThread.ManagedThreadId,
				Marshal.GetHRForException(e), e.StackTrace);

			Exception ie = e.InnerException;
			while (ie != null)
			{
				sb.AppendFormat("\r\n\r\n------------------------------------\r\n{0}\r\n\r\nException:{1}\r\nHRESULT:{2:X8}\r\nStackTrace:\r\n{3}",
					ie.Message, ie.GetType().FullName, Marshal.GetHRForException(ie), ie.StackTrace);
				ie = ie.InnerException;
			}

			InternalWrite(type, sb.ToString());
		}

		private static void InternalWrite(TraceType type, string message)
		{
			try
			{
				if (TunerFactory.IsTraceTarget(type) == false)
				{
					return;
				}

				Trace.WriteLine(
					string.Format("\r\n========================\r\n{0:yyyy/MM/dd HH:mm:ss}:{1}\r\n{2}\r\n========================\r\n",
					DateTime.Now, type, message));

				EventLogEntryType rawType = EventLogEntryType.Error;
				switch (type)
				{
					//fobOCxgOɏo͂Ƒ邽߁AfobO͖
					//case TraceType.Debug:
					//    rawType = EventLogEntryType.Information;
					//    break;
					case TraceType.Information:
						rawType = EventLogEntryType.Information;
						break;
					case TraceType.Warning:
						rawType = EventLogEntryType.Warning;
						break;
					case TraceType.Error:
						rawType = EventLogEntryType.Error;
						break;
				}

				EventLog.WriteEntry(StreamService.ServiceName, message, rawType, 0, 0);
			}
			catch
			{
			}
		}
	}
}
