//////////////////////////////////////////////////////////////////////////
// BonSDK Stream service
// Copyright (c) http://2sen.dip.jp/dtv/
//
// This is part of BonSDK.
// BonSDK comes with ABSOLUTELY NO WARRANTY;
// for details see 'GNU GENERAL PUBLIC LICENSE 2.0'.

using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Configuration.Install;
using System.Diagnostics;
using System.Globalization;
using System.IO;
using System.Net;
using System.Reflection;
using System.ServiceProcess;
using Microsoft.Win32;

namespace Bon
{
	/// <summary>
	/// T[rX̃CXg[łB
	/// </summary>
	[DesignTimeVisible(false)]
	[RunInstaller(true)]
	public partial class StreamServiceInstaller : Installer
	{
		/// <summary>
		/// RXgN^łB
		/// </summary>
		public StreamServiceInstaller()
		{
			InitializeComponent();

			ServiceProcessInstaller serviceProcess = new ServiceProcessInstaller();
			serviceProcess.Account = ServiceAccount.LocalSystem;

			base.Installers.Add(serviceProcess);

			ServiceInstaller service = new ServiceInstaller();
			service.StartType = ServiceStartMode.Automatic;
			service.ServiceName = StreamService.ServiceName;
			service.DisplayName = "BonSDK StreamService";
			service.Description = "BonSDK MPEG-TS Streaming service";

			try
			{
				using (RegistryKey key = Registry.LocalMachine.OpenSubKey(
					@"SYSTEM\CurrentControlSet\Services\HTTPFilter"))
				{
					if (key.ValueCount == 0)
					{
						service.ServicesDependedOn = new string[] { "SCardSvr", "RpcSs" };
					}
					else
					{
						service.ServicesDependedOn = new string[] { "HTTPFilter", "SCardSvr", "RpcSs" };
					}
				}
			}
			catch
			{
				service.ServicesDependedOn = new string[] { "SCardSvr", "RpcSs" };
			}

			base.Installers.Add(service);

			EventLogInstaller eventLog = new EventLogInstaller();
			eventLog.Log = "Application";
			eventLog.Source = StreamService.ServiceName;

			base.Installers.Add(eventLog);
		}

		/// <summary>
		/// CXg[ɌĂяo܂B
		/// </summary>
		/// <param name="stateSaver"></param>
		public override void Install(IDictionary stateSaver)
		{
			base.Install(stateSaver);

			try
			{
				if (HttpListener.IsSupported == false)
				{
					throw new InvalidOperationException("Architecture not supported. (Require Windows XP SP2 or later)");
				}

				// T[rXN̈ݒ肷
				using (RegistryKey key = Registry.LocalMachine.OpenSubKey(
					@"SYSTEM\CurrentControlSet\Services\" + StreamService.ServiceName, true))
				{
					string imagePath = key.GetValue("ImagePath") + " /service";
					key.SetValue("ImagePath", imagePath);
				}

				// WXgɃftHgl𔽉f
				using (StreamServiceRegistryAccessor accessor =
					new StreamServiceRegistryAccessor(true, CultureInfo.InvariantCulture))
				{
					accessor.ResetLocalWebServerRootPath();
					accessor.ResetDatabasePath();
					accessor.ResetExtraTimeSpan();
					accessor.ResetStoragePaths();
					accessor.ResetDefaultPlayers();
				}
			}
			catch (Exception ex)
			{
				EventLog.WriteEntry("StreamServiceInstaller", ex.ToString(), EventLogEntryType.Error);
				throw;
			}
		}

		/// <summary>
		/// R~bgɌĂяo܂B
		/// </summary>
		/// <param name="savedState"></param>
		public override void Commit(IDictionary savedState)
		{
			base.Commit(savedState);

			try
			{
				// T[rXN
				using (ServiceController controller = new ServiceController(StreamService.ServiceName))
				{
					controller.Start();
				}
			}
			catch
			{
			}
		}

		/// <summary>
		/// ACXg[ɌĂяo܂B
		/// </summary>
		/// <param name="savedState"></param>
		public override void Uninstall(IDictionary savedState)
		{
			try
			{
				// T[rX~
				using (ServiceController controller = new ServiceController(StreamService.ServiceName))
				{
					controller.Stop();
					controller.WaitForStatus(ServiceControllerStatus.Stopped, new TimeSpan(0, 0, 30));
				}
			}
			catch
			{
			}

			base.Uninstall(savedState);
		}
	}
}
