//////////////////////////////////////////////////////////////////////////
// BonSDK Stream service connection library (BonDriver interface)
// Copyright (c) http://2sen.dip.jp/dtv/
//
// This is part of BonSDK.
// BonSDK comes with ABSOLUTELY NO WARRANTY;
// for details see 'GNU GENERAL PUBLIC LICENSE 2.0'.

// BonTuner.cpp: CBonTuner NX̃Cve[V
//
//////////////////////////////////////////////////////////////////////

#include <windows.h>
#include <stdlib.h>
#include <tchar.h>
#include <vcclr.h>
#include <string>
#include <vector>
#include "resource.h"

#ifdef UNICODE
typedef std::wstring String;
#else
typedef std::string String;
#endif

#include "BonTuner.h"

#define	TS_PACKET_SIZE 188	// TSpPbgTCY
#define BUFFERSIZE (TS_PACKET_SIZE * 10 * 1024)


//////////////////////////////////////////////////////////////////////
// X}[gbN
//////////////////////////////////////////////////////////////////////

CSmartLock::CSmartLock()
{
	::InitializeCriticalSection(&m_Lock);
}

CSmartLock::~CSmartLock()
{
	::DeleteCriticalSection(&m_Lock);
}

void CSmartLock::Enter()
{
	::EnterCriticalSection(&m_Lock);
}

void CSmartLock::Leave()
{
	::LeaveCriticalSection(&m_Lock);
}

class CBlockLock
{
private:
	CSmartLock &m_Lock;

	CBlockLock(const CBlockLock &);
	void operator =(const CBlockLock &);

public:
	CBlockLock(CSmartLock &lock)
		: m_Lock(lock)
	{
		m_Lock.Enter();
	}

	~CBlockLock()
	{
		m_Lock.Leave();
	}
};


//////////////////////////////////////////////////////////////////////
// }l[Wobt@
//////////////////////////////////////////////////////////////////////

CManagedBuffer::CManagedBuffer(int bufferSize)
	: handle_(0), pBuffer_(0), bufferSize_(bufferSize)
{
	array<System::Byte>^ buffer = gcnew array<System::Byte>(bufferSize_);
	System::Runtime::InteropServices::GCHandle handle(
		System::Runtime::InteropServices::GCHandle::Alloc(buffer,
		System::Runtime::InteropServices::GCHandleType::Pinned));
	handle_ = System::Runtime::InteropServices::GCHandle::ToIntPtr(handle).ToPointer();
	pBuffer_ = static_cast<BYTE *>(handle.AddrOfPinnedObject().ToPointer());
}

CManagedBuffer::~CManagedBuffer()
{
	System::Runtime::InteropServices::GCHandle handle(
		System::Runtime::InteropServices::GCHandle::FromIntPtr(System::IntPtr(handle_)));
	handle.Free();
}

const DWORD CManagedBuffer::GetSize() const
{
	return bufferSize_;
}

const BYTE *CManagedBuffer::GetData() const
{
	return pBuffer_;
}

array<System::Byte>^ CManagedBuffer::GetManagedData()
{
	System::Runtime::InteropServices::GCHandle handle(
		System::Runtime::InteropServices::GCHandle::FromIntPtr(System::IntPtr(handle_)));
	return static_cast<array<System::Byte>^>(handle.Target);
}


//////////////////////////////////////////////////////////////////////
// W[}l[W
//////////////////////////////////////////////////////////////////////

const String CModuleManager::GetProfileString(LPCTSTR name, LPCTSTR defaultValue) const
{
	TCHAR value[257];
	::GetPrivateProfileString(m_ModuleName.c_str(), name, defaultValue, value, 256, m_ConfigrationPath.c_str());
	return value;
}

int CModuleManager::GetProfileInt32(LPCTSTR name, int defaultValue) const
{
	return ::GetPrivateProfileInt(m_ModuleName.c_str(), name, defaultValue, m_ConfigrationPath.c_str());
}

CModuleManager::CModuleManager()
{
	MEMORY_BASIC_INFORMATION mbi;
	::memset(&mbi, 0, sizeof mbi);
	::VirtualQuery(&m_Dummy, &mbi, sizeof mbi);

	TCHAR fullPath[_MAX_PATH + 1];
	::GetModuleFileName(reinterpret_cast<HMODULE>(mbi.AllocationBase), fullPath, _MAX_PATH);

	TCHAR drive[_MAX_DRIVE + 1], dir[_MAX_DIR + 1], fileName[_MAX_FNAME + 1], ext[_MAX_EXT + 1];
	::_tsplitpath_s(fullPath, drive, _MAX_DRIVE, dir, _MAX_DIR, fileName, _MAX_FNAME, ext, _MAX_EXT);

	m_ModuleName = fileName;

	TCHAR path[_MAX_PATH + 1];
	::_tmakepath_s(path, _MAX_PATH, drive, dir, TEXT("BonDriver_StreamService"), TEXT(".ini"));
	m_ConfigrationPath = path;
}

const String CModuleManager::GetModuleName() const
{
	return m_ModuleName;
}

const String CModuleManager::GetRemoteUrl() const
{
	return GetProfileString(TEXT("RemoteUrl"), TEXT("http://localhost:26111/remote/"));
}

const String CModuleManager::GetTunerName() const
{
	return GetProfileString(TEXT("TunerName"), TEXT("CBonBlackTuner"));
}

int CModuleManager::GetDeviceID() const
{
	return GetProfileInt32(TEXT("DeviceID"), 0);
}

void CModuleManager::WriteEvent(System::Diagnostics::EventLogEntryType type, const String &message) const
{
	try
	{
		System::Diagnostics::EventLog::WriteEntry(
			"BonDriver_StreamService", gcnew System::String(message.c_str()), type);
	}
	catch (...)
	{
	}
}

void CModuleManager::WriteEvent(System::Diagnostics::EventLogEntryType type, System::String^ message) const
{
	try
	{
		System::Diagnostics::EventLog::WriteEntry(
			"BonDriver_StreamService", message, type);
	}
	catch (...)
	{
	}
}

void CModuleManager::WriteEvent(System::Exception^ ex) const
{
	try
	{
		System::Diagnostics::EventLog::WriteEntry(
			"BonDriver_StreamService", ex->ToString(), System::Diagnostics::EventLogEntryType::Error);
	}
	catch (...)
	{
	}
}

int CModuleManager::m_Dummy = 0;


//////////////////////////////////////////////////////////////////////
// \z/
//////////////////////////////////////////////////////////////////////

CBonTuner::CBonTuner()
	: m_StreamService(gcnew Bon::Remoting::RemoteStreamService()),
	m_Buffer(BUFFERSIZE), m_Length(0), m_Position(0), m_SignalLevel(0), m_TuningSpace(0), m_Channel(0)
{
	try
	{
		Bon::IUrl^ url = gcnew Bon::Url(gcnew System::String(m_ModuleManager.GetRemoteUrl().c_str()));
		Bon::Remoting::IRemoteTunerCollection^ tunerCollection =
			m_StreamService->GetTunerCollection(url);

		for (int i = 0; i < tunerCollection->Count; i++)
		{
			Bon::Remoting::IRemoteTuner^ tuner = tunerCollection[i];
			Bon::Remoting::IRemoteTunerStatus^ status = tuner->GetTunerStatus();
			if ((status->TunerName == gcnew System::String(m_ModuleManager.GetTunerName().c_str())) &&
				(status->DeviceID == m_ModuleManager.GetDeviceID()) &&
				(status->StreamUrls->Length > 0))
			{
				pin_ptr<const wchar_t> pDeviceName = PtrToStringChars(status->DeviceName);
				m_DeviceName = pDeviceName;

				m_StreamUrl = status->StreamUrls[0];

				SetTuningSpaces(tuner);
				SetStatus(status);

				m_Tuner = tuner;

				return;
			}
		}

		m_ModuleManager.WriteEvent(System::Diagnostics::EventLogEntryType::Warning,
			System::String::Format("BonDriver: {0}\r\nRemote URL: {1}\r\nCannot found tuner: TunerName={2}, DeviceID={3}",
			System::Reflection::Assembly::GetExecutingAssembly()->FullName, url,
			gcnew System::String(m_ModuleManager.GetTunerName().c_str()), m_ModuleManager.GetDeviceID()));
	}
	catch (System::Exception^ ex)
	{
		m_ModuleManager.WriteEvent(ex);
		m_DeviceName = m_ModuleManager.GetModuleName() + TEXT(" (") + m_ModuleManager.GetRemoteUrl() + TEXT(", Not connected)");
	}
	catch (...)
	{
		m_ModuleManager.WriteEvent(
			System::Diagnostics::EventLogEntryType::Error, "Caught unknown exception: CBonTuner::CBonTuner()");
		m_DeviceName = m_ModuleManager.GetModuleName() + TEXT(" (") + m_ModuleManager.GetRemoteUrl() + TEXT(", Not connected)");
	}
}

CBonTuner::~CBonTuner()
{
	CloseTuner();

	m_StreamService->~IRemoteStreamService();
}

void CBonTuner::Release()
{
	delete this;
}

const BOOL CBonTuner::OpenTuner()
{
	CBlockLock lock(m_Lock);

	CloseTuner();

	try
	{
		Bon::Remoting::IRemoteTuner^ tuner = m_Tuner;
		if (tuner != nullptr)
		{
			m_StreamClient = gcnew Bon::Remoting::StreamClient();
			m_StreamClient->Connect(m_StreamUrl, -1, -1, -1, -1, 10000, 5000, false,
				System::String::Format("{0}: ({1})",
				gcnew System::String(m_ModuleManager.GetModuleName().c_str()),
				System::Reflection::Assembly::GetExecutingAssembly()->FullName));

			m_ModuleManager.WriteEvent(System::Diagnostics::EventLogEntryType::Information,
				System::String::Format("BonDriver: {0}\r\nConnected: {1}",
				System::Reflection::Assembly::GetExecutingAssembly()->FullName, m_StreamUrl));

			return TRUE;
		}
	}
	catch (System::Exception^ ex)
	{
		m_ModuleManager.WriteEvent(ex);
	}
	catch (...)
	{
		m_ModuleManager.WriteEvent(
			System::Diagnostics::EventLogEntryType::Error, "Caught unknown exception: CBonTuner::OpenTuner()");
	}

	CloseTuner();

	return FALSE;
}

void CBonTuner::CloseTuner()
{
	CBlockLock lock(m_Lock);

	Bon::Remoting::IStreamClient^ client = m_StreamClient;
	if (client != nullptr)
	{
		client->Close();
		m_StreamClient = nullptr;
	}
	m_Length = 0;
	m_Position = 0;

	m_SignalLevel = 0;
}

void CBonTuner::SetTuningSpaces(Bon::Remoting::IRemoteTuner^ tuner)
{
	m_TuningSpaces.clear();
	array<Bon::Remoting::IRemoteTuningSpaceInformation^>^ tuningSpaces = tuner->GetTuningSpaces();
	for (int i = 0; i < tuningSpaces->Length; i++)
	{
		m_TuningSpaces.push_back(TuningSpaceType());
		pin_ptr<const wchar_t> pTuningSpaceName = PtrToStringChars(tuningSpaces[i]->Name);
		m_TuningSpaces[i].first = pTuningSpaceName;
		array<Bon::Remoting::IRemoteChannelInformation^>^ channels = tuningSpaces[i]->Channels;
		for (int j = 0; j < channels->Length; j++)
		{
			pin_ptr<const wchar_t> pChannelName = PtrToStringChars(channels[j]->Name);
			m_TuningSpaces[i].second.push_back(pChannelName);
		}
	}
}

void CBonTuner::SetStatus(Bon::Remoting::IRemoteTunerStatus^ status)
{
	m_SignalLevel = status->SignalLevel;
	m_TuningSpace = status->TuningSpace;
	m_Channel = status->Channel;
}

void CBonTuner::UpdateStatus()
{
	Bon::Remoting::IRemoteTuner^ tuner = m_Tuner;
	if (tuner == nullptr)
	{
		return;
	}

	try
	{
		Bon::Remoting::IRemoteTunerStatus^ status = tuner->GetTunerStatus();
		SetStatus(status);
	}
	catch (...)
	{
	}
}

const BOOL CBonTuner::SetChannel(const BYTE bCh)
{
	CBlockLock lock(m_Lock);

	int channel = bCh;
	for (int tuningSpace = 0; tuningSpace < static_cast<int>(m_TuningSpaces.size()); tuningSpace++)
	{
		int channels = static_cast<int>(m_TuningSpaces[tuningSpace].second.size());
		if (channel < channels)
		{
			return SetChannel(tuningSpace, channel);
		}
		channel -= channels;
	}
	return FALSE;
}

const float CBonTuner::GetSignalLevel(void)
{
	CBlockLock lock(m_Lock);

	UpdateStatus();
	return m_SignalLevel;
}

const DWORD CBonTuner::WaitTsStream(const DWORD dwTimeOut)
{
	Bon::Remoting::IStreamClient^ client = m_StreamClient;
	if (client == nullptr)
	{
		return WAIT_ABANDONED;
	}

	if (m_Length > m_Position)
	{
		return WAIT_OBJECT_0;
	}

	m_Length = 0;
	m_Position = 0;
	System::IAsyncResult^ ar = nullptr;
	try
	{
		array<System::Byte>^ buffer = m_Buffer.GetManagedData();
		ar = client->BeginRead(buffer, 0, buffer->Length, nullptr, nullptr);

		bool result = ar->AsyncWaitHandle->WaitOne(dwTimeOut, true);
		if (result == false)
		{
			client->EndRead(ar);
			return WAIT_TIMEOUT;
		}
		m_Length = client->EndRead(ar);
	}
	catch (System::ObjectDisposedException^ ex)
	{
		m_ModuleManager.WriteEvent(ex);
		try
		{
			client->EndRead(ar);
		}
		catch (...)
		{
		}
		return WAIT_ABANDONED;
	}
	catch (System::Net::WebException^ ex)
	{
		m_ModuleManager.WriteEvent(ex);
		try
		{
			client->EndRead(ar);
		}
		catch (...)
		{
		}
		return WAIT_ABANDONED;
	}
	catch (...)
	{
		try
		{
			client->EndRead(ar);
		}
		catch (...)
		{
		}
		return WAIT_FAILED;
	}

	return WAIT_OBJECT_0;
}

const DWORD CBonTuner::GetReadyCount()
{
	WaitTsStream(0);
	return m_Length - m_Position;
}

const BOOL CBonTuner::GetTsStream(BYTE *pDst, DWORD *pdwSize, DWORD *pdwRemain)
{
	if ((m_Length - m_Position) == 0)
	{
		if (WaitTsStream(INFINITE) != WAIT_OBJECT_0)
		{
			return FALSE;
		}
		if ((m_Length - m_Position) == 0)
		{
			return FALSE;
		}
	}

	DWORD size = m_Length - m_Position;
	::memcpy(pDst, m_Buffer.GetData() + m_Position, size);
	m_Position += size;
	*pdwSize = size;
	*pdwRemain = m_Length - m_Position;
	return TRUE;
}

const BOOL CBonTuner::GetTsStream(BYTE **ppDst, DWORD *pdwSize, DWORD *pdwRemain)
{
	if ((m_Length - m_Position) == 0)
	{
		if (WaitTsStream(INFINITE) != WAIT_OBJECT_0)
		{
			return FALSE;
		}
		if ((m_Length - m_Position) == 0)
		{
			return FALSE;
		}
	}

	DWORD size = m_Length - m_Position;
	*ppDst = const_cast<BYTE *>(m_Buffer.GetData() + m_Position);
	m_Position += size;
	*pdwSize = size;
	*pdwRemain = m_Length - m_Position;
	return TRUE;
}

void CBonTuner::PurgeTsStream()
{
	CBlockLock lock(m_Lock);

	Bon::Remoting::IRemoteTuner^ tuner = m_Tuner;
	if (tuner == nullptr)
	{
		return;
	}

	Bon::Remoting::IStreamClient^ client = m_StreamClient;
	if (client == nullptr)
	{
		return;
	}

	try
	{
		tuner->PurgeStream(client->LocalEndPoint);
	}
	catch (System::Exception^ ex)
	{
		m_ModuleManager.WriteEvent(ex);
	}
	catch (...)
	{
		m_ModuleManager.WriteEvent(
			System::Diagnostics::EventLogEntryType::Error, "Caught unknown exception: CBonTuner::PurgeTsStream()");
	}
}

LPCTSTR CBonTuner::GetTunerName(void)
{
	CBlockLock lock(m_Lock);

	UpdateStatus();
	return m_DeviceName.c_str();
}

const BOOL CBonTuner::IsTunerOpening(void)
{
	return FALSE;
}

LPCTSTR CBonTuner::EnumTuningSpace(const DWORD dwSpace)
{
	if (dwSpace >= m_TuningSpaces.size())
	{
		return 0;
	}
	return m_TuningSpaces[dwSpace].first.c_str();
}

LPCTSTR CBonTuner::EnumChannelName(const DWORD dwSpace, const DWORD dwChannel)
{
	if (dwSpace >= m_TuningSpaces.size())
	{
		return 0;
	}
	const std::vector<String> &channels = m_TuningSpaces[dwSpace].second;
	if (dwChannel >= channels.size())
	{
		return 0;
	}
	return channels[dwChannel].c_str();
}

const BOOL CBonTuner::SetChannel(const DWORD dwSpace, const DWORD dwChannel)
{
	CBlockLock lock(m_Lock);

	Bon::Remoting::IRemoteTuner^ tuner = m_Tuner;
	if (tuner == nullptr)
	{
		return FALSE;
	}

	try
	{
		tuner->SetChannel(dwSpace, dwChannel, -1);
	}
	catch (System::Exception^ ex)
	{
		m_ModuleManager.WriteEvent(ex);
		return FALSE;
	}
	catch (...)
	{
		m_ModuleManager.WriteEvent(
			System::Diagnostics::EventLogEntryType::Error, "Caught unknown exception: CBonTuner::SetChannel()");
		return FALSE;
	}
	return TRUE;
}

const DWORD CBonTuner::GetCurSpace(void)
{
	CBlockLock lock(m_Lock);

	UpdateStatus();
	return m_TuningSpace;
}

const DWORD CBonTuner::GetCurChannel(void)
{
	CBlockLock lock(m_Lock);

	UpdateStatus();
	return m_Channel;
}

const DWORD CBonTuner::GetTotalDeviceNum(void)
{
	return 1;
}

const DWORD CBonTuner::GetActiveDeviceNum(void)
{
	// ނÃC^[tFCX͖Ǝv...
	return 0;
}

const BOOL CBonTuner::SetLnbPower(const BOOL bEnable)
{
	CBlockLock lock(m_Lock);

	Bon::Remoting::IRemoteTuner^ tuner = m_Tuner;
	if (tuner == nullptr)
	{
		return FALSE;
	}

	try
	{
		m_Tuner->SetLnbPower(bEnable == TRUE);
	}
	catch (System::Exception^ ex)
	{
		m_ModuleManager.WriteEvent(ex);
		return FALSE;
	}
	catch (...)
	{
		m_ModuleManager.WriteEvent(
			System::Diagnostics::EventLogEntryType::Error, "Caught unknown exception: CBonTuner::SetLnbPower()");
		return FALSE;
	}
	return TRUE;
}


//////////////////////////////////////////////////////////////////////
// CX^X֐
//////////////////////////////////////////////////////////////////////

#pragma warning(disable:4273)

extern "C" __declspec(dllexport) IBonDriver *CreateBonDriver()
{
	return new CBonTuner();
}

#pragma warning(default:4273)


//////////////////////////////////////////////////////////////////////
// CXg[NX
//////////////////////////////////////////////////////////////////////

namespace Bon
{
	[System::ComponentModel::DesignTimeVisible(false)]
	[System::ComponentModel::RunInstaller(true)]
	public ref class BonDriverInstaller sealed : public System::Configuration::Install::Installer
	{
	public:
		BonDriverInstaller(void)
		{
			System::Diagnostics::EventLogInstaller^ eventLog = gcnew System::Diagnostics::EventLogInstaller();
			eventLog->Log = "Application";
			eventLog->Source = "BonDriver_StreamService";

			System::Configuration::Install::Installer::Installers->Add(eventLog);
		}
	};
}
