//////////////////////////////////////////////////////////////////////////
// BonSDK Stream service connection library (BonDriver interface)
// Copyright (c) http://2sen.dip.jp/dtv/
//
// This is part of BonSDK.
// BonSDK comes with ABSOLUTELY NO WARRANTY;
// for details see 'GNU GENERAL PUBLIC LICENSE 2.0'.

// BonTuner.h: CBonTuner NX̃C^[tFCX
//
//////////////////////////////////////////////////////////////////////

#pragma once

#include "IBonDriver3.h"


//////////////////////////////////////////////////////////////////////
// X}[gbN
//////////////////////////////////////////////////////////////////////

class CSmartLock
{
private:
	CRITICAL_SECTION m_Lock;

	CSmartLock(const CSmartLock &);
	void operator =(const CSmartLock &);

public:
	CSmartLock();
	~CSmartLock();
	void Enter();
	void Leave();
};


//////////////////////////////////////////////////////////////////////
// }l[Wobt@
//////////////////////////////////////////////////////////////////////

class CManagedBuffer
{
private:
	const int bufferSize_;
	void *handle_;
	BYTE *pBuffer_;

	CManagedBuffer(const CManagedBuffer &);
	void operator =(const CManagedBuffer &);

public:
	CManagedBuffer(int bufferSize);
	~CManagedBuffer();

	const DWORD GetSize() const;
	const BYTE *GetData() const;

	array<System::Byte>^ GetManagedData();
};


//////////////////////////////////////////////////////////////////////
// W[}l[W
//////////////////////////////////////////////////////////////////////

class CModuleManager
{
private:
	static int m_Dummy;
	String m_ModuleName;
	String m_ConfigrationPath;

	const String GetProfileString(LPCTSTR name, LPCTSTR defaultValue) const;
	int GetProfileInt32(LPCTSTR name, int defaultValue) const;

	CModuleManager(const CModuleManager &);
	void operator =(const CModuleManager &);

public:
	CModuleManager();

	const String GetModuleName() const;
	const String GetRemoteUrl() const;
	const String GetTunerName() const;
	int GetDeviceID() const;

	void WriteEvent(System::Diagnostics::EventLogEntryType type, const String &message) const;
	void WriteEvent(System::Diagnostics::EventLogEntryType type, System::String^ message) const;
	void WriteEvent(System::Exception^ ex) const;
};


//////////////////////////////////////////////////////////////////////
// `[i[
//////////////////////////////////////////////////////////////////////

class CBonTuner : public IBonDriver3
{
public:
	CBonTuner();
	virtual ~CBonTuner();

// IBonDriver
	const BOOL OpenTuner(void);
	void CloseTuner(void);

	const BOOL SetChannel(const BYTE bCh);
	const float GetSignalLevel(void);

	const DWORD WaitTsStream(const DWORD dwTimeOut = 0);
	const DWORD GetReadyCount(void);

	const BOOL GetTsStream(BYTE *pDst, DWORD *pdwSize, DWORD *pdwRemain);
	const BOOL GetTsStream(BYTE **ppDst, DWORD *pdwSize, DWORD *pdwRemain);

	void PurgeTsStream(void);

	void Release(void);

// IBonDriver2(b)
	LPCTSTR GetTunerName(void);

	const BOOL IsTunerOpening(void);
	
	LPCTSTR EnumTuningSpace(const DWORD dwSpace);
	LPCTSTR EnumChannelName(const DWORD dwSpace, const DWORD dwChannel);

	const BOOL SetChannel(const DWORD dwSpace, const DWORD dwChannel);
	
	const DWORD GetCurSpace(void);
	const DWORD GetCurChannel(void);

// IBonDriver3(b)
	const DWORD GetTotalDeviceNum(void);
	const DWORD GetActiveDeviceNum(void);
	const BOOL SetLnbPower(const BOOL bEnable);

private:
	typedef std::pair<String, std::vector<String> > TuningSpaceType;
	CSmartLock m_Lock;
	CModuleManager m_ModuleManager;

	gcroot<Bon::Remoting::IRemoteStreamService^> m_StreamService;
	gcroot<Bon::Remoting::IRemoteTuner^> m_Tuner;
	gcroot<Bon::Remoting::IStreamClient^> m_StreamClient;
	CManagedBuffer m_Buffer;
	DWORD m_Length, m_Position;

	String m_DeviceName;
	gcroot<Bon::IUrl^> m_StreamUrl;
	float m_SignalLevel;
	int m_TuningSpace;
	int m_Channel;
	std::vector<TuningSpaceType> m_TuningSpaces;

	void SetTuningSpaces(Bon::Remoting::IRemoteTuner^ tuner);
	void SetStatus(Bon::Remoting::IRemoteTunerStatus^ status);
	void UpdateStatus();

	CBonTuner(const CBonTuner &);
	void operator =(const CBonTuner &);
};
