//////////////////////////////////////////////////////////////////////////
// BonSDK Friio (White/Black) tuner driver library
// Copyright (c) http://2sen.dip.jp/dtv/
//
// This is part of BonSDK.
// BonSDK comes with ABSOLUTELY NO WARRANTY;
// for details see 'GNU GENERAL PUBLIC LICENSE 2.0'.

// BonTunerBase.h: Friio`[i[NX
//
/////////////////////////////////////////////////////////////////////////////

#pragma once

#include <Vector>
#include <Queue>
#include "BonEnumerator.h"


/////////////////////////////////////////////////////////////////////////////
// Friiop@TS`[i HALC^tF[XNX
/////////////////////////////////////////////////////////////////////////////

#define TSRECVSIZE 48128UL			// TSf[^MTCY

class CBonTunerBase :	public CBonObject,
						public IHalDevice,
						public IHalTsTuner
{
public:
// IHalDevice
	virtual const BONGUID GetDeviceType(void) const;
	virtual const CBonString GetDeviceName() const = 0;
	virtual const DWORD GetTotalDeviceNum(void) const = 0;
	virtual const DWORD GetActiveDeviceNum(void) const = 0;

// IHalTsTuner
	virtual const DWORD OpenTuner(void);
	virtual void CloseTuner(void);
	
	virtual const CBonString EnumTuningSpace(const DWORD dwSpace) const = 0;
	virtual const CBonString EnumChannelName(const DWORD dwSpace, const DWORD dwChannel) const = 0;
	
	virtual const DWORD GetDeviceID(void) const;
	virtual const DWORD GetCurSpace(void) const;
	virtual const DWORD GetCurChannel(void) const;

	virtual const bool SetChannel(const DWORD dwSpace, const DWORD dwChannel);
	virtual const bool SetLnbPower(const bool bEnable);
	virtual const float GetSignalLevel(void) const = 0;

	virtual const bool SetOutputDecoder(IMediaDecoder *pDecoder);

	virtual ~CBonTunerBase(void);
		
	static void SetModuleHandle(const HINSTANCE hModule);

protected:
// CBonTunerBase
	CBonTunerBase(IBonObject *pOwner, int initSeqID, int endSeqID, LPCTSTR pName);

	virtual const bool OnOpenTuner(void) = 0;
	virtual const bool OnSetChannel(const DWORD dwSpace, const DWORD dwChannel) = 0;

	const CBonString GetStandardTuningSpaceName(const IStandardTuningSpace::STANDARD_TUNING_SPACES tuningSpace) const;
	const CBonString GetStandardChannelName(const IStandardTuningSpace::STANDARD_TUNING_SPACES tuningSpace, const DWORD dwChannel) const;

	const int SendIoRequest(const DWORD dwIoCode, const BYTE *pTxdData, const DWORD dwTxdSize, BYTE *pRxdData, const DWORD dwRxdSize) const;
	const bool SendFixedRequest(const UINT nID) const;
	static const HINSTANCE GetModuleInstance();

	// ŒI/ONGXgwb_
	#pragma pack(1)
	struct FIXED_REQ_HEAD
	{
		DWORD dwIoCode;		// I/ORg[R[h
		WORD wTxdSize;		// Mf[^TCY
		WORD wRxdSize;		// Mf[^TCY
	};
	#pragma pack()

	mutable CBonEnumerator m_BonEnumerator;

private:
	const bool AllocateBuffer(const DWORD dwBufferNum);
	void ReleaseBuffer(void);

	void IoReqThread(CSmartThread<CBonTunerBase> &thread, const ISignal &bKillSignal, const ULONG_PTR param);

	const bool PushIoRequest(void);
	const bool PopIoRequest(const ISignal &bKillSignal);

	// I/ONGXgL[f[^
	class ASYNC_REQ_DATA;

	LPCTSTR m_pName;

	IStandardTuningSpace *m_pStandardTuningSpace;
	IMediaDecoder *m_pOutputDecoder;

	HANDLE m_hFriioDriver;

	DWORD m_dwBufferSize;
	DWORD m_dwBufferPosition;
	ASYNC_REQ_DATA *m_pAsyncBuffer;
	std::queue<ASYNC_REQ_DATA *> m_AsyncReqQue;
	
	CSmartThread<CBonTunerBase> m_IoReqThread;
	
	DWORD m_dwDeviceIndex;
	DWORD m_dwCurSpace;
	DWORD m_dwCurChannel;

	const int m_initSeqID;
	const int m_endSeqID;

	static HINSTANCE m_hModule;
};
