//////////////////////////////////////////////////////////////////////////
// BonSDK Media decoder library
// Copyright (c) http://2sen.dip.jp/dtv/
//
// This is part of BonSDK.
// BonSDK comes with ABSOLUTELY NO WARRANTY;
// for details see 'GNU GENERAL PUBLIC LICENSE 2.0'.

// FileWriter.cpp: t@CC^fR[_
//
/////////////////////////////////////////////////////////////////////////////

#include "StdAfx.h"
#include "BonSDK.h"
#include "FileWriter.h"


/////////////////////////////////////////////////////////////////////////////
// t@C[J萔ݒ
/////////////////////////////////////////////////////////////////////////////

// ftHg̃obt@TCY
#define DEF_BUFFSIZE	0x100000UL							// 1MB
#define DEF_BUFFNUM		(DEF_BUFFSIZE / TS_PACKET_SIZE)		// 1MB 


/////////////////////////////////////////////////////////////////////////////
// t@CC^fR[_
/////////////////////////////////////////////////////////////////////////////

const bool CFileWriter::OnInputMedia(const IMediaData *pMediaData, const DWORD dwInputIndex)
{
	if(!pMediaData || (dwInputIndex >= GetInputNum()) || !m_pFile)return false;

	// t@C
	return (m_WriteThread.IsRunning())? WriteAsync(pMediaData) : WriteSync(pMediaData);
}

const bool CFileWriter::OpenFile(LPCTSTR lpszFileName, const bool bAsyncWrite, const DWORD dwBuffSize)
{
	CloseFile();

	try{
		CBlockLock AutoLock(&m_Lock);

		// obt@TCYvZ
		DWORD dwWriteSize;
		DWORD dwWriteNum;
		
		if(!bAsyncWrite){
			// 
			dwWriteSize = (dwBuffSize)? dwBuffSize : DEF_BUFFSIZE;
			dwWriteNum	= 0UL;
			}
		else{
			// 񓯊
			dwWriteSize = DEF_BUFFSIZE;
			dwWriteNum = (dwBuffSize)? dwBuffSize : DEF_BUFFNUM;
			}
	
		// X}[gt@CCX^X
		if(!(m_pFile = ::BON_SAFE_CREATE<ISmartFile *>(TEXT("CSmartFile"))))throw __LINE__;

		// t@CI[v
		if(!m_pFile->Open(lpszFileName, ISmartFile::MF_WRITE, dwWriteSize))throw __LINE__;

		// fBAv[m
		if(dwWriteNum){
			// fBAv[TCYTCY
			m_MediaPool.resize(dwWriteNum);
			if(m_MediaPool.size() < dwWriteNum)throw __LINE__;
			::ZeroMemory(&m_MediaPool[0], sizeof(IMediaData *) * dwWriteNum);
			
			// fBAf[^CX^Xm
			for(DWORD dwIndex = 0UL ; dwIndex < dwWriteNum ; dwIndex++){
				m_MediaPool[dwIndex] = ::BON_SAFE_CREATE<IMediaData *>(TEXT("CMediaData"));
				if(!m_MediaPool[dwIndex])throw __LINE__;
				}
			
			// fBAv[擪Ce[^Zbg
			m_itFreeMedia = m_MediaPool.begin();
			}
		}
	catch(...){
		// G[
		CloseFile();
		return false;
		}
		
	return true;
}

bool CFileWriter::CloseFile(void)
{
	CBlockLock AutoLock(&m_Lock);

	// t@CN[Y
	if(m_pFile)m_pFile->Close();

	// X}[gt@CCX^XJ
	BON_SAFE_RELEASE(m_pFile);

	// fBAL[NA
	while(!m_MediaQue.empty())m_MediaQue.pop();

	// fBAf[^CX^XJ
	for(DWORD dwIndex = 0UL ; dwIndex < m_MediaPool.size() ; dwIndex++){
		BON_SAFE_RELEASE(m_MediaPool[dwIndex]);		
		}
	
	// fBAv[J
	m_MediaPool.clear();

	return true;
}

const ULONGLONG CFileWriter::GetFileLength(void)
{
	// t@CTCYԂ
	return (m_pFile)? m_pFile->GetLength() : 0ULL;
}

CFileWriter::CFileWriter(IBonObject *pOwner)
	: CMediaDecoder(pOwner, 1UL, 0UL)
	, m_pFile(NULL)
	, m_WriteThread(new CFlagSignal(1))
{
	// Ȃ
}

CFileWriter::~CFileWriter(void)
{
	// t@C
	CloseFile();
}

const bool CFileWriter::OnPlay(void)
{
	OnStop();

	// fBAL[NA
	while(!m_MediaQue.empty())m_MediaQue.pop();
	
	// fBAv[擪Ce[^Zbg
	m_itFreeMedia = m_MediaPool.begin();

	// XbhJn(TXyhԂŊJn)
	return (m_MediaPool.size())? m_WriteThread.StartThread(TEXT("CFileWriter"), this, &CFileWriter::WriteThread, NULL, false) : true;
}

const bool CFileWriter::OnStop(void)
{
	// XbhI
	m_WriteThread.EndThread(true);

	return true;
}

const bool CFileWriter::WriteSync(const IMediaData *pMediaData)
{
	// f[^擾
	const BYTE * const pData = pMediaData->GetData();
	const DWORD dwSize = pMediaData->GetSize();
	if(!pData || !dwSize)return true;

	// t@C
	if(m_pFile->WriteData(pData, dwSize) != dwSize){
		CloseFile();
		return false;
		}
	
	return true;
}

const bool CFileWriter::WriteAsync(const IMediaData *pMediaData)
{
	DWORD dwIdleNum;

	// f[^`FbN
	const BYTE * const pData = pMediaData->GetData();
	const DWORD dwSize = pMediaData->GetSize();
	if(!pData || !dwSize)return true;

	// AChL[擾
	{
		CBlockLock AutoLock(&m_Lock);
		dwIdleNum = m_MediaPool.size() - m_MediaQue.size();
	}

	// L[t̏ꍇ̏
	if(!dwIdleNum){
		// Cxgʒm
		if(SendDecoderEvent(EID_BUFF_OVERFLOW)){
			// L[󂭂܂ŃubN
			do{
				::Sleep(1UL);

				// |[O
				CBlockLock AutoLock(&m_Lock);
				dwIdleNum = m_MediaPool.size() - m_MediaQue.size();
				}
			while(!dwIdleNum);			
			}
		else{
			// ftHg̏AÂf[^̂Ă
			CBlockLock AutoLock(&m_Lock);
			m_MediaQue.pop();		
			}		
		}

	// f[^Rs[
	if((*m_itFreeMedia)->CopyData(pMediaData) != dwSize)return false;

	// fBAv[ɋ󂫂ƂAL[ɓ
	CBlockLock AutoLock(&m_Lock);
	m_MediaQue.push(*m_itFreeMedia);
	if(++m_itFreeMedia == m_MediaPool.end())m_itFreeMedia = m_MediaPool.begin();

	// ݃L[W[
	m_WriteThread.ResumeThread();

	return true;
}

void CFileWriter::WriteThread(CSmartThread<CFileWriter> &thread, const ISignal &bKillSignal, const ULONG_PTR param)
{
	const IMediaData *pMediaData;

	while(!bKillSignal){
		// t@C
		while(1){
			{
				// L[`FbN
				CBlockLock AutoLock(&m_Lock);
				pMediaData = (!m_MediaQue.empty())? m_MediaQue.front() : NULL;
				if(!pMediaData)break;
			}

			// t@C
			WriteSync(pMediaData);

			// L[o
			CBlockLock AutoLock(&m_Lock);
			m_MediaQue.pop();
			}

		// L[Ƀf[^܂ŃTXyh
		if(!bKillSignal)thread.SuspendThread();
		}
}
