//////////////////////////////////////////////////////////////////////////
// BonSDK Media decoder library
// Copyright (c) http://2sen.dip.jp/dtv/
//
// This is part of BonSDK.
// BonSDK comes with ABSOLUTELY NO WARRANTY;
// for details see 'GNU GENERAL PUBLIC LICENSE 2.0'.

// ProgAnalyzer.cpp: vOAiCUfR[_
//
/////////////////////////////////////////////////////////////////////////////

#include "StdAfx.h"
#include "BonSDK.h"
#include "TsTable.h"
#include "ProgAnalyzer.h"
#include <TChar.h>


/////////////////////////////////////////////////////////////////////////////
// vOAiCUfR[_
/////////////////////////////////////////////////////////////////////////////

const bool CProgAnalyzer::OnInputMedia(const IMediaData *pMediaData, const DWORD dwInputIndex)
{
	// ̓p[^`FbN
	const ITsPacket * const pTspacket = dynamic_cast<const ITsPacket *>(pMediaData);
	if(!pTspacket)return false;

	CBlockLock lock(m_Lock);

	// PID}bvɓ
	m_TsPidMapper.StorePacket(pTspacket);

	// ʃfR[_Ƀf[^o
	OutputMedia(pMediaData);

	return true;
}

const WORD CProgAnalyzer::GetNetworkID(void) const
{
	// IWilbg[NIDԂ
	return m_ProgContext.wNetworkID;
}

const WORD CProgAnalyzer::GetTsID(void) const
{
	// TS IDԂ
	return m_ProgContext.wTsID;
}

const WORD CProgAnalyzer::GetServiceIDFromPID(const WORD wPID) const
{
	CBlockLock lock(m_Lock);

	// T[rXIDԂ
	for(ProgramList::const_iterator itProgram = m_ProgContext.ProgList.begin() ; itProgram != m_ProgContext.ProgList.end() ; itProgram++){
		if(itProgram->bAvailable){
			if((wPID == itProgram->wPmtPID) || (wPID == itProgram->wPcrPID) ||
				(wPID == itProgram->wVideoPID) || (wPID == itProgram->wAudioPID)){
				return itProgram->wServiceID;
			}
		}
	}
		
	return 0x0000U;
}

const WORD CProgAnalyzer::GetPmtPIDFromServiceID(const WORD wServiceID) const
{
	CBlockLock lock(m_Lock);

	// Pmt PIDԂ
	for(ProgramList::const_iterator itProgram = m_ProgContext.ProgList.begin() ; itProgram != m_ProgContext.ProgList.end() ; itProgram++){
		if(itProgram->bAvailable){
			if(wServiceID == itProgram->wServiceID) {
				return itProgram->wPmtPID;
			}
		}
	}
		
	return 0x0000U;
}

const DWORD CProgAnalyzer::GetServiceNum(void) const
{
	CBlockLock lock(m_Lock);

	// LȃvOԂ
	DWORD dwNum = 0UL;
	
	for(ProgramList::const_iterator itProgram = m_ProgContext.ProgList.begin() ; itProgram != m_ProgContext.ProgList.end() ; itProgram++){
		if(itProgram->bAvailable)dwNum++;
		}
		
	return dwNum;
}

const WORD CProgAnalyzer::GetServiceID(const DWORD dwIndex) const
{
	CBlockLock lock(m_Lock);

	// T[rXIDԂ
	DWORD dwPos = 0UL;
	
	for(ProgramList::const_iterator itProgram = m_ProgContext.ProgList.begin() ; itProgram != m_ProgContext.ProgList.end() ; itProgram++){
		if(itProgram->bAvailable){
			if(dwPos++ == dwIndex)return itProgram->wServiceID;
			}
		}
		
	return 0x0000U;
}

const WORD CProgAnalyzer::GetProgramVideoPID(const WORD wServiceID) const
{
	CBlockLock lock(m_Lock);

	// fPIDԂ
	const PROGRAM_ITEM * pItem = FindProgramItem(wServiceID);
	return (pItem)? pItem->wVideoPID : TS_INVALID_PID;
}

const WORD CProgAnalyzer::GetProgramAudioPID(const WORD wServiceID) const
{
	CBlockLock lock(m_Lock);

	// PIDԂ
	const PROGRAM_ITEM * pItem = FindProgramItem(wServiceID);
	return (pItem)? pItem->wAudioPID : TS_INVALID_PID;
}

const WORD CProgAnalyzer::GetProgramPcrPID(const WORD wServiceID) const
{
	// PCRPIDԂ
	const PROGRAM_ITEM * pItem = FindProgramItem(wServiceID);
	return (pItem)? pItem->wPcrPID : TS_INVALID_PID;
}

const CBonString CProgAnalyzer::GetNetworkName() const
{
	CBlockLock lock(m_Lock);

	// lbg[NԂ
	return m_ProgContext.NetworkName;
}

const CBonString CProgAnalyzer::GetServiceName(const WORD wServiceID) const
{
	CBlockLock lock(m_Lock);

	// T[rXԂ
	const PROGRAM_ITEM * pItem = FindProgramItem(wServiceID);
	if(!pItem)return CBonString::GetEmpty();

	return pItem->ServiceName;
}

const bool CProgAnalyzer::GetStreamTime(SYSTEMTIME *pStreamTime) const
{
	CBlockLock lock(m_Lock);

	// Xg[̎ԂԂ
	ITotTable *pTotTable = dynamic_cast<ITotTable *>(m_TsPidMapper.GetMapTarget(0x0014U));
	::BON_ASSERT(pTotTable != NULL);
	
	CTsTime StreamTime = pTotTable->GetDateTime();
	if(StreamTime.IsEmpty())return false;

	// ԂZbg	
	if(pStreamTime)*pStreamTime = StreamTime;
	
	return true;
}

const WORD CProgAnalyzer::GetCurrentServiceID(void) const
{
	// ݂̃T[rXIDԂ
	return m_ProgContext.wCurServiceID;
}

const bool CProgAnalyzer::SetCurrentServiceID(const WORD wServiceID)
{
	CBlockLock lock(m_Lock);

	// ݂̃T[rXIDݒ肷
	if(wServiceID){
		// wT[rXIDɕύX
		if(FindProgramItem(wServiceID))return false;
		m_ProgContext.wCurServiceID = wServiceID;
		}
	else{
		// ftHgvOIDɕύX(PAT̍ŏ̃T[rXID)
		if(!m_ProgContext.ProgList.size())return false;
		m_ProgContext.wCurServiceID = m_ProgContext.ProgList[0].wServiceID;
		}

	return true;
}

const IEpgItem * CProgAnalyzer::GetCurrentEpgInfo(const WORD wServiceID) const
{
	CBlockLock lock(m_Lock);

	// ݂EPGԂ
	const PROGRAM_ITEM * pItem = FindProgramItem(wServiceID);
	return (pItem)? pItem->pCurEpgItem : NULL;
}

const IEpgItem * CProgAnalyzer::GetNextEpgInfo(const WORD wServiceID) const
{
	CBlockLock lock(m_Lock);

	// EPGԂ
	const PROGRAM_ITEM * pItem = FindProgramItem(wServiceID);
	return (pItem)? pItem->pNextEpgItem : NULL;
}

CProgAnalyzer::CProgAnalyzer(IBonObject *pOwner, IMediaDataHeap *pHeap)
	: CMediaDecoder(pOwner, 1UL, 1UL, pHeap)
	, m_TsPidMapper(NULL)
{
	// PATe[uPID}bvɒǉ
	m_TsPidMapper.MapPid(0x0000U, dynamic_cast<ITsPidMapTarget *>(CPatTable::CreateInstance(dynamic_cast<IPatTable::IHandler *>(this))));

	// NITe[uPID}bvɒǉ
	m_TsPidMapper.MapPid(0x0010U, dynamic_cast<ITsPidMapTarget *>(CNitTable::CreateInstance(dynamic_cast<INitTable::IHandler *>(this))));

	// SDTe[uPID}bvɒǉ
	m_TsPidMapper.MapPid(0x0011U, dynamic_cast<ITsPidMapTarget *>(CSdtTable::CreateInstance(dynamic_cast<ISdtTable::IHandler *>(this))));

	// EITe[uPID}bvɒǉ
	m_TsPidMapper.MapPid(0x0012U, dynamic_cast<ITsPidMapTarget *>(CEitTable::CreateInstance(dynamic_cast<IEitTable::IHandler *>(this))));

	// TOTe[uPID}bvɒǉ
	m_TsPidMapper.MapPid(0x0014U, dynamic_cast<ITsPidMapTarget *>(CTotTable::CreateInstance(dynamic_cast<ITotTable::IHandler *>(this))));

	// ԃZbg
	OnReset();
}

CProgAnalyzer::~CProgAnalyzer(void)
{
	OnReset();
}

const bool CProgAnalyzer::OnPlay(void)
{
	return OnReset();
}

const bool CProgAnalyzer::OnReset(void)
{
	CBlockLock lock(m_Lock);

	// ReLXgZbg
	m_ProgContext.wNetworkID = 0x0000U;
	m_ProgContext.wTsID = 0x0000U;
	m_ProgContext.wCurServiceID = 0x0000U;
	m_ProgContext.NetworkName.Clear();

	for(ProgramList::iterator itProgram = m_ProgContext.ProgList.begin() ; itProgram != m_ProgContext.ProgList.end() ; itProgram++){
		BON_SAFE_RELEASE(itProgram->pCurEpgItem);
		BON_SAFE_RELEASE(itProgram->pNextEpgItem);
		}

	m_ProgContext.ProgList.clear();

	// PID}bvZbg
	ResetPidMap();

	// PATZbg
	m_TsPidMapper.ResetPid(0x0000U);

	return true;
}

void CProgAnalyzer::OnPatTable(const IPatTable *pPatTable)
{
	CBlockLock lock(m_Lock);

	// TS ID̕ω`FbN
	if(m_ProgContext.wTsID != pPatTable->GetTsID()){
		// TS IDωƂ͏ԂZbg
		ResetPidMap();
		m_ProgContext.wTsID = pPatTable->GetTsID();
		}

	// PMTe[u}bv
	for(DWORD dwIndex = 0UL ; dwIndex < pPatTable->GetProgramNum() ; dwIndex++){
		m_TsPidMapper.MapPid(pPatTable->GetPmtPID(dwIndex), dynamic_cast<ITsPidMapTarget *>(CPmtTable::CreateInstance(dynamic_cast<IPmtTable::IHandler *>(this))));
		}

	// ݂ȂT[rXID폜
	for(ProgramList::iterator itProgram = m_ProgContext.ProgList.begin() ; itProgram != m_ProgContext.ProgList.end() ; ){

		// PAT
		bool bFound = false;
	
		for(DWORD dwIndex = 0UL ; (dwIndex < pPatTable->GetProgramNum()) && !bFound; dwIndex++){
			if(itProgram->wServiceID == pPatTable->GetServiceID(dwIndex)){
				bFound = true;
				}			
			}

		if(!bFound){
			// vOXg폜
			BON_SAFE_RELEASE(itProgram->pCurEpgItem);
			BON_SAFE_RELEASE(itProgram->pNextEpgItem);

			m_ProgContext.ProgList.erase(itProgram);
			itProgram = m_ProgContext.ProgList.begin();
			}
		else{
			itProgram++;
			}
		}

	// VK̃T[rXIDǉ
	for(DWORD dwIndex = 0UL ; dwIndex < pPatTable->GetProgramNum() ; dwIndex++){
		if(FindProgramItem(pPatTable->GetServiceID(dwIndex)))continue;

		// T[rXIDɂȂ悤ɑ}
		ProgramList::iterator itProgram = m_ProgContext.ProgList.begin();

		for( ; itProgram != m_ProgContext.ProgList.end() ; itProgram++){
			if(itProgram->wServiceID > pPatTable->GetServiceID(dwIndex))break;
			}
		
		const PROGRAM_ITEM NewItem =
		{
			false,														// bAvailable
			pPatTable->GetPmtPID(dwIndex),								// wPmtPID
			pPatTable->GetServiceID(dwIndex),							// wServiceID
			TS_INVALID_PID,												// wVideoPID
			TS_INVALID_PID,												// wAudioPID
			TS_INVALID_PID,												// wPcrPID
			TEXT(""),													// szServiceName[]
			dynamic_cast<IEpgItem *>(CEpgItem::CreateInstance(NULL)),	// pCurEpgItem
			dynamic_cast<IEpgItem *>(CEpgItem::CreateInstance(NULL))	// pNextEpgItem
		};

		m_ProgContext.ProgList.insert(itProgram, NewItem);
		}

	// ݂̃T[rXID̕ύX𔻒肷
	if(!FindProgramItem(m_ProgContext.wCurServiceID)){
		// VK̃T[rXIDɕύX
		m_ProgContext.wCurServiceID = (m_ProgContext.ProgList.size())? m_ProgContext.ProgList[0].wServiceID : 0x0000U;
		}
}

void CProgAnalyzer::OnPmtTable(const IPmtTable *pPmtTable)
{
	CBlockLock lock(m_Lock);

	// vOXV
	PROGRAM_ITEM *pProgItem = FindProgramItem(pPmtTable->GetServiceID());
	if(!pProgItem)return;

	// PCRPIDXV
	pProgItem->wPcrPID = pPmtTable->GetPcrPID();

	// f/PIDXV
	for(DWORD dwIndex = 0UL ; dwIndex < pPmtTable->GetEsNum() ; dwIndex++){
		switch(pPmtTable->GetStreamTypeID(dwIndex)){
			case 0x02U :	// ITU-TH.262|ISO/IEC 13818-2f
				pProgItem->wVideoPID = pPmtTable->GetEsPID(dwIndex);
				break;
				
			case 0x0FU :	// ISO/IEC 13818-7iADTSgX|[g\j
				pProgItem->wAudioPID = pPmtTable->GetEsPID(dwIndex);
				break;
			}
		}

	// LtOXV
	if((pProgItem->wVideoPID != TS_INVALID_PID) && (pProgItem->wAudioPID != TS_INVALID_PID)){
		// MPEG2fAACvOLƂ
		pProgItem->bAvailable = true;
		}
}

void CProgAnalyzer::OnNitTable(const INitTable *pNitTable)
{
	CBlockLock lock(m_Lock);

	// lbg[NIDXV
	m_ProgContext.wNetworkID = pNitTable->GetNetworkID();

	// lbg[NXV
	const INetworkNameDesc *pNetworkNameDesc = dynamic_cast<const INetworkNameDesc *>(pNitTable->GetNetworkDesc()->GetDescByTag(INetworkNameDesc::DESC_TAG));
	if(pNetworkNameDesc)m_ProgContext.NetworkName = pNetworkNameDesc->GetNetworkName();
}

void CProgAnalyzer::OnSdtTable(const ISdtTable *pSdtTable)
{
	CBlockLock lock(m_Lock);

	// T[rXXV
	for(DWORD dwIndex = 0UL ; dwIndex < pSdtTable->GetServiceNum() ; dwIndex++){
		const WORD wServiceID = pSdtTable->GetServiceID(dwIndex);
		PROGRAM_ITEM *pProgItem = FindProgramItem(wServiceID);
		if(!pProgItem)continue;

		// T[rXXV	
		const IServiceDesc *pServiceDesc = dynamic_cast<const IServiceDesc *>(pSdtTable->GetServiceDesc(wServiceID)->GetDescByTag(IServiceDesc::DESC_TAG));
		if(!pServiceDesc)continue;

		pProgItem->ServiceName = pServiceDesc->GetServiceName();
		}
}

void CProgAnalyzer::OnTotTable(const ITotTable *pTotTable)
{
	// Ȃ
}

void CProgAnalyzer::OnEitTable(const IEitTable *pEitTable, const IEitItem *pEitItem)
{
	CBlockLock lock(m_Lock);

	// EPGXV
	if(pEitTable->GetOrgNetworkID() != GetNetworkID())return;
	if(pEitTable->GetTsID() != GetTsID())return;
	
	PROGRAM_ITEM *pProgItem = FindProgramItem(pEitTable->GetServiceID());
	if(!pProgItem)return;

	if(pEitTable->GetTableID() == IEitTable::TABLE_ID_CURORNEXT_SELF){
		//  or EPGXgA
		switch(pEitTable->GetSectionNumber()){
			case 0U :
				if(pProgItem->pCurEpgItem->StoreEitItem(pEitItem)){
					// ԑgXVCxg
//					TCHAR szInfo[4096] = TEXT("M");
//					pProgItem->pCurEpgItem->GetEventDetail(szInfo);
//					::BON_TRACE(TEXT("Info: %04X\n%s\n\n"), pProgItem->pCurEpgItem->GetServiceID(), szInfo);
					}
				break;

			case 1U :
				if(pProgItem->pNextEpgItem->StoreEitItem(pEitItem)){
					// ԑgXVCxg
					}
				break; 

			default :
				// Ki͂肦Ȃ
				break;
			}
		}
	else{
		// XPW[ςEPGXgA
		}	
}

void CProgAnalyzer::ResetPidMap(void)
{
	// NIT/SDT/TOTZbgAȊO̓A}bv
	for(WORD wPID = 0x0000U ; wPID < 0x2000U ; wPID++){
		switch(wPID){
			case 0x0000 :	// PAT
				break;
			
			case 0x0010 :	// NIT
			case 0x0011 :	// SDT
			case 0x0012 :	// EIT
			case 0x0014 :	// TOT
				// Zbg
				m_TsPidMapper.ResetPid(wPID);
				break;
			
			default :
				// A}bv
				m_TsPidMapper.UnmapPid(wPID);
				break;			
			}
		}
}

CProgAnalyzer::PROGRAM_ITEM * CProgAnalyzer::FindProgramItem(WORD wServiceID) const
{
	// T[rXIDȗ݂͌̃vO
	if(!wServiceID)wServiceID = GetCurrentServiceID();

	// T[rXIDΉACeĕԂ
	for(ProgramList::const_iterator itProgram = m_ProgContext.ProgList.begin() ; itProgram != m_ProgContext.ProgList.end() ; itProgram++){
		if(itProgram->wServiceID == wServiceID){
			return const_cast<CProgAnalyzer::PROGRAM_ITEM *>(&(*itProgram));
			}
		}

	return NULL;
}
