//////////////////////////////////////////////////////////////////////////
// BonSDK Media decoder library
// Copyright (c) http://2sen.dip.jp/dtv/
//
// This is part of BonSDK.
// BonSDK comes with ABSOLUTELY NO WARRANTY;
// for details see 'GNU GENERAL PUBLIC LICENSE 2.0'.

// Multi2Decoder.cpp: MULTI2ÍfR[_NX
//
//////////////////////////////////////////////////////////////////////

#include "StdAfx.h"
#include "BonSDK.h"
#include "Multi2Decoder.h"
#include <stdlib.h>
#include <intrin.h>


/////////////////////////////////////////////////////////////////////////////
// t@C[J萔ݒ
/////////////////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////////////////////
// MULTI2ÍfR[_NX
/////////////////////////////////////////////////////////////////////////////

class CMulti2DecoderImpl
{
protected:
	typedef ULONG64 DATKEY;	// Data Key(Dk) 64bit

	class SYSKEY	// System Key(Sk), Expanded Key(Wk) 256bit
	{
	public:
		__forceinline void SYSKEY::SetHexData(const BYTE *pHexData)
		{
			// oCgI[_[ϊ
			reinterpret_cast<ULONG64 *>(this)[0] = ::_byteswap_uint64(reinterpret_cast<const ULONG64 *>(pHexData)[0]);
			reinterpret_cast<ULONG64 *>(this)[1] = ::_byteswap_uint64(reinterpret_cast<const ULONG64 *>(pHexData)[1]);
			reinterpret_cast<ULONG64 *>(this)[2] = ::_byteswap_uint64(reinterpret_cast<const ULONG64 *>(pHexData)[2]);
			reinterpret_cast<ULONG64 *>(this)[3] = ::_byteswap_uint64(reinterpret_cast<const ULONG64 *>(pHexData)[3]);
		}

		__forceinline void SYSKEY::GetHexData(BYTE *pHexData) const
		{
			// oCgI[_[ϊ
			reinterpret_cast<ULONG64 *>(pHexData)[0]  = ::_byteswap_uint64(reinterpret_cast<const ULONG64 *>(this)[0]);
			reinterpret_cast<ULONG64 *>(pHexData)[1]  = ::_byteswap_uint64(reinterpret_cast<const ULONG64 *>(this)[1]);
			reinterpret_cast<ULONG64 *>(pHexData)[2]  = ::_byteswap_uint64(reinterpret_cast<const ULONG64 *>(this)[2]);
			reinterpret_cast<ULONG64 *>(pHexData)[3]  = ::_byteswap_uint64(reinterpret_cast<const ULONG64 *>(this)[3]);
		}
	
		DWORD dwKey2, dwKey1, dwKey4, dwKey3, dwKey6, dwKey5, dwKey8, dwKey7;
	};

	DATKEY m_InitialCbc;
	SYSKEY m_SystemKey;
	SYSKEY m_WorkKeyOdd, m_WorkKeyEven;
	
	bool m_bIsSysKeyValid;
	bool m_bIsWorkKeyValid;

	static __forceinline const DATKEY DATKEY_FromHexData(const BYTE *pHexData)
	{
		return ::_byteswap_uint64(*reinterpret_cast<const ULONG64 *>(pHexData));
	}

	static __forceinline void DATKEY_ToHexData(BYTE *pHexData, const DATKEY dataKey)
	{
		*reinterpret_cast<ULONG64 *>(pHexData) = ::_byteswap_uint64(dataKey);
	}

	static __forceinline void RoundFuncPi1(DATKEY &block)
	{
		// Elementary Encryption Function 1
		block ^= static_cast<const DWORD>(block >> 32);
	}

	static __forceinline void RoundFuncPi2(DATKEY &block, const DWORD dwK1)
	{
		// Elementary Encryption Function 2
		const DWORD dwY = static_cast<const DWORD>(block) + dwK1;
		const DWORD dwZ = _lrotl(dwY, 1) + dwY - 1UL;
		block ^= static_cast<const ULONG64>(_lrotl(dwZ, 4) ^ dwZ) << 32;
	}

	static __forceinline void RoundFuncPi3(DATKEY &block, const DWORD dwK2, const DWORD dwK3)
	{
		// Elementary Encryption Function 3
		const DWORD dwY = static_cast<const DWORD>(block >> 32) + dwK2;
		const DWORD dwZ = _lrotl(dwY, 2) + dwY + 1UL;
		const DWORD dwA = _lrotl(dwZ, 8) ^ dwZ;
		const DWORD dwB = dwA + dwK3;
		const DWORD dwC = _lrotl(dwB, 1) - dwB;
		block ^= _lrotl(dwC, 16) ^ (dwC | static_cast<const DWORD>(block >> 32));
	}

	static __forceinline void RoundFuncPi4(DATKEY &block, const DWORD dwK4)
	{
		// Elementary Encryption Function 4
		const DWORD dwY = static_cast<const DWORD>(block) + dwK4;
		block ^= static_cast<const ULONG64>(_lrotl(dwY, 2) + dwY + 1UL) << 32;
	}

	static __forceinline void KeySchedule(SYSKEY &workKey, const SYSKEY &SysKey, DATKEY &dataKey)
	{
		// Key Schedule
		RoundFuncPi1(dataKey);									// 1

		RoundFuncPi2(dataKey, SysKey.dwKey1);					// 2
		workKey.dwKey1 = static_cast<const DWORD>(dataKey >> 32);

		RoundFuncPi3(dataKey, SysKey.dwKey2, SysKey.dwKey3);	// 3
		workKey.dwKey2 = static_cast<const DWORD>(dataKey);

		RoundFuncPi4(dataKey, SysKey.dwKey4);					// 4
		workKey.dwKey3 = static_cast<const DWORD>(dataKey >> 32);
		
		RoundFuncPi1(dataKey);									// 1
		workKey.dwKey4 = static_cast<const DWORD>(dataKey);
		
		RoundFuncPi2(dataKey, SysKey.dwKey5);					// 2
		workKey.dwKey5 = static_cast<const DWORD>(dataKey >> 32);

		RoundFuncPi3(dataKey, SysKey.dwKey6, SysKey.dwKey7);	// 3
		workKey.dwKey6 = static_cast<const DWORD>(dataKey);

		RoundFuncPi4(dataKey, SysKey.dwKey8);					// 4
		workKey.dwKey7 = static_cast<const DWORD>(dataKey >> 32);

		RoundFuncPi1(dataKey);									// 1
		workKey.dwKey8 = static_cast<const DWORD>(dataKey);
	}

	static __forceinline void DecryptBlock(DATKEY &block, const SYSKEY &workKey)
	{
		// block Decryption
		RoundFuncPi4(block, workKey.dwKey8);
		RoundFuncPi3(block, workKey.dwKey6, workKey.dwKey7);
		RoundFuncPi2(block, workKey.dwKey5);
		RoundFuncPi1(block);
		RoundFuncPi4(block, workKey.dwKey4);
		RoundFuncPi3(block, workKey.dwKey2, workKey.dwKey3);
		RoundFuncPi2(block, workKey.dwKey1);
		RoundFuncPi1(block);

		RoundFuncPi4(block, workKey.dwKey8);
		RoundFuncPi3(block, workKey.dwKey6, workKey.dwKey7);
		RoundFuncPi2(block, workKey.dwKey5);
		RoundFuncPi1(block);
		RoundFuncPi4(block, workKey.dwKey4);
		RoundFuncPi3(block, workKey.dwKey2, workKey.dwKey3);
		RoundFuncPi2(block, workKey.dwKey1);
		RoundFuncPi1(block);

		RoundFuncPi4(block, workKey.dwKey8);
		RoundFuncPi3(block, workKey.dwKey6, workKey.dwKey7);
		RoundFuncPi2(block, workKey.dwKey5);
		RoundFuncPi1(block);
		RoundFuncPi4(block, workKey.dwKey4);
		RoundFuncPi3(block, workKey.dwKey2, workKey.dwKey3);
		RoundFuncPi2(block, workKey.dwKey1);
		RoundFuncPi1(block);

		RoundFuncPi4(block, workKey.dwKey8);
		RoundFuncPi3(block, workKey.dwKey6, workKey.dwKey7);
		RoundFuncPi2(block, workKey.dwKey5);
		RoundFuncPi1(block);
		RoundFuncPi4(block, workKey.dwKey4);
		RoundFuncPi3(block, workKey.dwKey2, workKey.dwKey3);
		RoundFuncPi2(block, workKey.dwKey1);
		RoundFuncPi1(block);
	}

	static __forceinline void EncryptBlock(DATKEY &block, const SYSKEY &workKey)
	{
		// block Encryption
		RoundFuncPi1(block);
		RoundFuncPi2(block, workKey.dwKey1);
		RoundFuncPi3(block, workKey.dwKey2, workKey.dwKey3);
		RoundFuncPi4(block, workKey.dwKey4);
		RoundFuncPi1(block);
		RoundFuncPi2(block, workKey.dwKey5);
		RoundFuncPi3(block, workKey.dwKey6, workKey.dwKey7);
		RoundFuncPi4(block, workKey.dwKey8);

		RoundFuncPi1(block);
		RoundFuncPi2(block, workKey.dwKey1);
		RoundFuncPi3(block, workKey.dwKey2, workKey.dwKey3);
		RoundFuncPi4(block, workKey.dwKey4);
		RoundFuncPi1(block);
		RoundFuncPi2(block, workKey.dwKey5);
		RoundFuncPi3(block, workKey.dwKey6, workKey.dwKey7);
		RoundFuncPi4(block, workKey.dwKey8);

		RoundFuncPi1(block);
		RoundFuncPi2(block, workKey.dwKey1);
		RoundFuncPi3(block, workKey.dwKey2, workKey.dwKey3);
		RoundFuncPi4(block, workKey.dwKey4);
		RoundFuncPi1(block);
		RoundFuncPi2(block, workKey.dwKey5);
		RoundFuncPi3(block, workKey.dwKey6, workKey.dwKey7);
		RoundFuncPi4(block, workKey.dwKey8);

		RoundFuncPi1(block);
		RoundFuncPi2(block, workKey.dwKey1);
		RoundFuncPi3(block, workKey.dwKey2, workKey.dwKey3);
		RoundFuncPi4(block, workKey.dwKey4);
		RoundFuncPi1(block);
		RoundFuncPi2(block, workKey.dwKey5);
		RoundFuncPi3(block, workKey.dwKey6, workKey.dwKey7);
		RoundFuncPi4(block, workKey.dwKey8);
	}

public:
	CMulti2DecoderImpl(void)
		: m_bIsSysKeyValid(false)
		, m_bIsWorkKeyValid(false)
	{
	}

	virtual ~CMulti2DecoderImpl(void)
	{
	}
	
	void Initialize(const BYTE *pSystemKey, const BYTE *pInitialCbc)
	{
		m_bIsSysKeyValid = true;
		m_bIsWorkKeyValid = false;

		// Descrambling System KeyZbg
		m_SystemKey.SetHexData(pSystemKey);
		
		// Descrambler CBC Initial ValueZbg
		m_InitialCbc = DATKEY_FromHexData(pInitialCbc);
	}

	const bool SetScrambleKey(const BYTE *pScrambleKey)
	{
		if(!m_bIsSysKeyValid)return false;
		
		if(!pScrambleKey){
			// L[ݒ肳Ȃꍇ̓fR[hs\ɂ(sȕɂjh~̂)
			m_bIsWorkKeyValid = false;
			return false;
			}

		// oCgI[_[ϊ
		DATKEY scrKeyOdd = DATKEY_FromHexData(&pScrambleKey[0]);
		DATKEY scrKeyEven = DATKEY_FromHexData(&pScrambleKey[8]);

		// L[XPW[
		KeySchedule(m_WorkKeyOdd, m_SystemKey, scrKeyOdd);
		KeySchedule(m_WorkKeyEven, m_SystemKey, scrKeyEven);
		
		m_bIsWorkKeyValid = true;
		
		return true;
	}

	virtual const bool Decode(BYTE *pData, const DWORD dwSize, const BYTE byScrCtrl) const
	{
		if(!byScrCtrl)return true;										// XNuȂ
		else if(!m_bIsSysKeyValid || !m_bIsWorkKeyValid)return false;	// XNuL[ݒ
		else if((byScrCtrl != 2U) && (byScrCtrl != 3U))return false;	// XNus

		// [NL[I
		const SYSKEY &workKey = (byScrCtrl == 3)? m_WorkKeyOdd : m_WorkKeyEven;

		DWORD dwPos;
		DATKEY cbcData = m_InitialCbc;

		// CBC[h
		for(dwPos = 0UL ; (dwSize - dwPos) >= sizeof(DATKEY) ; dwPos += sizeof(DATKEY))
		{
			DATKEY srcData = DATKEY_FromHexData(&pData[dwPos]);
			DATKEY originalData = srcData;
			DecryptBlock(srcData, workKey);
			srcData ^= cbcData;
			cbcData = originalData;
			DATKEY_ToHexData(&pData[dwPos], srcData);
		}

		// OFB[h
		if(dwPos < dwSize)
		{
			BYTE remain[sizeof(DATKEY)];
			EncryptBlock(cbcData, workKey);
			DATKEY_ToHexData(remain, cbcData);

			for(BYTE *pRemain = remain ; (dwSize - dwPos) > 0UL ; dwPos++, pRemain++)
			{
				pData[dwPos] ^= *pRemain;
			}
		}

		return true;
	}
};


/////////////////////////////////////////////////////////////////////////////////
// SSE2ΉiFrom TVTestj
// elick̃R[hQlɂĂ܂B܂œK\Ǝv܂B

#define MM_SHIFT_COUNT(n) (m_ShiftCount##n)
#define MM_ROTATE(v, l, r) LeftRotateSSE2(v, MM_SHIFT_COUNT(l), MM_SHIFT_COUNT(r))

class CMulti2DecoderSSE2Impl : public CMulti2DecoderImpl
{
private:
	static const __m128i m_ShiftCount1;
	static const __m128i m_ShiftCount2;
	static const __m128i m_ShiftCount4;
	static const __m128i m_ShiftCount8;
	static const __m128i m_ShiftCount16;
	static const __m128i m_ShiftCount31;
	static const __m128i m_ShiftCount30;
	static const __m128i m_ShiftCount28;
	static const __m128i m_ShiftCount24;
	static const __m128i m_Immediate1;
	static const __m128i m_SwapMask;

	static __forceinline void RoundFuncPi1SSE2(const __m128i &Left, __m128i &Right)
	{
		Right = _mm_xor_si128(Right, Left);
	}

	static __forceinline void RoundFuncPi2SSE2(__m128i &Left, const __m128i &Right, const __m128i &Key1)
	{
		const __m128i t0 = _mm_add_epi32(Right, Key1);
		const __m128i t1 = _mm_sub_epi32(_mm_add_epi32(MM_ROTATE(t0, 1, 31), t0), m_Immediate1);
		const __m128i t2 = _mm_xor_si128(MM_ROTATE(t1, 4, 28), t1);

		Left = _mm_xor_si128(Left, t2);
	}

	static __forceinline void RoundFuncPi3SSE2(const __m128i &Left, __m128i &Right, const __m128i &Key2, const __m128i &Key3)
	{
		const __m128i t0 = _mm_add_epi32(Left, Key2);
		const __m128i t1 = _mm_add_epi32(_mm_add_epi32(MM_ROTATE(t0, 2, 30), t0), m_Immediate1);
		const __m128i t2 = _mm_xor_si128(MM_ROTATE(t1, 8, 24), t1);
		const __m128i t3 = _mm_add_epi32(t2, Key3);
		const __m128i t4 = _mm_sub_epi32(MM_ROTATE(t3, 1, 31), t3);
		const __m128i t5 = _mm_xor_si128(MM_ROTATE(t4, 16, 16), _mm_or_si128(t4, Left));

		Right = _mm_xor_si128(Right, t5);
	}

	static __forceinline void RoundFuncPi4SSE2(__m128i &Left, const __m128i &Right, const __m128i &Key4)
	{
		const __m128i t0 = _mm_add_epi32(Right, Key4);
		const __m128i t1 = _mm_add_epi32(_mm_add_epi32(MM_ROTATE(t0, 2, 30), t0), m_Immediate1);

		Left = _mm_xor_si128(Left, t1);
	}

	static __forceinline __m128i LeftRotateSSE2(const __m128i &Value, const __m128i &Rotate, const __m128i &InvRotate)
	{
		return _mm_or_si128(_mm_sll_epi32(Value, Rotate), _mm_srl_epi32(Value, InvRotate));
	}

	static __forceinline __m128i ByteSwapSSE2(const __m128i &Value)
	{
		__m128i src = Value;
		__m128i tmp = _mm_and_si128(src, m_SwapMask);
		tmp = _mm_srl_epi32(tmp, MM_SHIFT_COUNT(8));
		src = _mm_sll_epi32(src, MM_SHIFT_COUNT(8));
		src = _mm_and_si128(src, m_SwapMask);
		src = _mm_or_si128(src, tmp);
		return MM_ROTATE(src, 16, 16);
	}

public:
	CMulti2DecoderSSE2Impl()
	{
	}

	static bool IsSSE2Available()
	{
		int cpuInfo[4];
		__cpuid(cpuInfo, 1);

		return (cpuInfo[3] & (0x00000001UL < 26)) > 0;
	}

	virtual const bool Decode(BYTE *pData, const DWORD dwSize, const BYTE byScrCtrl) const
	{
		if(!byScrCtrl)return true;										// XNuȂ
		else if(!m_bIsSysKeyValid || !m_bIsWorkKeyValid)return false;	// XNuL[ݒ
		else if((byScrCtrl != 2U) && (byScrCtrl != 3U))return false;	// XNus

		// [NL[I
		const SYSKEY &workKey = (byScrCtrl == 3) ? m_WorkKeyOdd : m_WorkKeyEven;

		const __m128i workKey1 = _mm_set1_epi32(workKey.dwKey1);
		const __m128i workKey2 = _mm_set1_epi32(workKey.dwKey2);
		const __m128i workKey3 = _mm_set1_epi32(workKey.dwKey3);
		const __m128i workKey4 = _mm_set1_epi32(workKey.dwKey4);
		const __m128i workKey5 = _mm_set1_epi32(workKey.dwKey5);
		const __m128i workKey6 = _mm_set1_epi32(workKey.dwKey6);
		const __m128i workKey7 = _mm_set1_epi32(workKey.dwKey7);
		const __m128i workKey8 = _mm_set1_epi32(workKey.dwKey8);

		DWORD remainSize = dwSize & 0xFFFFFFE0UL;
		BYTE *pEnd = pData + remainSize;
		BYTE *p = pData;

		// CBC[h
		__m128i cbc;
		cbc.m128i_u64[0] = _byteswap_uint64(m_InitialCbc);
		cbc.m128i_u64[1] = 0;

		while (p < pEnd)
		{
			// r2 l2 r1 l1
			const __m128i Src1 = _mm_loadu_si128((__m128i*)p);
			// r4 l4 r3 l3
			const __m128i Src2 = _mm_loadu_si128((__m128i*)(p + 16));

			__m128i Left = ByteSwapSSE2(Src1);
			__m128i Right = ByteSwapSSE2(Src2);

			{
				// r2 r1 l2 l1
				__m128i x = _mm_shuffle_epi32(Left, _MM_SHUFFLE(3, 1, 2, 0));

				// r4 r3 l4 l3
				__m128i y = _mm_shuffle_epi32(Right, _MM_SHUFFLE(3, 1, 2, 0));

				// l4 l2 l3 l1
				Left = _mm_unpacklo_epi32(x, y);

				// r4 r2 r3 r1
				Right = _mm_unpackhi_epi32(x, y);
			}

			RoundFuncPi4SSE2(Left, Right, workKey8);
			RoundFuncPi3SSE2(Left, Right, workKey6, workKey7);
			RoundFuncPi2SSE2(Left, Right, workKey5);
			RoundFuncPi1SSE2(Left, Right);
			RoundFuncPi4SSE2(Left, Right, workKey4);
			RoundFuncPi3SSE2(Left, Right, workKey2, workKey3);
			RoundFuncPi2SSE2(Left, Right, workKey1);
			RoundFuncPi1SSE2(Left, Right);

			RoundFuncPi4SSE2(Left, Right, workKey8);
			RoundFuncPi3SSE2(Left, Right, workKey6, workKey7);
			RoundFuncPi2SSE2(Left, Right, workKey5);
			RoundFuncPi1SSE2(Left, Right);
			RoundFuncPi4SSE2(Left, Right, workKey4);
			RoundFuncPi3SSE2(Left, Right, workKey2, workKey3);
			RoundFuncPi2SSE2(Left, Right, workKey1);
			RoundFuncPi1SSE2(Left, Right);

			RoundFuncPi4SSE2(Left, Right, workKey8);
			RoundFuncPi3SSE2(Left, Right, workKey6, workKey7);
			RoundFuncPi2SSE2(Left, Right, workKey5);
			RoundFuncPi1SSE2(Left, Right);
			RoundFuncPi4SSE2(Left, Right, workKey4);
			RoundFuncPi3SSE2(Left, Right, workKey2, workKey3);
			RoundFuncPi2SSE2(Left, Right, workKey1);
			RoundFuncPi1SSE2(Left, Right);

			RoundFuncPi4SSE2(Left, Right, workKey8);
			RoundFuncPi3SSE2(Left, Right, workKey6, workKey7);
			RoundFuncPi2SSE2(Left, Right, workKey5);
			RoundFuncPi1SSE2(Left, Right);
			RoundFuncPi4SSE2(Left, Right, workKey4);
			RoundFuncPi3SSE2(Left, Right, workKey2, workKey3);
			RoundFuncPi2SSE2(Left, Right, workKey1);
			RoundFuncPi1SSE2(Left, Right);

			{
				// l4 l3 l2 l1
				__m128i a = _mm_shuffle_epi32(Left, _MM_SHUFFLE(3, 1, 2, 0));

				// r4 r3 r2 r1
				__m128i b = _mm_shuffle_epi32(Right, _MM_SHUFFLE(3, 1, 2, 0));

				// r2 l2 r1 l1
				__m128i x = _mm_unpacklo_epi32(a, b);

				// r4 l4 r3 l3
				__m128i y = _mm_unpackhi_epi32(a, b);

				x = _mm_xor_si128(ByteSwapSSE2(x), _mm_or_si128(_mm_slli_si128(Src1, 8), cbc));
				y = _mm_xor_si128(ByteSwapSSE2(y), _mm_loadu_si128((__m128i*)(p + 8)));
				_mm_storeu_si128((__m128i*)p, x);
				_mm_storeu_si128((__m128i*)p + 1, y);

				cbc = _mm_srli_si128(Src2, 8);
			}

			p += 32;
		}

		DATKEY cbcData = _byteswap_uint64(cbc.m128i_u64[0]);
		remainSize= dwSize & 0x00000018UL;
		pEnd = p + remainSize;
		while (p < pEnd)
		{
			DATKEY srcData = DATKEY_FromHexData(p);
			DATKEY originalData = srcData;
			DecryptBlock(srcData, workKey);
			srcData ^= cbcData;
			cbcData = originalData;
			DATKEY_ToHexData(p, srcData);
			p += sizeof(DATKEY);
		}

		// OFB[h
		remainSize = dwSize & 0x00000007UL;
		if (remainSize > 0)
		{
			BYTE remain[sizeof(DATKEY)];
			EncryptBlock(cbcData, workKey);
			DATKEY_ToHexData(remain, cbcData);

			switch (remainSize)
			{
			case 7: p[6] ^= remain[6];
			case 6: p[5] ^= remain[5];
			case 5: p[4] ^= remain[4];
			case 4: p[3] ^= remain[3];
			case 3: p[2] ^= remain[2];
			case 2: p[1] ^= remain[1];
			case 1: p[0] ^= remain[0];
			}
		}

		return true;
	}
};

const __m128i CMulti2DecoderSSE2Impl::m_ShiftCount1 = _mm_set_epi32(0, 0, 0, 1);
const __m128i CMulti2DecoderSSE2Impl::m_ShiftCount2 = _mm_set_epi32(0, 0, 0, 2);
const __m128i CMulti2DecoderSSE2Impl::m_ShiftCount4 = _mm_set_epi32(0, 0, 0, 4);
const __m128i CMulti2DecoderSSE2Impl::m_ShiftCount8 = _mm_set_epi32(0, 0, 0, 8);
const __m128i CMulti2DecoderSSE2Impl::m_ShiftCount16 = _mm_set_epi32(0, 0, 0, 16);
const __m128i CMulti2DecoderSSE2Impl::m_ShiftCount31 = _mm_set_epi32(0, 0, 0, 31);
const __m128i CMulti2DecoderSSE2Impl::m_ShiftCount30 = _mm_set_epi32(0, 0, 0, 30);
const __m128i CMulti2DecoderSSE2Impl::m_ShiftCount28 = _mm_set_epi32(0, 0, 0, 28);
const __m128i CMulti2DecoderSSE2Impl::m_ShiftCount24 = _mm_set_epi32(0, 0, 0, 24);
const __m128i CMulti2DecoderSSE2Impl::m_Immediate1 = _mm_set1_epi32(1);
const __m128i CMulti2DecoderSSE2Impl::m_SwapMask = _mm_set1_epi32(0xFF00FF00);


/////////////////////////////////////////////////////////////////////////////////
// CMulti2DecoderCMulti2DecoderImpl̃t@T[h

CMulti2Decoder::CMulti2Decoder(void)
	: m_pDecoder(CMulti2DecoderSSE2Impl::IsSSE2Available() ? new CMulti2DecoderSSE2Impl() : new CMulti2DecoderImpl())
{
}

CMulti2Decoder::~CMulti2Decoder(void)
{
	delete m_pDecoder;
}

void CMulti2Decoder::Initialize(const BYTE *pSystemKey, const BYTE *pInitialCbc)
{
	m_pDecoder->Initialize(pSystemKey, pInitialCbc);
}

const bool CMulti2Decoder::SetScrambleKey(const BYTE *pScrambleKey)
{
	return m_pDecoder->SetScrambleKey(pScrambleKey);
}

const bool CMulti2Decoder::Decode(BYTE *pData, const DWORD dwSize, const BYTE byScrCtrl) const
{
	return m_pDecoder->Decode(pData, dwSize, byScrCtrl);
}
