//////////////////////////////////////////////////////////////////////////
// BonSDK Media decoder library
// Copyright (c) http://2sen.dip.jp/dtv/
//
// This is part of BonSDK.
// BonSDK comes with ABSOLUTELY NO WARRANTY;
// for details see 'GNU GENERAL PUBLIC LICENSE 2.0'.

// TsPacket.cpp: TSpPbgNX
//
/////////////////////////////////////////////////////////////////////////////

#include "StdAfx.h"
#include "BonSDK.h"
#include "TsPacket.h"


/////////////////////////////////////////////////////////////////////////////
// ITsPacketNX
/////////////////////////////////////////////////////////////////////////////

const DWORD CTsPacket::ParsePacket(BYTE *pContinuityCounter)
{
	// TSpPbgwb_
	const BYTE *pData = GetData();
	m_Header.bySyncByte					= pData[0];							// +0
	m_Header.bTransportErrorIndicator	= (pData[1] & 0x80U)? true : false;	// +1 bit7
	m_Header.bPayloadUnitStartIndicator	= (pData[1] & 0x40U)? true : false;	// +1 bit6
	m_Header.bTransportPriority			= (pData[1] & 0x20U)? true : false;	// +1 bit5
	m_Header.wPID = ((WORD)(pData[1] & 0x1F) << 8) | (WORD)pData[2];		// +1 bit4-0, +2
	m_Header.byTransportScramblingCtrl	= (pData[3] & 0xC0U) >> 6;			// +3 bit7-6
	m_Header.byAdaptationFieldCtrl		= (pData[3] & 0x30U) >> 4;			// +3 bit5-4
	m_Header.byContinuityCounter		= pData[3] & 0x0FU;					// +3 bit3-0
	
	// A_ve[VtB[h
	::ZeroMemory(&m_AdaptationField, sizeof(m_AdaptationField));
	
	if(m_Header.byAdaptationFieldCtrl & 0x02){
		// A_ve[VtB[h
		if(m_AdaptationField.byAdaptationFieldLength = pData[4]){								// +4
			// tB[hȍ~
			m_AdaptationField.bDiscontinuityIndicator	= (pData[5] & 0x80U)? true : false;	// +5 bit7
			m_AdaptationField.bRamdomAccessIndicator	= (pData[5] & 0x40U)? true : false;	// +5 bit6
			m_AdaptationField.bEsPriorityIndicator		= (pData[5] & 0x20U)? true : false;	// +5 bit5
			m_AdaptationField.bPcrFlag					= (pData[5] & 0x10U)? true : false;	// +5 bit4
			m_AdaptationField.bOpcrFlag					= (pData[5] & 0x08U)? true : false;	// +5 bit3
			m_AdaptationField.bSplicingPointFlag		= (pData[5] & 0x04U)? true : false;	// +5 bit2
			m_AdaptationField.bTransportPrivateDataFlag	= (pData[5] & 0x02U)? true : false;	// +5 bit1
			m_AdaptationField.bAdaptationFieldExtFlag	= (pData[5] & 0x01U)? true : false;	// +5 bit0
			
			if(pData[4] > 1U){
				m_AdaptationField.pOptionData			= &pData[6];
				m_AdaptationField.byOptionSize			= pData[4] - 1U;
				}
			}
		}

	// pPbg̃tH[}bgK`FbN
	if(m_Header.bySyncByte != 0x47U)return EC_FORMAT;								// oCgs
	if(m_Header.bTransportErrorIndicator)return EC_TRANSPORT;						// rbg肠
	if((m_Header.wPID >= 0x0002U) && (m_Header.wPID <= 0x000FU))return EC_FORMAT;	// `PID͈
	if(m_Header.byTransportScramblingCtrl == 0x01U)return EC_FORMAT;				// `XNul
	if(m_Header.byAdaptationFieldCtrl == 0x00U)return EC_FORMAT;					// `A_ve[VtB[hl
	if((m_Header.byAdaptationFieldCtrl == 0x02U) && (m_AdaptationField.byAdaptationFieldLength > 183U))return EC_FORMAT;	// A_ve[VtB[hُ
	if((m_Header.byAdaptationFieldCtrl == 0x03U) && (m_AdaptationField.byAdaptationFieldLength > 182U))return EC_FORMAT;	// A_ve[VtB[hُ

	if(!pContinuityCounter || m_Header.wPID == 0x1FFFU)return EC_VALID;

	// A`FbN
	const BYTE byOldCounter = pContinuityCounter[m_Header.wPID];
	const BYTE byNewCounter = (m_Header.byAdaptationFieldCtrl & 0x01U)? m_Header.byContinuityCounter : 0x10U;
	pContinuityCounter[m_Header.wPID] = byNewCounter;

	if(!m_AdaptationField.bDiscontinuityIndicator){
		if((byOldCounter < 0x10U) && (byNewCounter < 0x10U)){
			if(((byOldCounter + 1U) & 0x0FU) != byNewCounter){
				return EC_CONTINUITY;
				}
			}
		}

	return EC_VALID;
}

BYTE * CTsPacket::GetRawData(void)
{
	// pPbg̃oCif[^Ԃ
	return (GetSize() == TS_PACKET_SIZE)? GetData() : NULL;
}

const BYTE * CTsPacket::GetRawData(void) const
{
	// pPbg̃oCif[^Ԃ
	return (GetSize() == TS_PACKET_SIZE)? GetData() : NULL;
}

const WORD CTsPacket::GetPID(void) const
{
	// PIDԂ
	return m_Header.wPID;
}

const bool CTsPacket::IsScrambled(void) const
{
	// XNuLԂ
	return (m_Header.byTransportScramblingCtrl & 0x02U)? true : false;
}

const bool CTsPacket::HaveAdaptationField(void) const
{
	// A_ve[VtB[h̗LԂ
	return (m_Header.byAdaptationFieldCtrl & 0x02U)? true : false;
}

BYTE * CTsPacket::GetPayloadData(void)
{
	// yC[h|C^Ԃ
	switch(m_Header.byAdaptationFieldCtrl){
		case 1U :	// yC[ĥ
			return GetData() + 4;
		
		case 3U :	// A_ve[VtB[hAyC[h
			return GetData() + m_AdaptationField.byAdaptationFieldLength + 5;
		
		default :	// A_ve[VtB[ĥ or O
			return NULL;
		}
}

const BYTE * CTsPacket::GetPayloadData(void) const
{
	// yC[h|C^Ԃ
	switch(m_Header.byAdaptationFieldCtrl){
		case 1U :	// yC[ĥ
			return GetData() + 4;
		
		case 3U :	// A_ve[VtB[hAyC[h
			return GetData() + m_AdaptationField.byAdaptationFieldLength + 5;
		
		default :	// A_ve[VtB[ĥ or O
			return NULL;
		}
}

const BYTE CTsPacket::GetPayloadSize(void) const
{
	// yC[hTCYԂ
	switch(m_Header.byAdaptationFieldCtrl){
		case 1U :	// yC[ĥ
			return (TS_PACKET_SIZE - 4U);
		
		case 3U :	// A_ve[VtB[hAyC[h
			return (TS_PACKET_SIZE - m_AdaptationField.byAdaptationFieldLength - 5U);
		
		default :	// A_ve[VtB[ĥ or O
			return 0U;
		}
}

const bool CTsPacket::CopyPacket(const ITsPacket *pSrc)
{
	if(!pSrc)return false;
	
	// oCif[^擾
	const BYTE *pSrcRawData = pSrc->GetRawData();
	if(!pSrcRawData)return false;

	// f[^Rs[
	SetData(pSrcRawData, TS_PACKET_SIZE);
	
	// pPbg
	ParsePacket();

	return true;
}

const bool CTsPacket::ComparePacket(const ITsPacket *pSrc) const
{
	// pPbgr
	if(!pSrc)return false;

	const BYTE *pSrcRawData = pSrc->GetRawData();
	
	// TCY`FbN
	if(!pSrcRawData || (GetSize() != TS_PACKET_SIZE))return false;

	// pPbgr
	return (!::memcmp(pSrcRawData, GetData(), TS_PACKET_SIZE))? true : false;
}

const DWORD CTsPacket::CopyData(const IMediaData *pSrc)
{
	// ITsPacketC^tF[X擾
	const ITsPacket * const pTsPacket = dynamic_cast<const ITsPacket *>(pSrc);

	if(pTsPacket){
		// ITsPacketƂăRs[
		return (CopyPacket(pTsPacket))? TS_PACKET_SIZE : 0UL;
		}
	else{
		// IMediaDataƂăRs[
		return CMediaData::CopyData(pSrc);
		}
}

IMediaData *CTsPacket::Clone(IMediaDataHeap *pHeap) const
{
	// ftHg̎I[o[ChāAK؂Ȍ^ŐKv
	CTsPacket *pTsPacket = InternalClone<CTsPacket>(pHeap);
	pTsPacket->m_Header = m_Header;
	pTsPacket->m_AdaptationField = m_AdaptationField;
	return pTsPacket;
}

CTsPacket::CTsPacket(IBonObject *pOwner, IMediaDataHeap *pHeap)
	: CMediaData(pOwner, pHeap)
{
	// obt@m
	GetBuffer(TS_PACKET_SIZE);
	
	// wb_NA
	::ZeroMemory(&m_Header, sizeof(m_Header));
	::ZeroMemory(&m_AdaptationField, sizeof(m_AdaptationField));
}

CTsPacket::~CTsPacket(void)
{

}
