//////////////////////////////////////////////////////////////////////////
// BonSDK Media decoder library
// Copyright (c) http://2sen.dip.jp/dtv/
//
// This is part of BonSDK.
// BonSDK comes with ABSOLUTELY NO WARRANTY;
// for details see 'GNU GENERAL PUBLIC LICENSE 2.0'.

// TsSection.cpp: PSIZNVNX
//
/////////////////////////////////////////////////////////////////////////////

#include "StdAfx.h"
#include "BonSDK.h"
#include "TsSection.h"


/////////////////////////////////////////////////////////////////////////////
// PSIZNVNX
/////////////////////////////////////////////////////////////////////////////

const bool CPsiSection::ParseHeader(const bool bIsExtended)
{
	const DWORD dwHeaderSize = (bIsExtended)? 8UL : 3UL;

	// wb_TCY`FbN
	if(GetSize() < dwHeaderSize)return false;

	// W`wb_
	const BYTE *pData = GetData();
	m_Header.byTableID					= pData[0];									// +0
	m_Header.bSectionSyntaxIndicator	= (pData[1] & 0x80U)? true : false;			// +1 bit7
	m_Header.bPrivateIndicator			= (pData[1] & 0x40U)? true : false;			// +1 bit6
	m_Header.wSectionLength = ((WORD)(pData[1] & 0x0FU) << 8) | (WORD)pData[2];		// +1 bit5-0, +2

	if(m_Header.bSectionSyntaxIndicator && bIsExtended){
		// g`̃wb_
		m_Header.wTableIdExtension		= (WORD)pData[3] << 8 | (WORD)pData[4];		// +3, +4
		m_Header.byVersionNo			= (pData[5] & 0x3EU) >> 1;					// +5 bit5-1
		m_Header.bCurrentNextIndicator	= (pData[5] & 0x01U)? true : false;			// +5 bit0
		m_Header.bySectionNumber		= pData[6];									// +6
		m_Header.byLastSectionNumber	= pData[7];									// +7
		}

	// wb_̃tH[}bgK`FbN
	if((pData[1] & 0x30U) != 0x30U)return false;										// Œrbgُ
	else if(m_Header.wSectionLength > 4093U)return false;								// ZNVُ
	else if(m_Header.bSectionSyntaxIndicator){
		// g`̃G[`FbN
		if(!bIsExtended)return false;													// ړĨwb_ł͂Ȃ
		else if((pData[5] & 0xC0U) != 0xC0U)return false;								// Œrbgُ
		else if(m_Header.bySectionNumber > m_Header.byLastSectionNumber)return false;	// ZNVԍُ
		else if(m_Header.wSectionLength < 9U)return false;								// ZNVُ
		}
	else{
		// W`̃G[`FbN	
		if(bIsExtended)return false;													// ړĨwb_ł͂Ȃ
		else if(m_Header.wSectionLength < 4U)return false;								// ZNVُ
		}

	return true;
}

void CPsiSection::Reset(void)
{
	// f[^Zbg
	ClearSize();	
	::ZeroMemory(&m_Header, sizeof(m_Header));
}

BYTE * CPsiSection::GetPayloadData(void)
{
	// yC[h|C^Ԃ
	const DWORD dwHeaderSize = (m_Header.bSectionSyntaxIndicator)? 8UL : 3UL;

	return (GetSize() > dwHeaderSize)? GetData() + dwHeaderSize : NULL;
}

const BYTE * CPsiSection::GetPayloadData(void) const
{
	// yC[h|C^Ԃ
	const DWORD dwHeaderSize = (m_Header.bSectionSyntaxIndicator)? 8UL : 3UL;

	return (GetSize() > dwHeaderSize)? GetData() + dwHeaderSize : NULL;
}

const WORD CPsiSection::GetPayloadSize(void) const
{
	// ۂɕێĂyC[hTCYԂ(wb_̃ZNVł͂Ȃ)
	const DWORD dwHeaderSize = (m_Header.bSectionSyntaxIndicator)? 8UL : 3UL;

	const DWORD dataSize = GetSize();
	if(dataSize <= dwHeaderSize){
		return 0U;
		}
	else if(m_Header.bSectionSyntaxIndicator){
		// gZNV
		return (dataSize >= (m_Header.wSectionLength + 3UL))? (m_Header.wSectionLength - 9U) : (WORD)(dataSize - dwHeaderSize);
		}
	else{
		// WZNV
		return (dataSize >= (m_Header.wSectionLength + 3UL))? m_Header.wSectionLength : (WORD)(dataSize - dwHeaderSize);
		}
}

const BYTE CPsiSection::GetTableID(void) const
{
	// e[uIDԂ
	return m_Header.byTableID;
}

const bool CPsiSection::IsExtendedSection(void) const
{
	// ZNVV^bNXCWP[^Ԃ
	return m_Header.bSectionSyntaxIndicator;
}

const bool CPsiSection::IsPrivate(void) const
{
	// vCx[gCWP[^Ԃ
	return m_Header.bPrivateIndicator;
}

const WORD CPsiSection::GetSectionLength(void) const
{
	// ZNVԂ
	return m_Header.wSectionLength;
}

const WORD CPsiSection::GetTableIdExtension(void) const
{
	// e[uIDgԂ
	return m_Header.wTableIdExtension;
}

const BYTE CPsiSection::GetVersionNo(void) const
{
	// o[WԍԂ
	return m_Header.byVersionNo;
}

const bool CPsiSection::IsCurrentNext(void) const
{
	// JglNXgCWP[^Ԃ
	return m_Header.bCurrentNextIndicator;
}

const BYTE CPsiSection::GetSectionNumber(void) const
{
	// ZNVԍԂ
	return m_Header.bySectionNumber;
}

const BYTE CPsiSection::GetLastSectionNumber(void) const
{
	// XgZNVԍԂ
	return m_Header.byLastSectionNumber;
}

const DWORD CPsiSection::GetSectionCrc(void) const
{
	// CRCԂ
	if(!m_Header.bSectionSyntaxIndicator)return 0UL;

	const DWORD dataSize = GetSize();
	if(dataSize <= 12UL)return 0U;
	else if(dataSize < (m_Header.wSectionLength + 3UL))return 0U;
	else {
		const BYTE *pData = GetData();
		const DWORD dwCrcPos = m_Header.wSectionLength - 1UL;
		return ((DWORD)pData[dwCrcPos] << 24) | ((DWORD)pData[dwCrcPos + 1UL] << 16) | ((DWORD)pData[dwCrcPos + 2UL] << 8) | (DWORD)pData[dwCrcPos + 3UL];
	}
}

const bool CPsiSection::CopySection(const IPsiSection *pSrc)
{
	// IMediaDataC^tF[X擾
	const IMediaData * const pMediaData = dynamic_cast<const IMediaData *>(pSrc);
	if(!pMediaData)return false;

	// ZNVf[^Rs[
	if(CMediaData::CopyData(pMediaData) != GetSize())return false;
	
	// wb_Rs[
	m_Header.byTableID					= pSrc->GetTableID();
	m_Header.bSectionSyntaxIndicator	= pSrc->IsExtendedSection();
	m_Header.bPrivateIndicator			= pSrc->IsPrivate();
	m_Header.wSectionLength				= pSrc->GetSectionLength();
	m_Header.wTableIdExtension			= pSrc->GetTableIdExtension();
	m_Header.byVersionNo				= pSrc->GetVersionNo();
	m_Header.bCurrentNextIndicator		= pSrc->IsCurrentNext();
	m_Header.bySectionNumber			= pSrc->GetSectionNumber();
	m_Header.byLastSectionNumber		= pSrc->GetLastSectionNumber();

	return true;
}

const DWORD CPsiSection::CompareSection(const IPsiSection *pSrc) const
{
	if(!pSrc)return CR_IDENTICAL;

	DWORD dwReturn = CR_IDENTICAL;

	// e[uID`FbN
	if(GetTableID() != pSrc->GetTableID())dwReturn |= CR_TABLE_ID;

	// e[uIDg`FbN
	if(GetTableIdExtension() != pSrc->GetTableIdExtension())dwReturn |= CR_EXTENSION;

	// ZNVԍ
	if(GetSectionNumber() != pSrc->GetSectionNumber())dwReturn |= CR_EXTENSION;

	// XgZNVԍ
	if(GetLastSectionNumber() != pSrc->GetLastSectionNumber())dwReturn |= CR_EXTENSION;

	// ZNV̓er
	if(GetPayloadSize() != pSrc->GetPayloadSize())dwReturn |= CR_PAYLOAD;

	const BYTE *pSrcData = GetPayloadData();
	const BYTE *pDstData = pSrc->GetPayloadData();
	
	// yC[hoCir
	for(DWORD dwPos = 0 ; dwPos < GetPayloadSize() ; dwPos++){
		if(pSrcData[dwPos] != pDstData[dwPos]){
			dwReturn |= CR_PAYLOAD;
			break;
			}
		}
		
	// CRCr
	if(GetSectionCrc() != pSrc->GetSectionCrc())dwReturn |= CR_OTHER_ALL;
	
	
	// v
	return dwReturn;
}

const DWORD CPsiSection::CopyData(const IMediaData *pSrc)
{
	// IPsiSectionC^tF[X擾
	const IPsiSection * const pPsiSection = dynamic_cast<const IPsiSection *>(pSrc);

	if(pPsiSection){
		// ITsPacketƂăRs[
		return (CopySection(pPsiSection))? GetSize() : 0UL;
		}
	else{
		// IMediaDataƂăRs[
		return CMediaData::CopyData(pSrc);
		}
}

IMediaData *CPsiSection::Clone(IMediaDataHeap *pHeap) const
{
	// ftHg̎I[o[ChāAK؂Ȍ^ŐKv
	CPsiSection *pSection = InternalClone<CPsiSection>(pHeap);
	pSection->m_Header = m_Header;
	return pSection;
}

CPsiSection::CPsiSection(IBonObject *pOwner, IMediaDataHeap *pHeap)
	: CMediaData(pOwner, pHeap)
{
	// f[^
	Reset();
}

CPsiSection::~CPsiSection(void)
{

}


/////////////////////////////////////////////////////////////////////////////
// IPsiSectionParserNX
/////////////////////////////////////////////////////////////////////////////

void CPsiSectionParser::SetExtSection(const bool bExtSection)
{
	// WZNVƊgZNVݒ肷
	if(m_bTargetExt != bExtSection){
		m_bTargetExt = bExtSection;
		Reset();
		}
}

const bool CPsiSectionParser::StorePacket(const ITsPacket *pTsPacket)
{
	const BYTE * const pData = pTsPacket->GetPayloadData();
	const BYTE bySize = pTsPacket->GetPayloadSize();

	if(!bySize || !pData)return false;

	const BYTE byUnitStartPos = (pTsPacket->m_Header.bPayloadUnitStartIndicator)? (pData[0] + 1U) : 0U;

	if(byUnitStartPos){
		// [wb_f | yC[hf] + [X^btBOoCg] + wb_擪 + [wb_f] + [yC[hf] + [X^btBOoCg]
		BYTE byPos = 1U;
		
		if(byUnitStartPos > 1U){
			// jbgJnʒu擪ł͂Ȃꍇ(fЂꍇ)
			byPos += StoreHeader(&pData[byPos], bySize - byPos);
			byPos += StorePayload(&pData[byPos], bySize - byPos);
			}
		
		// jbgJnʒuVKZNṼXgAJn
		m_bIsStoring = false;
		m_PsiSection.ClearSize();

		byPos = byUnitStartPos;
		byPos += StoreHeader(&pData[byPos], bySize - byPos);
		byPos += StorePayload(&pData[byPos], bySize - byPos);
		}
	else{
		// [wb_f] + yC[h + [X^btBOoCg]
		BYTE byPos = 0U;
		byPos += StoreHeader(&pData[byPos], bySize - byPos);
		byPos += StorePayload(&pData[byPos], bySize - byPos);
		}

	return true;
}

void CPsiSectionParser::Reset(void)
{
	// ԂZbg
	m_bIsStoring = false;
	m_wStoreSize = 0U;
	m_dwCrcErrorNum = 0UL;
	m_PsiSection.Reset();
}

const DWORD CPsiSectionParser::GetCrcErrorNum(void) const
{
	// CRCG[Ԃ
	return m_dwCrcErrorNum;
}

CPsiSectionParser::CPsiSectionParser(IBonObject *pOwner)
	: m_pHandler(dynamic_cast<IHandler *>(pOwner))
	, m_PsiSection(NULL)
	, m_bTargetExt(true)
	, m_bIsStoring(false)
	, m_dwCrcErrorNum(0UL)
{
	if(!m_pHandler)::DebugBreak();
}

CPsiSectionParser::~CPsiSectionParser(void)
{

}

const BYTE CPsiSectionParser::StoreHeader(const BYTE *pPayload, const BYTE byRemain)
{
	// wb_͂ăZNṼXgAJn
	if(m_bIsStoring)return 0U;

	const BYTE byHeaderSize = (m_bTargetExt)? 8U : 3U;
	const BYTE byHeaderRemain = byHeaderSize - (BYTE)m_PsiSection.GetSize();

	if(byRemain >= byHeaderRemain){
		// wb_XgAAwb_͂ăyC[h̃XgAJn
		m_PsiSection.AddData(pPayload, byHeaderRemain);
		if(m_PsiSection.ParseHeader(m_bTargetExt)){
			// wb_tH[}bgOKAwb_݂̂CRCvZ
			m_wStoreSize = m_PsiSection.GetSectionLength() + 3U;
			m_dwStoreCrc = CALCULATE_CRC(pPayload, byHeaderSize, 0xffffffffUL);
			m_bIsStoring = true;
			return byHeaderRemain;
			}
		else{
			// wb_G[
			m_PsiSection.Reset();
			return byRemain;
			}
		}
	else{
		// wb_XgAÃf[^҂
		m_PsiSection.AddData(pPayload, byRemain);
		return byRemain;
		}
}

const BYTE CPsiSectionParser::StorePayload(const BYTE *pPayload, const BYTE byRemain)
{
	// ZNṼXgA
	if(!m_bIsStoring)return 0U;
	
	const WORD wStoreRemain = m_wStoreSize - (WORD)m_PsiSection.GetSize();

	if(wStoreRemain <= (WORD)byRemain){
		// XgA
		m_PsiSection.AddData(pPayload, wStoreRemain);
				
		if(!CALCULATE_CRC(pPayload, wStoreRemain, m_dwStoreCrc)){
			// CRCAnhɃZNVn
			if(m_pHandler)m_pHandler->OnPsiSection(this, &m_PsiSection);
			}
		else{
			// CRCُ
			if(m_dwCrcErrorNum < 0xFFFFFFFFUL)m_dwCrcErrorNum++;
			}
		
		// ԂÃZNVMɔ
		m_PsiSection.Reset();
		m_bIsStoring = false;

		return (BYTE)wStoreRemain;
		}
	else{
		// XgAÃyC[h҂
		m_PsiSection.AddData(pPayload, byRemain);
		m_dwStoreCrc = CALCULATE_CRC(pPayload, byRemain, m_dwStoreCrc);
		return byRemain;
		}
}
