//////////////////////////////////////////////////////////////////////////
// BonSDK EARTHSOFT PT1 tuner library
// Copyright (c) http://2sen.dip.jp/dtv/
//
// This is part of BonSDK.
// BonSDK comes with ABSOLUTELY NO WARRANTY;
// for details see 'GNU GENERAL PUBLIC LICENSE 2.0'.

// PtBaseTuner.cpp: PT`[iNX
//
/////////////////////////////////////////////////////////////////////////////

#include "StdAfx.h"
#include "BonSDK.h"
#include "PtBaseTuner.h"
#include <TChar.h>


/////////////////////////////////////////////////////////////////////////////
// t@C[J萔ݒ
/////////////////////////////////////////////////////////////////////////////

// foCX
#define DEVICENAME_TER	TEXT("Earthsoft PT1 ISDB-T fW^`[i")
#define DEVICENAME_SAT	TEXT("Earthsoft PT1 ISDB-S fW^`[i")

// obt@ݒ
#define FIFOBUFFTIME	2UL														// obt@ = 2b
#define FIFOBLOCKSIZE	(CPtBoard::DMABLK_SIZE / CPtBoard::DMACHK_UNIT / 3UL)	// ubNTCY
#define FIFOBUFFNUM		(0x3C0000UL / FIFOBLOCKSIZE * FIFOBUFFTIME)				// 30MbpsƂ


/////////////////////////////////////////////////////////////////////////////
// PT1`[ivpeBNX
/////////////////////////////////////////////////////////////////////////////

const bool CPtBaseTunerProperty::LoadProperty(IConfigStorage *pStorage)
{
	if(!pStorage)return false;

	// ݒǂݏo
	m_bDefaultLnbPower = pStorage->GetBoolItem(TEXT("DefaultLnbPower"	), false);
	m_bEnableVHF	   = pStorage->GetBoolItem(TEXT("EnableVHF"			), true	);
	m_bEnableCATV	   = pStorage->GetBoolItem(TEXT("EnableCATV"		), true	);
	m_bEnableCS		   = pStorage->GetBoolItem(TEXT("EnableCS"			), true	);
	m_bEnableBS		   = pStorage->GetBoolItem(TEXT("EnableBS"			), true	);

	return true;
}

const bool CPtBaseTunerProperty::SaveProperty(IConfigStorage *pStorage)
{
	if(!pStorage)return false;
	
	// ݒۑ
	try{
		if(!pStorage->SetBoolItem(TEXT("DefaultLnbPower"), m_bDefaultLnbPower	))throw ::BON_EXPECTION();
		if(!pStorage->SetBoolItem(TEXT("EnableVHF"		), m_bEnableVHF			))throw ::BON_EXPECTION();
		if(!pStorage->SetBoolItem(TEXT("EnableCATV"		), m_bEnableCATV		))throw ::BON_EXPECTION();
		if(!pStorage->SetBoolItem(TEXT("EnableCS"		), m_bEnableCS			))throw ::BON_EXPECTION();
		if(!pStorage->SetBoolItem(TEXT("EnableBS"		), m_bEnableBS			))throw ::BON_EXPECTION();
		}
	catch(const CBonException &ex){
		// G[
		::BON_TRACE(TraceType_Error, ex.GetDescription());
		return false;
		}
	
	return true;
}

IConfigTarget * CPtBaseTunerProperty::GetConfigTarget(void)
{
	// RtBO^[QbgԂ
	return m_pConfigTarget;
}

const bool CPtBaseTunerProperty::CopyProperty(const IConfigProperty *pProperty)
{
	const IPtBaseTunerProperty *pPtBaseTunerProperty = dynamic_cast<const IPtBaseTunerProperty *>(pProperty);
	if(!pPtBaseTunerProperty)return false;
	
	// vpeBRs[
	*(static_cast<IPtBaseTunerProperty *>(this)) = *pPtBaseTunerProperty;
	
	return true;
}

const CBonString CPtBaseTunerProperty::GetDialogClassName()
{
	// vpeB_CAO͔
	return CBonString::GetEmpty();
}

CPtBaseTunerProperty::CPtBaseTunerProperty(IBonObject *pOwner)
	: CBonObject(pOwner)
	, m_pConfigTarget(dynamic_cast<IConfigTarget *>(pOwner))
{
	// ftHglݒ
	m_bDefaultLnbPower = false;
	m_bEnableVHF = true;
	m_bEnableCATV = true;
	m_bEnableCS = true;
	m_bEnableBS = true;
}

CPtBaseTunerProperty::~CPtBaseTunerProperty(void)
{
	// Ȃ
}


/////////////////////////////////////////////////////////////////////////////
// PT1`[iNX
/////////////////////////////////////////////////////////////////////////////

CPtManager *CPtBaseTuner::m_pPtManager = NULL;
DWORD CPtBaseTuner::m_dwInstanceCount = 0UL;

const BONGUID CPtBaseTuner::GetDeviceType(void) const
{
	// foCX̃^CvԂ
	return ::BON_NAME_TO_GUID(TEXT("IHalTsTuner"));
}

const CBonString CPtBaseTuner::GetDeviceName() const
{
	static const CBonString deviceName[] =
	{
		CBonString(DEVICENAME_TER), CBonString(DEVICENAME_SAT)
	};

	// foCXԂ
	return deviceName[m_dwTunerType];
}

const DWORD CPtBaseTuner::GetTotalDeviceNum(void) const
{
	// `[ȋԂ
	return m_pPtManager->GetTotalTunerNum(m_dwTunerType);
}

const DWORD CPtBaseTuner::GetActiveDeviceNum(void) const
{
	// gp̃`[iԂ
	return m_pPtManager->GetActiveTunerNum(m_dwTunerType);
}

const DWORD CPtBaseTuner::OpenTuner(void)
{
	// UN[Y
	CloseTuner();
	
	try{
		// `[iI[v
		if(!(m_pTuner = m_pPtManager->OpenTuner(m_dwTunerType)))throw ::BON_EXPECTION(TEXT("`[iI[vs"));

		// LNBdݒ
		if(m_dwTunerType == Device::ISDB_S){
			if(!SetLnbPower(m_Property.m_bDefaultLnbPower))throw ::BON_EXPECTION(TEXT("LNBdݒ莸s"));
			}

		// ftHg`lݒ
		SetChannel(m_dwCurSpace, m_dwCurChannel);

		// Xg[MJn
		if(!m_pTuner->StartStream(this))throw ::BON_EXPECTION(TEXT("TSXg[MJns"));
		}
	catch(const CBonException &){
		// G[
		CloseTuner();
		return EC_OPEN_FAILED;
		}
	
	return EC_OPEN_SUCCESS;
}

void CPtBaseTuner::CloseTuner(void)
{
	// `[iN[Y
	if(m_pTuner){
		m_pTuner->StopStream();
		m_pTuner->CloseTuner();
		m_pTuner = NULL;
		}
}

const DWORD CPtBaseTuner::GetDeviceID(void) const
{
	return m_pTuner->GetTunerID();
}

const DWORD CPtBaseTuner::GetCurSpace(void) const
{
	// ݂̃`[jOԂԂ
	return m_dwCurSpace;
}

const DWORD CPtBaseTuner::GetCurChannel(void) const
{
	// ݂̃`lԂ
	return m_dwCurChannel;
}

const bool CPtBaseTuner::SetChannel(const DWORD dwSpace, const DWORD dwChannel)
{
	// ȂAhNXŃI[o[ChȂ΂ȂȂ
	::BON_ASSERT(false);
	
	return false;
}

const bool CPtBaseTuner::SetLnbPower(const bool bEnable)
{
	// LNBd
	return (m_pTuner)? m_pTuner->SetLnbPower(bEnable) : false;
}

const float CPtBaseTuner::GetSignalLevel(void) const
{
	// CNR擾
	return (m_pTuner)? m_pTuner->GetSignalLevel() : 0.0f;
}

const bool CPtBaseTuner::SetProperty(const IConfigProperty *pProperty)
{
	if(!pProperty)return false;

	// vpeBݒ
	return m_Property.CopyProperty(pProperty);
}

const bool CPtBaseTuner::GetProperty(IConfigProperty *pProperty)
{
	if(!pProperty)return false;

	// vpeB擾
	return pProperty->CopyProperty(&m_Property);
}

const CBonString CPtBaseTuner::GetPropertyClassName()
{
	static const CBonString propertyClassName(TEXT("CPtBaseTunerProperty"));

	// vpeBNX擾
	return propertyClassName;
}

const bool CPtBaseTuner::SetOutputDecoder(IMediaDecoder *pDecoder)
{
	m_pOutputDecoder = pDecoder;
	return true;
}

CPtBaseTuner::CPtBaseTuner(IBonObject *pOwner, const Device::ISDB dwTunerType)
	: CBonObject(pOwner)
	, m_Property(static_cast<IConfigTarget *>(this))
	, m_pTuner(NULL)
	, m_dwTunerType(dwTunerType)
	, m_dwCurSpace(0UL)
	, m_dwCurChannel(0UL)
	, m_pStandardTuningSpace(BON_SAFE_CREATE<IStandardTuningSpace *>(TEXT("CStandardTuningSpace"), 0))
	, m_pOutputDecoder(0)
{
	// }l[WCX^X
	if(!m_dwInstanceCount++){
		m_pPtManager = new CPtManager;
		}

	::BON_ASSERT(m_pPtManager != NULL);

	// ftHgvpeBf
	SetProperty(&m_Property);
}

CPtBaseTuner::~CPtBaseTuner(void)
{
	CloseTuner();
	
	// }l[WCX^XJ
	if(!--m_dwInstanceCount){
		BON_SAFE_DELETE(m_pPtManager);
		}

	BON_SAFE_RELEASE(m_pStandardTuningSpace);
}

void CPtBaseTuner::OnTsStream(CPtTuner *pTuner, const BYTE *pData, const DWORD dwSize)
{
	if (m_pOutputDecoder != 0)
	{
		m_pOutputDecoder->InputMedia(pData, dwSize);
	}
}

const CBonString CPtBaseTuner::GetStandardTuningSpaceName(const IStandardTuningSpace::STANDARD_TUNING_SPACES tuningSpace) const
{
	return m_pStandardTuningSpace->GetStandardTuningSpaceName(tuningSpace);
}

const CBonString CPtBaseTuner::GetStandardChannelName(const IStandardTuningSpace::STANDARD_TUNING_SPACES tuningSpace, const DWORD dwChannel) const
{
	return m_pStandardTuningSpace->GetStandardChannelName(tuningSpace, dwChannel);
}
