//////////////////////////////////////////////////////////////////////////
// BonSDK EARTHSOFT PT1 tuner library
// Copyright (c) http://2sen.dip.jp/dtv/
//
// This is part of BonSDK.
// BonSDK comes with ABSOLUTELY NO WARRANTY;
// for details see 'GNU GENERAL PUBLIC LICENSE 2.0'.

// PtBaseTuner.h: PT`[iNX
//
/////////////////////////////////////////////////////////////////////////////

#pragma once


#include "PtDevice.h"
#include "PtDriverIf.h"
#include <vector>
#include <queue>


/////////////////////////////////////////////////////////////////////////////
// PT1`[ivpeBNX
/////////////////////////////////////////////////////////////////////////////

class CPtBaseTunerProperty	:	public CBonObject,
								public IConfigProperty,
								public IPtBaseTunerProperty
{
public:
// CBonObject
	DECLARE_IBONOBJECT(CPtBaseTunerProperty)

// IConfigProperty
	virtual const bool LoadProperty(IConfigStorage *pStorage);
	virtual const bool SaveProperty(IConfigStorage *pStorage);
	virtual const bool CopyProperty(const IConfigProperty *pProperty);
	virtual IConfigTarget * GetConfigTarget(void);
	virtual const CBonString GetDialogClassName();

// CPtBaseTunerProperty
	CPtBaseTunerProperty(IBonObject *pOwner);
	virtual ~CPtBaseTunerProperty(void);
	
protected:
	IConfigTarget *m_pConfigTarget;
};


/////////////////////////////////////////////////////////////////////////////
// PT1`[iNX
/////////////////////////////////////////////////////////////////////////////

class CPtBaseTuner :	public CBonObject,
						public IHalDevice,
						public IHalTsTuner,
						public IConfigTarget,
						protected CPtTuner::ITsReceiver
{
public:
// IHalDevice
	virtual const BONGUID GetDeviceType(void) const;
	virtual const CBonString GetDeviceName() const;
	virtual const DWORD GetTotalDeviceNum(void) const;
	virtual const DWORD GetActiveDeviceNum(void) const;

// IHalTsTuner
	virtual const DWORD OpenTuner(void);
	virtual void CloseTuner(void);
	
	virtual const CBonString EnumTuningSpace(const DWORD dwSpace) const = 0;
	virtual const CBonString EnumChannelName(const DWORD dwSpace, const DWORD dwChannel) const = 0;
	
	virtual const DWORD GetDeviceID(void) const;
	virtual const DWORD GetCurSpace(void) const;
	virtual const DWORD GetCurChannel(void) const;

	virtual const bool SetChannel(const DWORD dwSpace, const DWORD dwChannel);
	virtual const bool SetLnbPower(const bool bEnable);
	virtual const float GetSignalLevel(void) const;

	virtual const bool SetOutputDecoder(IMediaDecoder *pDecoder);

// IConfigTarget
	virtual const bool SetProperty(const IConfigProperty *pProperty);
	virtual const bool GetProperty(IConfigProperty *pProperty);
	virtual const CBonString GetPropertyClassName();

// CPtBaseTuner
	CPtBaseTuner(IBonObject *pOwner, const Device::ISDB dwTunerType = Device::ISDB_T);
	virtual ~CPtBaseTuner(void);

	struct PT_CH_CONFIG
	{
		const DWORD dwChIndex;			// PT1`lCfbNX
		const WORD wTsID;				// TS ID
	};

	struct PT_CH_SPACE
	{
		const PT_CH_CONFIG *pChConfig;	// `lݒ
		const DWORD dwChNum;			// `l
	};

protected:
// CPtTuner::ITsReceiver
	virtual void OnTsStream(CPtTuner *pTuner, const BYTE *pData, const DWORD dwSize);

	const CBonString GetStandardTuningSpaceName(const IStandardTuningSpace::STANDARD_TUNING_SPACES tuningSpace) const;
	const CBonString GetStandardChannelName(const IStandardTuningSpace::STANDARD_TUNING_SPACES tuningSpace, const DWORD dwChannel) const;

	static CPtManager *m_pPtManager;
	static DWORD m_dwInstanceCount;

	IMediaDecoder *m_pOutputDecoder;

	CPtBaseTunerProperty m_Property;

	CPtTuner *m_pTuner;

	const Device::ISDB m_dwTunerType;
	std::vector<const PT_CH_SPACE *> m_TuningSpace;

	DWORD m_dwCurSpace;
	DWORD m_dwCurChannel;

private:
	IStandardTuningSpace *m_pStandardTuningSpace;
};
