//////////////////////////////////////////////////////////////////////////
// BonSDK EARTHSOFT PT2 tuner library
// Copyright (c) http://2sen.dip.jp/dtv/
//
// This is part of BonSDK.
// BonSDK comes with ABSOLUTELY NO WARRANTY;
// for details see 'GNU GENERAL PUBLIC LICENSE 2.0'.

// Pt2BaseTuner.cpp: PT2`[iNX
//
/////////////////////////////////////////////////////////////////////////////

#include "StdAfx.h"
#include "BonSDK.h"
#include "Pt2BaseTuner.h"
#include <TChar.h>


/////////////////////////////////////////////////////////////////////////////
// t@C[J萔ݒ
/////////////////////////////////////////////////////////////////////////////

// foCX
#define DEVICENAME_TER	TEXT("Earthsoft PT2 ISDB-T fW^`[i")
#define DEVICENAME_SAT	TEXT("Earthsoft PT2 ISDB-S fW^`[i")

// obt@ݒ
#define FIFOBUFFTIME	2UL														// obt@ = 2b
#define FIFOBLOCKSIZE	(CPt2Board::DMABLK_SIZE / CPt2Board::DMACHK_UNIT / 3UL)	// ubNTCY
#define FIFOBUFFNUM		(0x3C0000UL / FIFOBLOCKSIZE * FIFOBUFFTIME)				// 30MbpsƂ


/////////////////////////////////////////////////////////////////////////////
// PT2`[ivpeBNX
/////////////////////////////////////////////////////////////////////////////

const bool CPt2BaseTunerProperty::LoadProperty(IConfigStorage *pStorage)
{
	if(!pStorage)return false;

	// ݒǂݏo
	m_bDefaultLnbPower = pStorage->GetBoolItem(TEXT("DefaultLnbPower"	), false);
	m_bEnaleVHF		   = pStorage->GetBoolItem(TEXT("EnaleVHF"			), true	);
	m_bEnaleCATV	   = pStorage->GetBoolItem(TEXT("EnaleCATV"			), true	);
	m_bEnaleCS		   = pStorage->GetBoolItem(TEXT("EnaleCS"			), true	);
	m_bEnaleBS		   = pStorage->GetBoolItem(TEXT("EnaleBS"			), true	);

	return true;
}

const bool CPt2BaseTunerProperty::SaveProperty(IConfigStorage *pStorage)
{
	if(!pStorage)return false;
	
	// ݒۑ
	try{
		if(!pStorage->SetBoolItem(TEXT("DefaultLnbPower"), m_bDefaultLnbPower	))throw ::BON_EXPECTION();
		if(!pStorage->SetBoolItem(TEXT("EnaleVHF"		), m_bEnaleVHF			))throw ::BON_EXPECTION();
		if(!pStorage->SetBoolItem(TEXT("EnaleCATV"		), m_bEnaleCATV			))throw ::BON_EXPECTION();
		if(!pStorage->SetBoolItem(TEXT("EnaleCS"		), m_bEnaleCS			))throw ::BON_EXPECTION();
		if(!pStorage->SetBoolItem(TEXT("EnaleBS"		), m_bEnaleBS			))throw ::BON_EXPECTION();
		}
	catch(const CBonException &ex){
		// G[
		::BON_TRACE(TraceType_Error, ex.GetDescription());
		return false;
		}
	
	return true;
}

IConfigTarget * CPt2BaseTunerProperty::GetConfigTarget(void)
{
	// RtBO^[QbgԂ
	return m_pConfigTarget;
}

const bool CPt2BaseTunerProperty::CopyProperty(const IConfigProperty *pProperty)
{
	const IPt2BaseTunerProperty *pPt2BaseTunerProperty = dynamic_cast<const IPt2BaseTunerProperty *>(pProperty);
	if(!pPt2BaseTunerProperty)return false;
	
	// vpeBRs[
	*(static_cast<IPt2BaseTunerProperty *>(this)) = *pPt2BaseTunerProperty;
	
	return true;
}

const CBonString CPt2BaseTunerProperty::GetDialogClassName()
{
	// vpeB_CAO͔
	return CBonString::GetEmpty();
}

CPt2BaseTunerProperty::CPt2BaseTunerProperty(IBonObject *pOwner)
	: CBonObject(pOwner)
	, m_pConfigTarget(dynamic_cast<IConfigTarget *>(pOwner))
{
	// ftHglݒ
	m_bDefaultLnbPower = false;
	m_bEnaleVHF = true;
	m_bEnaleCATV = true;
	m_bEnaleCS = true;
	m_bEnaleBS = true;
}

CPt2BaseTunerProperty::~CPt2BaseTunerProperty(void)
{
	// Ȃ
}


/////////////////////////////////////////////////////////////////////////////
// PT2`[iNX
/////////////////////////////////////////////////////////////////////////////

CPt2Manager *CPt2BaseTuner::m_pPt2Manager = NULL;
DWORD CPt2BaseTuner::m_dwInstanceCount = 0UL;

const BONGUID CPt2BaseTuner::GetDeviceType(void) const
{
	// foCX̃^CvԂ
	return ::BON_NAME_TO_GUID(TEXT("IHalTsTuner"));
}

const CBonString CPt2BaseTuner::GetDeviceName() const
{
	static const CBonString deviceName[] =
	{
		CBonString(DEVICENAME_TER), CBonString(DEVICENAME_SAT)
	};

	// foCXԂ
	return deviceName[m_dwTunerType];
}

const DWORD CPt2BaseTuner::GetTotalDeviceNum(void) const
{
	// `[ȋԂ
	return m_pPt2Manager->GetTotalTunerNum(m_dwTunerType);
}

const DWORD CPt2BaseTuner::GetActiveDeviceNum(void) const
{
	// gp̃`[iԂ
	return m_pPt2Manager->GetActiveTunerNum(m_dwTunerType);
}

const DWORD CPt2BaseTuner::OpenTuner(void)
{
	// UN[Y
	CloseTuner();
	
	try{
		// `[iI[v
		if(!(m_pTuner = m_pPt2Manager->OpenTuner(m_dwTunerType)))throw ::BON_EXPECTION(TEXT("`[iI[vs"));

		// LNBdݒ
		if(m_dwTunerType == Device::ISDB_S){
			if(!SetLnbPower(m_Property.m_bDefaultLnbPower))throw ::BON_EXPECTION(TEXT("LNBdݒ莸s"));
			}

		// ftHg`lݒ
		SetChannel(m_dwCurSpace, m_dwCurChannel);

		// Xg[MJn
		if(!m_pTuner->StartStream(this))throw ::BON_EXPECTION(TEXT("TSXg[MJns"));
		}
	catch(const CBonException &){
		// G[
		CloseTuner();
		return EC_OPEN_FAILED;
		}
	
	return EC_OPEN_SUCCESS;
}

void CPt2BaseTuner::CloseTuner(void)
{
	// `[iN[Y
	if(m_pTuner){
		m_pTuner->StopStream();
		m_pTuner->CloseTuner();
		m_pTuner = NULL;
		}
}

const DWORD CPt2BaseTuner::GetDeviceID(void) const
{
	return m_pTuner->GetTunerID();
}

const DWORD CPt2BaseTuner::GetCurSpace(void) const
{
	// ݂̃`[jOԂԂ
	return m_dwCurSpace;
}

const DWORD CPt2BaseTuner::GetCurChannel(void) const
{
	// ݂̃`lԂ
	return m_dwCurChannel;
}

const bool CPt2BaseTuner::SetChannel(const DWORD dwSpace, const DWORD dwChannel)
{
	// ȂAhNXŃI[o[ChȂ΂ȂȂ
	::BON_ASSERT(false);
	
	return false;
}

const bool CPt2BaseTuner::SetLnbPower(const bool bEnable)
{
	// LNBd
	return (m_pTuner)? m_pTuner->SetLnbPower(bEnable) : false;
}

const float CPt2BaseTuner::GetSignalLevel(void) const
{
	// CNR擾
	return (m_pTuner)? m_pTuner->GetSignalLevel() : 0.0f;
}

const bool CPt2BaseTuner::SetProperty(const IConfigProperty *pProperty)
{
	if(!pProperty)return false;

	// vpeBݒ
	return m_Property.CopyProperty(pProperty);
}

const bool CPt2BaseTuner::GetProperty(IConfigProperty *pProperty)
{
	if(!pProperty)return false;

	// vpeB擾
	return pProperty->CopyProperty(&m_Property);
}

const CBonString CPt2BaseTuner::GetPropertyClassName()
{
	static const CBonString propertyClassName(TEXT("CPt2BaseTunerProperty"));

	// vpeBNX擾
	return propertyClassName;
}

const bool CPt2BaseTuner::SetOutputDecoder(IMediaDecoder *pDecoder)
{
	m_pOutputDecoder = pDecoder;
	return true;
}

CPt2BaseTuner::CPt2BaseTuner(IBonObject *pOwner, const Device::ISDB dwTunerType)
	: CBonObject(pOwner)
	, m_Property(static_cast<IConfigTarget *>(this))
	, m_pTuner(NULL)
	, m_dwTunerType(dwTunerType)
	, m_dwCurSpace(0UL)
	, m_dwCurChannel(0UL)
	, m_pStandardTuningSpace(BON_SAFE_CREATE<IStandardTuningSpace *>(TEXT("CStandardTuningSpace"), 0))
	, m_pOutputDecoder(0)
{
	// }l[WCX^X
	if(!m_dwInstanceCount++){
		m_pPt2Manager = new CPt2Manager;
		}

	::BON_ASSERT(m_pPt2Manager != NULL);

	// ftHgvpeBf
	SetProperty(&m_Property);
}

CPt2BaseTuner::~CPt2BaseTuner(void)
{
	CloseTuner();
	
	// }l[WCX^XJ
	if(!--m_dwInstanceCount){
		BON_SAFE_DELETE(m_pPt2Manager);
		}

	BON_SAFE_RELEASE(m_pStandardTuningSpace);
}

void CPt2BaseTuner::OnTsStream(CPt2Tuner *pTuner, const BYTE *pData, const DWORD dwSize)
{
	if (m_pOutputDecoder != 0)
	{
		m_pOutputDecoder->InputMedia(pData, dwSize);
	}
}

const CBonString CPt2BaseTuner::GetStandardTuningSpaceName(const IStandardTuningSpace::STANDARD_TUNING_SPACES tuningSpace) const
{
	return m_pStandardTuningSpace->GetStandardTuningSpaceName(tuningSpace);
}

const CBonString CPt2BaseTuner::GetStandardChannelName(const IStandardTuningSpace::STANDARD_TUNING_SPACES tuningSpace, const DWORD dwChannel) const
{
	return m_pStandardTuningSpace->GetStandardChannelName(tuningSpace, dwChannel);
}
