//////////////////////////////////////////////////////////////////////////
// BonSDK EARTHSOFT PT2 tuner library
// Copyright (c) http://2sen.dip.jp/dtv/
//
// This is part of BonSDK.
// BonSDK comes with ABSOLUTELY NO WARRANTY;
// for details see 'GNU GENERAL PUBLIC LICENSE 2.0'.

// Pt2BaseTuner.h: PT2`[iNX
//
/////////////////////////////////////////////////////////////////////////////

#pragma once


#include "Pt2Device.h"
#include "Pt2DriverIf.h"
#include <vector>
#include <queue>


/////////////////////////////////////////////////////////////////////////////
// PT2`[ivpeBNX
/////////////////////////////////////////////////////////////////////////////

class CPt2BaseTunerProperty	:	public CBonObject,
								public IConfigProperty,
								public IPt2BaseTunerProperty
{
public:
// CBonObject
	DECLARE_IBONOBJECT(CPt2BaseTunerProperty)

// IConfigProperty
	virtual const bool LoadProperty(IConfigStorage *pStorage);
	virtual const bool SaveProperty(IConfigStorage *pStorage);
	virtual const bool CopyProperty(const IConfigProperty *pProperty);
	virtual IConfigTarget * GetConfigTarget(void);
	virtual const CBonString GetDialogClassName();

// CPt2BaseTunerProperty
	CPt2BaseTunerProperty(IBonObject *pOwner);
	virtual ~CPt2BaseTunerProperty(void);
	
protected:
	IConfigTarget *m_pConfigTarget;
};


/////////////////////////////////////////////////////////////////////////////
// PT2`[iNX
/////////////////////////////////////////////////////////////////////////////

class CPt2BaseTuner :	public CBonObject,
						public IHalDevice,
						public IHalTsTuner,
						public IConfigTarget,
						protected CPt2Tuner::ITsReceiver
{
public:
// IHalDevice
	virtual const BONGUID GetDeviceType(void) const;
	virtual const CBonString GetDeviceName() const;
	virtual const DWORD GetTotalDeviceNum(void) const;
	virtual const DWORD GetActiveDeviceNum(void) const;

// IHalTsTuner
	virtual const DWORD OpenTuner(void);
	virtual void CloseTuner(void);
	
	virtual const CBonString EnumTuningSpace(const DWORD dwSpace) const = 0;
	virtual const CBonString EnumChannelName(const DWORD dwSpace, const DWORD dwChannel) const = 0;
	
	virtual const DWORD GetDeviceID(void) const;
	virtual const DWORD GetCurSpace(void) const;
	virtual const DWORD GetCurChannel(void) const;

	virtual const bool SetChannel(const DWORD dwSpace, const DWORD dwChannel);
	virtual const bool SetLnbPower(const bool bEnable);
	virtual const float GetSignalLevel(void) const;

	virtual const bool SetOutputDecoder(IMediaDecoder *pDecoder);

// IConfigTarget
	virtual const bool SetProperty(const IConfigProperty *pProperty);
	virtual const bool GetProperty(IConfigProperty *pProperty);
	virtual const CBonString GetPropertyClassName();

// CPt2BaseTuner
	CPt2BaseTuner(IBonObject *pOwner, const Device::ISDB dwTunerType = Device::ISDB_T);
	virtual ~CPt2BaseTuner(void);

	struct PT_CH_CONFIG
	{
		const DWORD dwChIndex;			// PT2`lCfbNX
		const WORD wTsID;				// TS ID
	};

	struct PT_CH_SPACE
	{
		const PT_CH_CONFIG *pChConfig;	// `lݒ
		const DWORD dwChNum;			// `l
	};

protected:
// CPt2Tuner::ITsReceiver
	virtual void OnTsStream(CPt2Tuner *pTuner, const BYTE *pData, const DWORD dwSize);

	const CBonString GetStandardTuningSpaceName(const IStandardTuningSpace::STANDARD_TUNING_SPACES tuningSpace) const;
	const CBonString GetStandardChannelName(const IStandardTuningSpace::STANDARD_TUNING_SPACES tuningSpace, const DWORD dwChannel) const;

	static CPt2Manager *m_pPt2Manager;
	static DWORD m_dwInstanceCount;

	IMediaDecoder *m_pOutputDecoder;

	CPt2BaseTunerProperty m_Property;

	CPt2Tuner *m_pTuner;

	const Device::ISDB m_dwTunerType;
	std::vector<const PT_CH_SPACE *> m_TuningSpace;

	DWORD m_dwCurSpace;
	DWORD m_dwCurChannel;

private:
	IStandardTuningSpace *m_pStandardTuningSpace;
};
