//////////////////////////////////////////////////////////////////////////
// BonSDK EARTHSOFT PT2 tuner library
// Copyright (c) http://2sen.dip.jp/dtv/
//
// This is part of BonSDK.
// BonSDK comes with ABSOLUTELY NO WARRANTY;
// for details see 'GNU GENERAL PUBLIC LICENSE 2.0'.

// Pt2Device.cpp: PT2foCXNX
//
/////////////////////////////////////////////////////////////////////////////

#include "StdAfx.h"
#include "BonSDK.h"
#include "Pt2Device.h"
#include <TChar.h>
#include <MmSystem.h>


#pragma comment(lib, "WinMm.lib")


/////////////////////////////////////////////////////////////////////////////
// PT2}l[WNX
/////////////////////////////////////////////////////////////////////////////

CPt2Manager::CPt2Manager(void)
	: m_pPtBus(NULL)
	, m_pPtLibrary(new Library)
{
	// ^C}
	::timeBeginPeriod(1UL);

	::BON_ASSERT(m_pPtLibrary != NULL);
	EnumerateBoards();
}

CPt2Manager::~CPt2Manager(void)
{
	::BON_ASSERT(ReleaseInstance());

	BON_SAFE_DELETE(m_pPtLibrary);

	// ^C}J
	::timeEndPeriod(1UL);
}

const DWORD CPt2Manager::GetTotalTunerNum(const Device::ISDB dwTunerType)
{
	// ĂPT2̃`[iԂ
	return m_Boards.size() * CPt2Board::TUNER_NUM;
}

const DWORD CPt2Manager::GetActiveTunerNum(const Device::ISDB dwTunerType)
{
	// gpĂPT2̃`[iԂ
	DWORD dwActiveNum = 0UL;

	for(DWORD dwIndex = 0UL ; dwIndex < m_Boards.size() ; dwIndex++){
		dwActiveNum += m_Boards[dwIndex]->GetActiveTunerNum(dwTunerType);
		}

	return dwActiveNum;
}

CPt2Tuner * CPt2Manager::OpenTuner(const Device::ISDB dwTunerType)
{
	// gp̃`[iĕԂ
	for(DWORD dwBoardID = 0UL ; dwBoardID < m_Boards.size() ; dwBoardID++){
		CPt2Tuner *pTuner = m_Boards[dwBoardID]->OpenTuner(dwTunerType);
		if(pTuner)return pTuner;
		}

	return NULL;
}

const bool CPt2Manager::EnumerateBoards(void)
{
	// PT2{[h񋓂
	try{
		// BusCX^Xt@Ng[擾
		Bus::NewBusFunction pfnBusFactory = m_pPtLibrary->Function();
		if(!pfnBusFactory)throw ::BON_EXPECTION(TEXT("SDK_EARTHSOFT_PT2.dll[hł܂"));

		// BusCX^X
		if(pfnBusFactory(&m_pPtBus) != STATUS_OK)throw ::BON_EXPECTION(TEXT("BusCX^Xs"));
		
		// PT2{[h
		Bus::DeviceInfo DeviceInfo[8];
		DWORD dwDeviceInfoNum = sizeof(DeviceInfo) / sizeof(*DeviceInfo);
		
		if(m_pPtBus->Scan(DeviceInfo, &dwDeviceInfoNum) != STATUS_OK)throw ::BON_EXPECTION(TEXT("PT{[h񋓎s"));

		// PT2foCXCX^X
		for(DWORD dwBoardID = 0UL ; dwBoardID < dwDeviceInfoNum ; dwBoardID++){
			// SĂPT2{[h̃foCXCX^X𐶐
			Device *pNewDevice = NULL;
			
			if(m_pPtBus->NewDevice(&DeviceInfo[dwBoardID], &pNewDevice) != STATUS_OK)throw ::BON_EXPECTION(TEXT("PTfoCXCX^Xs"));
			
			// Xgɒǉ
			m_Boards.push_back(new CPt2Board(dwBoardID, pNewDevice, &DeviceInfo[dwBoardID]));
			}
		}
	catch(const CBonException &){
		// G[
		ReleaseInstance();
		return false;
		}
	
	return true;
}

const bool CPt2Manager::ReleaseInstance(void)
{
	// PT{[h̃CX^XJ
	for(DWORD dwBoardID = 0UL ; dwBoardID < m_Boards.size() ; dwBoardID++){
		delete m_Boards[dwBoardID];
		}
		
	m_Boards.clear();

	// oXCX^XJ
	if(m_pPtBus){
		m_pPtBus->Delete();
		m_pPtBus = NULL;
		}

	return true;
}


/////////////////////////////////////////////////////////////////////////////
// PT{[hNX
/////////////////////////////////////////////////////////////////////////////

CPt2Board::CPt2Board(const DWORD dwBoardID, Device *pDevice, const Bus::DeviceInfo *pDeviceInfo)
	: m_dwBoardID(dwBoardID)
	, m_pDevice(pDevice)
	, m_DeviceInfo(*pDeviceInfo)
	, m_StreamingThread(new CFlagSignal(1))
{
	::BON_ASSERT(m_pDevice != NULL);

	// `[iCX^X
	for(DWORD dwTunerID = 0UL ; dwTunerID < TUNER_NUM ; dwTunerID++){
		m_TunerList.push_back(new CPt2Tuner(*this, dwTunerID, Device::ISDB_S));
		m_TunerList.push_back(new CPt2Tuner(*this, dwTunerID, Device::ISDB_T));
		}
}

CPt2Board::~CPt2Board(void)
{
	// `[iCX^XJ
	for(DWORD dwIndex = 0UL ; dwIndex < m_TunerList.size() ; dwIndex++){
		delete m_TunerList[dwIndex];
		}

	// foCXCX^XJ
	m_pDevice->Delete();
}

CPt2Tuner * CPt2Board::OpenTuner(const Device::ISDB dwTunerType)
{
	CBlockLock AutoLock(&m_TunerLock);
	CPt2Tuner *pOpenTuner = NULL;

	// I[v`[i(Aei[qS1/T2D)
	DWORD dwTunerID = m_TunerList.size();
	
	while(dwTunerID--){
		if(!m_TunerList[dwTunerID]->m_bIsOpen && (m_TunerList[dwTunerID]->m_dwTunerType == dwTunerType)){
			// gp`[iނv
			pOpenTuner = m_TunerList[dwTunerID];
			}
		}

	if(!pOpenTuner)return NULL;
	
	if(!GetOpenTunerNum()){
		// S`[igp̏ꍇ̓{[hI[v
		if(!OpenBoard())return NULL;
		}
		
	// I[vtOZbg
	pOpenTuner->m_bIsOpen = true;

	return pOpenTuner;
}

void CPt2Board::CloseTuner(CPt2Tuner *pTuner)
{
	CBlockLock AutoLock(&m_TunerLock);

	if(!IsValidTuner(pTuner))return;
	if(!pTuner->m_bIsOpen)return;

	// Xg[~AI[vtONA
	pTuner->StopStream();
	pTuner->m_bIsOpen = false;

	if(!GetOpenTunerNum()){
		// S`[igp̏ꍇ̓{[hN[Y
		CloseBoard();
		}
}

const DWORD CPt2Board::GetActiveTunerNum(const Device::ISDB dwTunerType)
{
	// {[hO[obNMutex݊mF	u/oXԍ/Xbgԍ/t@NV/PT2/vZXID/v
	TCHAR szMutexName[1024];
	::_stprintf(szMutexName, TEXT("/%lu/%lu/%lu/PT2/"), m_DeviceInfo.Bus, m_DeviceInfo.Slot, m_DeviceInfo.Function);

	if(!CSmartMutex::IsExist(szMutexName)){
		// {[hgp̏ꍇ
		return 0UL;
		}
	else if(!GetOpenTunerNum()){
		// vZX{[hgpĂꍇ
		return TUNER_NUM;
		}
	else{
		// vZX{[hgpĂꍇ
		DWORD dwActiveNum = 0UL;

		for(DWORD dwIndex = 0UL ; dwIndex < m_TunerList.size() ; dwIndex++){
			if(m_TunerList[dwIndex]->m_dwTunerType == dwTunerType){
				if(m_TunerList[dwIndex]->m_bIsOpen)dwActiveNum++;
				}
			}
		
		// gp̃`[iԂ
		return dwActiveNum;
		}
}

const bool CPt2Board::StartStream(CPt2Tuner *pTuner)
{
	if(!IsValidTuner(pTuner))return false;
	if(pTuner->m_bIsStream)return true;

	try{
		// Xg[]Jn
		if(m_pDevice->SetStreamEnable(pTuner->m_dwTunerID, pTuner->m_dwTunerType, true) != STATUS_OK)::BON_EXPECTION(TEXT("PT`[iXg[]Jns"));

		if(!GetStreamTunerNum()){
			// Xg[~OXbhJn
			if(!m_StreamingThread.StartThread(TEXT("CPt2Board"), this, &CPt2Board::StreamingThread, NULL, true))throw ::BON_EXPECTION(TEXT("Xg[~OXbhNs"));

			// |[O߂΁ATHREAD_PRIORITY_TIME_CRITICALɈڍsł
			m_StreamingThread.SetPriority(THREAD_PRIORITY_HIGHEST);
			}
		}
	catch(const CBonException &ex){
		// G[
		::BON_TRACE(TraceType_Error, ex.GetDescription());
		return false;
		}

	// Xg[MtOZbg
	pTuner->m_bIsStream = true;

	return true;
}

void CPt2Board::StopStream(CPt2Tuner *pTuner)
{
	if(!IsValidTuner(pTuner))return;
	if(!pTuner->m_bIsStream)return;

	// Xg[MtONA
	pTuner->m_bIsStream = false;

	if(!GetStreamTunerNum()){
		// Xg[~OXbh~
		m_StreamingThread.EndThread();
		}

	// Xg[]I
	::BON_ASSERT(m_pDevice->SetStreamEnable(pTuner->m_dwTunerID, pTuner->m_dwTunerType, false) == STATUS_OK, TEXT("PT`[iXg[]~s"));
}

const bool CPt2Board::SetLnbPower(CPt2Tuner *pTuner, const bool bEnable)
{
	if(!IsValidTuner(pTuner))return false;
	if(pTuner->m_dwTunerType != Device::ISDB_S)return false;

	// XVOLNBdԎ擾
	const bool bLastState = (GetLnbOnTunerNum())? true : false;
	
	// LNBdԍXV
	pTuner->m_bIsLnbOn = bEnable;

	// XVLNBdԎ擾
	const bool bNewState = (GetLnbOnTunerNum())? true : false;
	if(bLastState == bNewState)return true;			

	// LNBdԐݒ
	::BON_ASSERT(m_pDevice->SetLnbPower((bNewState)? Device::LNB_POWER_15V : Device::LNB_POWER_OFF) == STATUS_OK, TEXT("PT`[iLNBd䎸s"));
	
	return true;
}

const bool CPt2Board::IsValidTuner(const CPt2Tuner *pTuner) const
{
	// Lȃ`[iCX^X̗LԂ
	for(DWORD dwIndex = 0UL ; dwIndex < m_TunerList.size() ; dwIndex++){
		if(m_TunerList[dwIndex] == pTuner)return true;
		}

	return false;
}

const DWORD CPt2Board::GetOpenTunerNum(void) const
{
	// I[vς݃`[iԂ
	DWORD dwOpenNum = 0UL;

	for(DWORD dwIndex = 0UL ; dwIndex < m_TunerList.size() ; dwIndex++){
		if(m_TunerList[dwIndex]->m_bIsOpen)dwOpenNum++;
		}

	return dwOpenNum;
}

const DWORD CPt2Board::GetLnbOnTunerNum(void) const
{
	// LNBdOÑ`[iԂ
	DWORD dwLnbOnNum = 0UL;

	for(DWORD dwIndex = 0UL ; dwIndex < m_TunerList.size() ; dwIndex++){
		if((m_TunerList[dwIndex]->m_dwTunerType == Device::ISDB_S) && m_TunerList[dwIndex]->m_bIsLnbOn)dwLnbOnNum++;
		}

	return dwLnbOnNum;
}

const DWORD CPt2Board::GetStreamTunerNum(void) const
{
	// Xg[M̃`[iԂ
	DWORD dwOpenNum = 0UL;

	for(DWORD dwIndex = 0UL ; dwIndex < m_TunerList.size() ; dwIndex++){
		if(m_TunerList[dwIndex]->m_bIsStream)dwOpenNum++;
		}

	return dwOpenNum;
}

const bool CPt2Board::OpenBoard(void)
{
	// {[hI[v
	try{
		// foCXI[v
		if(m_pDevice->Open() != STATUS_OK)throw ::BON_EXPECTION(TEXT("PTfoCXI[vs"));
		
		// `[i[uI/Cl[uv
		if(m_pDevice->SetTunerPowerReset(Device::TUNER_POWER_ON_RESET_ENABLE) != STATUS_OK)throw ::BON_EXPECTION(TEXT("PT`[iZbgڍss"));

		// 20msEFCg
		::Sleep(20UL);

		// `[i[uI/fBZ[uv
		if(m_pDevice->SetTunerPowerReset(Device::TUNER_POWER_ON_RESET_DISABLE) != STATUS_OK)throw ::BON_EXPECTION(TEXT("PT`[iZbgs"));

		// 1msEFCg
		::Sleep(1UL);
		
		for(DWORD dwTunerID = 0UL ; dwTunerID < TUNER_NUM ; dwTunerID++){
			// `[iW[
			if(m_pDevice->InitTuner(dwTunerID) != STATUS_OK)throw ::BON_EXPECTION(TEXT("PT`[is"));

			// `[iX[v
			for(DWORD dwTunerType = 0UL ; dwTunerType < Device::ISDB_COUNT ; dwTunerType++){
				if(m_pDevice->SetTunerSleep(dwTunerID, (Device::ISDB)dwTunerType, false) != STATUS_OK)throw ::BON_EXPECTION(TEXT("PT`[iȓd͉s"));
				}
			}
		
		// {[hO[obNMutex쐬	u/oXԍ/Xbgԍ/t@NV/PT2/v
		TCHAR szMutexName[1024];
		::_stprintf(szMutexName, TEXT("/%lu/%lu/%lu/PT2/"), m_DeviceInfo.Bus, m_DeviceInfo.Slot, m_DeviceInfo.Function);
		if(!m_BoardLock.Create(szMutexName))throw ::BON_EXPECTION(TEXT("PT{[hO[obNMutex쐬s"));
		}
	catch(const CBonException &){
		// G[
		CloseBoard();		
		return false;
		}

	return true;
}

void CPt2Board::CloseBoard(void)
{
	// {[hN[Y
	try{
		// `[iX[vڍs
		for(DWORD dwTunerID = 0UL ; dwTunerID < TUNER_NUM ; dwTunerID++){
			for(DWORD dwTunerType = 0UL ; dwTunerType < Device::ISDB_COUNT ; dwTunerType++){
				if(m_pDevice->SetTunerSleep(dwTunerID, (Device::ISDB)dwTunerType, false) != STATUS_OK)throw ::BON_EXPECTION(TEXT("PT`[iȓd͈ڍss"));
				}
			}	

		// {[hN[Y
		if(m_pDevice->Close() != STATUS_OK)throw ::BON_EXPECTION(TEXT("PTfoCXN[Ys"));
		}
	catch(const CBonException &ex){
		// G[
		::BON_TRACE(TraceType_Error, ex.GetDescription());
		}

	// {[hbNMutexJ
	m_BoardLock.Close();
}

void CPt2Board::StreamingThread(CSmartThread<CPt2Board> &thread, const ISignal &bKillSignal, const ULONG_PTR param)
{
	struct PT_DMABUF_SET
	{
		DWORD *pdwPacket;			// obt@|C^
		DWORD *pdwEndMark;			// ]}[Ñ|C^
		PT_DMABUF_SET *pNextSet;	// ̃obt@Zbgւ̃|C^
	};

	// DMAobt@Zbg
	PT_DMABUF_SET aDmaBufSet[DMABUF_SIZE * DMABUF_NUM * DMACHK_UNIT];
	PT_DMABUF_SET *pDmaBufSet = aDmaBufSet;

	// fBXpb`obt@
	DWORD dwWaitTime = 0UL;
	DWORD dwDispatchTerm = 0UL;
	CMediaData DispatchBuffer(0);
	DispatchBuffer.GetBuffer(DMABLK_SIZE);

	try{
		// DMAobt@TCYݒ
		const static Device::BufferInfo BufInfo = {DMABUF_SIZE, DMABUF_NUM, DMABUF_SIZE};
		if(m_pDevice->SetBufferInfo(&BufInfo) != STATUS_OK)throw ::BON_EXPECTION(TEXT("PTfoCXDMAobt@mێs"));

		// DMAobt@|C^擾(󃊃Xg`)
		for(DWORD dwBuffIndex = 0UL ; dwBuffIndex < DMABUF_NUM ; dwBuffIndex++){
			// obt@|C^擾		
			DWORD *pdwPointer = NULL;
			if(m_pDevice->GetBufferPtr(dwBuffIndex, (PVOID*)&pdwPointer) != STATUS_OK)throw ::BON_EXPECTION(TEXT("PTfoCXDMAobt@|C^擾s"));
		
			for(DWORD dwBlockIndex = 0UL ; dwBlockIndex < (DMABUF_SIZE * DMACHK_UNIT) ; dwBlockIndex++){
				// obt@|C^Zbg
				pDmaBufSet->pdwPacket = &pdwPointer[0]; 
			
				// ]}[N|C^Zbg
				pDmaBufSet->pdwEndMark = &pdwPointer[DMABLK_SIZE / DMACHK_UNIT / 4 - 1];
				pdwPointer = &pDmaBufSet->pdwEndMark[1];
		
				// ̃Zbgւ̃|C^Zbg
				pDmaBufSet->pNextSet = &pDmaBufSet[1];
				pDmaBufSet++;
				}
			}

		// ŏIvf̎Zbg|C^␳
		aDmaBufSet[sizeof(aDmaBufSet) / sizeof(*aDmaBufSet) - 1].pNextSet = aDmaBufSet;

		// DMA]~ꍇ͂蒼(goto͎gȂƂ肠ނȂ)
		StartDmaTransfer:

		pDmaBufSet = aDmaBufSet;

		// ]}[NZbg
		for(DWORD dwIndex = 0UL ; dwIndex < (sizeof(aDmaBufSet) / sizeof(*aDmaBufSet)) ; dwIndex++){
			*aDmaBufSet[dwIndex].pdwEndMark = 0UL;
			}

		// fBXpb`obt@Zbg
		for(DWORD dwIndex = 0UL ; dwIndex < m_TunerList.size() ; dwIndex++){
			m_aDispatchBuff[dwIndex].dwPacketPos = 0UL;
			}
		
		dwDispatchTerm = 0UL;
		DispatchBuffer.ClearSize();

		// DMA]~
		if(m_pDevice->SetTransferEnable(false) != STATUS_OK)throw ::BON_EXPECTION(TEXT("PTfoCXDMA]~s"));
	
		// ]JE^Zbg
		if(m_pDevice->ResetTransferCounter() != STATUS_OK)throw ::BON_EXPECTION(TEXT("PTfoCXDMA]JE^Zbgs"));

		// ]JE^CNg
		for(DWORD dwIndex = 0UL ; dwIndex < (DMABUF_SIZE * DMABUF_NUM) ; dwIndex++){
			if(m_pDevice->IncrementTransferCounter() != STATUS_OK)throw ::BON_EXPECTION(TEXT("PTfoCXDMA]JE^CNgs"));
			}

		// DMA]Jn
		if(m_pDevice->SetTransferEnable(true) != STATUS_OK)throw ::BON_EXPECTION(TEXT("PTfoCXDMA]Jns"));

		// DMA][v
		while(!bKillSignal){
		
			for(DWORD dwUnit = 0UL ; !bKillSignal && (dwUnit < DMACHK_UNIT) ; dwUnit++){

				// ]EFCg
				while(!bKillSignal){

					//	^CAEgJE^Zbg
					dwWaitTime = 0UL;

					// ubN]`FbN
					if(*pDmaBufSet->pdwEndMark){

						// fBXpb`f[^Rs[(DMAobt@CPULbVȂ)
						DispatchBuffer.AddData((const BYTE *)pDmaBufSet->pdwPacket, DMABLK_SIZE / DMACHK_UNIT);

						// }CNpPbgfBXpb`
						if(++dwDispatchTerm >= GetStreamTunerNum()){
							DispatchStream((const DWORD *)DispatchBuffer.GetData(), DispatchBuffer.GetSize() / 4UL);
							DispatchBuffer.ClearSize();
							dwDispatchTerm = 0UL;
							}
					
						// ]}[J[Zbg
						*pDmaBufSet->pdwEndMark = 0UL;
	
						// DMAobt@ZbgXV
						pDmaBufSet = pDmaBufSet->pNextSet;
					
						break;
						}
					else{
						if(++dwWaitTime >= (DMACHK_TIMEOUT / DMACHK_WAIT)){
							// DMA]^CAEg̕A
							::BON_TRACE(TraceType_Warning, TEXT("DMA]^CAEg\n"));
							goto StartDmaTransfer;
							}
						else{						
							// EFCg
							::Sleep(DMACHK_WAIT);
							}				
						}
					}
				}

			// ]JE^CNg
			if(m_pDevice->IncrementTransferCounter() != STATUS_OK)throw ::BON_EXPECTION(TEXT("PTfoCXDMA]JE^CNgs"));
			}

		// DMA]~
		if(m_pDevice->SetTransferEnable(false) != STATUS_OK)throw ::BON_EXPECTION(TEXT("PTfoCXDMA]~s"));

		// DMAobt@J
		if(m_pDevice->SetBufferInfo(NULL) != STATUS_OK)throw ::BON_EXPECTION(TEXT("PTfoCXDMAobt@Js"));
		}
	catch(const CBonException &ex){
		// G[
		::BON_TRACE(TraceType_Error, ex.GetDescription());
		return;
		}
}

void CPt2Board::DispatchStream(const DWORD *pdwData, const DWORD dwNum)
{
	#pragma pack(1)
	struct MICRO_PACKET
	{
		BYTE abyData[3];	// f[^	 2  1  0 ƂȂ
		BYTE Error : 1;		// G[tO
		BYTE Store : 1;		// TSpPbgJntO
		BYTE Count : 3;		// Xg[JE^
		BYTE Tuner : 3;		// Xg[CfbNX
	};
	#pragma pack()

	const MICRO_PACKET *pPacket = (const MICRO_PACKET *)pdwData;

	// fBXpb`obt@Zbg
	for(DWORD dwIndex = 0UL ; dwIndex < m_TunerList.size() ; dwIndex++){
		m_aDispatchBuff[dwIndex].pDataPtr = m_aDispatchBuff[dwIndex].abyDataBuff;
		}

	// }CNpPbgWJAfBXpb`
	for(DWORD dwPos = 0UL ; dwPos < dwNum ; dwPos++, pPacket++){
		if((pPacket->Tuner >= 1) && (pPacket->Tuner <= 4)){
			DISPATCH_BUFF &DispatchBuff = m_aDispatchBuff[pPacket->Tuner - 1];

			for(DWORD dwData = 0UL ; dwData < 3UL ; dwData++){
				if(!DispatchBuff.dwPacketPos){
					// TSpPbg҂
					if(pPacket->Store && (pPacket->abyData[2 - dwData] == 0x47)){
						// TSpPbg擪
						*(DispatchBuff.pDataPtr++) = pPacket->abyData[2 - dwData];
						DispatchBuff.dwPacketPos = 1UL;
						}
					}
				else if(DispatchBuff.dwPacketPos++ < 188UL){
					// TSpPbgXgA
					*(DispatchBuff.pDataPtr++) = pPacket->abyData[2 - dwData];
					
					if(DispatchBuff.dwPacketPos == 188UL){
						// TSpPbgXgA
						DispatchBuff.dwPacketPos = 0UL;
						}
					}
				}
			}		
		}

	// e`[iɏo
	for(DWORD dwIndex = 0UL ; dwIndex < m_TunerList.size() ; dwIndex++){
		if(m_TunerList[dwIndex]->m_bIsStream){
			m_TunerList[dwIndex]->OnTsStream(m_aDispatchBuff[dwIndex].abyDataBuff, m_aDispatchBuff[dwIndex].pDataPtr - m_aDispatchBuff[dwIndex].abyDataBuff);
			}
		}
}


/////////////////////////////////////////////////////////////////////////////
// PT`[iNX
/////////////////////////////////////////////////////////////////////////////

CPt2Tuner::CPt2Tuner(CPt2Board &Board, const DWORD dwTunerID, const Device::ISDB dwTunerType)
	: m_Board(Board)
	, m_dwTunerID(dwTunerID)
	, m_dwTunerType(dwTunerType)
	, m_bIsOpen(false)
	, m_bIsStream(false)
	, m_bIsLnbOn(false)
	, m_pTsReceiver(NULL)
{
	// Ȃ
}

CPt2Tuner::~CPt2Tuner()
{
	// `[iN[Y
	CloseTuner();
}

void CPt2Tuner::CloseTuner(void)
{
	// LNBdOFF
	SetLnbPower(false);

	// `[iN[Y
	m_Board.CloseTuner(this);
}

const bool CPt2Tuner::StartStream(ITsReceiver *pTsReceiver)
{
	// TSV[oCX^Xo^
	{
		CBlockLock AutoLock(&m_Lock);
		m_pTsReceiver = pTsReceiver;
	}

	// Xg[MX^[g
	return m_Board.StartStream(this);
}

void CPt2Tuner::StopStream(void)
{
	// TSV[oCX^Xo^
	{
		CBlockLock AutoLock(&m_Lock);
		m_pTsReceiver = NULL;
	}

	// Xg[MI
	m_Board.StopStream(this);
}

const DWORD CPt2Tuner::GetTunerID(void)
{
	return m_dwTunerID;
}

const bool CPt2Tuner::SetChannel(const DWORD dwChannel, const DWORD dwOffset)
{
	try{
		// Mgݒ
		if(m_Board.m_pDevice->SetFrequency(m_dwTunerID, m_dwTunerType, dwChannel, dwOffset) != STATUS_OK)throw ::BON_EXPECTION(TEXT("PTMgݒ莸s"));
		}
	catch(const CBonException &){
		// G[
		return false;
		}
	
	return true;
}

const bool CPt2Tuner::SetTsID(const WORD wTsID)
{
	DWORD dwCurTsID;

	try{
		// `[i̎ނ`FbN
		if(m_dwTunerType != Device::ISDB_S)throw ::BON_EXPECTION(TEXT("PTMTS IDݒΉ"));
	
		// MTS IDݒ
		if(m_Board.m_pDevice->SetIdS(m_dwTunerID, wTsID) != STATUS_OK)throw ::BON_EXPECTION(TEXT("PTMTS IDݒ莸s"));
		
		// fEFCg
		const DWORD dwStartTime = ::GetTickCount();
		
		do{
			::Sleep(1UL);

			// TS ID擾
			if(m_Board.m_pDevice->GetIdS(m_dwTunerID, &dwCurTsID) != STATUS_OK)throw ::BON_EXPECTION(TEXT("PTMTS ID擾s"));
			
			// TS IDݒ^CAEg
			if((::GetTickCount() - dwStartTime) >= 500UL){
				// TS ID^CAEg
				throw ::BON_EXPECTION(TEXT("PTMTS IDݒ^CAEg"));
				}
			}
		while((WORD)dwCurTsID != wTsID);
		}
	catch(const CBonException &ex){
		// G[
		::BON_TRACE(TraceType_Error, ex.GetDescription());
		return false;
		}
	
	return true;
}

const bool CPt2Tuner::SetLnbPower(const bool bEnable)
{
	// LNBd
	return m_Board.SetLnbPower(this, bEnable);
}

const float CPt2Tuner::GetSignalLevel(void)
{
	DWORD dwCnr100 = 0UL, dwCurAgc = 0UL, dwMaxAgc = 0UL;
	
	try{
		// CNR擾
		if(m_Board.m_pDevice->GetCnAgc(m_dwTunerID, m_dwTunerType, &dwCnr100, &dwCurAgc, &dwMaxAgc) != STATUS_OK)throw ::BON_EXPECTION(TEXT("PT`[iCNR擾s"));
		}
	catch(const CBonException &){
		// G[
		return 0.0f;
		}
		
	// dBɕϊĕԂ
	return (float)dwCnr100 / 100.0f;
}

void CPt2Tuner::OnTsStream(const BYTE *pData, const DWORD dwSize)
{
	CBlockLock AutoLock(&m_Lock);

	// TSV[oɓ]
	if(m_pTsReceiver){
		m_pTsReceiver->OnTsStream(this, pData, dwSize);
		}
}
