//////////////////////////////////////////////////////////////////////////
// BonSDK EARTHSOFT PT2 tuner library
// Copyright (c) http://2sen.dip.jp/dtv/
//
// This is part of BonSDK.
// BonSDK comes with ABSOLUTELY NO WARRANTY;
// for details see 'GNU GENERAL PUBLIC LICENSE 2.0'.

// Pt2Device.h: PT2foCXNX
//
/////////////////////////////////////////////////////////////////////////////

#pragma once


#include "EARTH_PT.h"
#include "OS_Library.h"
#include <Vector>
#include <Queue>


using namespace EARTH;
using namespace OS;
using namespace PT;
using namespace std;


/////////////////////////////////////////////////////////////////////////////
// PT}l[WNX
/////////////////////////////////////////////////////////////////////////////

class CPt2Board;
class CPt2Tuner;

class CPt2Manager
{
public:
	CPt2Manager(void);
	virtual ~CPt2Manager(void);

	const DWORD GetTotalTunerNum(const Device::ISDB dwTunerType);
	const DWORD GetActiveTunerNum(const Device::ISDB dwTunerType);

	CPt2Tuner * OpenTuner(const Device::ISDB dwTunerType);

protected:
	const bool EnumerateBoards(void);
	const bool ReleaseInstance(void);
	
	Bus *m_pPtBus;
	vector<CPt2Board *> m_Boards;

	Library *m_pPtLibrary;
};


/////////////////////////////////////////////////////////////////////////////
// PT2{[hNX
/////////////////////////////////////////////////////////////////////////////

class CPt2Board
{
friend CPt2Tuner;

public:
	enum{TUNER_1ST = 0UL, TUNER_2ND = 1UL, TUNER_NUM = 2UL};

	enum{
		DMABUF_SIZE = 6UL,									// DMAubN
		DMABUF_NUM  = 1UL,									// DMAobt@
		DMABLK_SIZE = 4096UL * Device::BUFFER_PAGE_COUNT,	// DMAubNTCY
		DMACHK_UNIT = 1UL,									// DMA]`FbN
		DMACHK_WAIT = 5UL,									// DMA]`FbNEFCg
		DMACHK_TIMEOUT = 1000UL								// DMA]^CAEg
		};

	CPt2Board(const DWORD dwBoardID, Device *pDevice, const Bus::DeviceInfo *pDeviceInfo);
	virtual ~CPt2Board(void);

	CPt2Tuner * OpenTuner(const Device::ISDB dwTunerType);
	void CloseTuner(CPt2Tuner *pTuner);

	const DWORD GetActiveTunerNum(const Device::ISDB dwTunerType);

protected:
	const bool StartStream(CPt2Tuner *pTuner);
	void StopStream(CPt2Tuner *pTuner);

	const bool SetLnbPower(CPt2Tuner *pTuner, const bool bEnable);

	const bool IsValidTuner(const CPt2Tuner *pTuner) const;
	const DWORD GetOpenTunerNum(void) const;
	const DWORD GetStreamTunerNum(void) const;
	const DWORD GetLnbOnTunerNum(void) const;

	struct DISPATCH_BUFF
	{
		DWORD dwPacketPos;
		BYTE *pDataPtr;
		BYTE abyDataBuff[DMABLK_SIZE * Device::ISDB_COUNT * TUNER_NUM];
	};
	
	const DWORD m_dwBoardID;
	Device *m_pDevice;
	Bus::DeviceInfo m_DeviceInfo;
	
	vector<CPt2Tuner *> m_TunerList;
	DISPATCH_BUFF m_aDispatchBuff[TUNER_NUM * Device::ISDB_COUNT];

	CSmartMutex m_BoardLock;
	CSmartLock m_TunerLock;

private:
	const bool OpenBoard(void);
	void CloseBoard(void);

	void StreamingThread(CSmartThread<CPt2Board> &thread, const ISignal &bKillSignal, const ULONG_PTR param);
	void DispatchStream(const DWORD *pdwData, const DWORD dwNum);

	CSmartThread<CPt2Board> m_StreamingThread;
};


/////////////////////////////////////////////////////////////////////////////
// PT2`[iNX
/////////////////////////////////////////////////////////////////////////////

class CPt2Tuner
{
friend CPt2Board;

public:
	class ITsReceiver :	public IBonObject
	{
	public:
		virtual void OnTsStream(CPt2Tuner *pTuner, const BYTE *pData, const DWORD dwSize) = 0;
	};

	CPt2Tuner(CPt2Board &Board, const DWORD dwTunerID, const Device::ISDB dwTunerType);
	virtual ~CPt2Tuner();

	void CloseTuner(void);

	const bool StartStream(ITsReceiver *pTsReceiver);
	void StopStream(void);

	const DWORD GetTunerID(void);
	const bool SetChannel(const DWORD dwChannel, const DWORD dwOffset = 0UL);
	const bool SetTsID(const WORD wTsID);
	const bool SetLnbPower(const bool bEnable);
	const float GetSignalLevel(void);

protected:
	void OnTsStream(const BYTE *pData, const DWORD dwSize);

	CPt2Board &m_Board;

	const DWORD m_dwTunerID;
	const Device::ISDB m_dwTunerType;

	bool m_bIsOpen;
	bool m_bIsStream;
	bool m_bIsLnbOn;
	
	ITsReceiver *m_pTsReceiver;
	CSmartLock m_Lock;
};
